#include "eventstester.h"
#include <QDebug>
#include <QTextStream>
#include <QFile>
#include <QFileInfo>
#include <QMimeDatabase>


#include "dumper.h"
#include "mediametadata.h"
#include "mediautils.h"

EventsTester::EventsTester(Telegram &telegram, QObject *parent) :
    QObject(parent), m_telegram(telegram) {
    //updates
    connect(&m_telegram, SIGNAL(updatesTooLong()), this, SLOT(onUpdatesTooLong()));
    connect(&m_telegram, SIGNAL(updateShortMessage(qint32,qint32,QString,qint32,qint32,qint32)), this, SLOT(onUpdateShortMessage(qint32,qint32,QString,qint32,qint32,qint32)));
    connect(&m_telegram, SIGNAL(updateShortChatMessage(qint32,qint32,qint32,QString,qint32,qint32,qint32)), this, SLOT(onUpdateShortChatMessage(qint32,qint32,qint32,QString,qint32,qint32,qint32)));
    connect(&m_telegram, SIGNAL(updateShort(Update,qint32)), this, SLOT(onUpdateShort(Update,qint32)));
    connect(&m_telegram, SIGNAL(updatesCombined(QList<Update>,QList<User>,QList<Chat>,qint32,qint32,qint32)), this, SLOT(onUpdatesCombined(QList<Update>,QList<User>,QList<Chat>,qint32,qint32,qint32)));
    connect(&m_telegram, SIGNAL(updates(QList<Update>,QList<User>,QList<Chat>,qint32,qint32)), this, SLOT(onUpdates(QList<Update>,QList<User>,QList<Chat>,qint32,qint32)));
    //errors
    connect(&m_telegram, SIGNAL(error(qint64,qint32,QString)), this, SLOT(onError(qint64,qint32,QString)));
    connect(&m_telegram, SIGNAL(authSignInError(qint64,qint32,QString)), this, SLOT(onAuthSignInError(qint64,qint32,QString)));
    connect(&m_telegram, SIGNAL(authSignUpError(qint64,qint32,QString)), this, SLOT(onAuthSignUpError(qint64,qint32,QString)));
    //responses
    connect(&m_telegram, SIGNAL(authNeeded()), this, SLOT(onAuthNeeded()));
    connect(&m_telegram, SIGNAL(authCheckPhoneAnswer(qint64,bool,bool)), this, SLOT(onAuthCheckPhoneAnswer(qint64,bool,bool)));
    connect(&m_telegram, SIGNAL(authSendCodeAnswer(qint64,bool,qint32)), this, SLOT(onAuthSendCodeAnswer(qint64,bool,qint32)));
    connect(&m_telegram, SIGNAL(authSendCallAnswer(qint64,bool)), this, SLOT(onAuthSendCallAnswer(qint64,bool)));
    connect(&m_telegram, SIGNAL(authLoggedIn()), this, SLOT(onAuthLoggedIn()));
    connect(&m_telegram, SIGNAL(accountRegisterDeviceAnswer(qint64,bool)), this, SLOT(onAccountRegisterDeviceAnswer(qint64,bool)));
    connect(&m_telegram, SIGNAL(accountUnregisterDeviceAnswer(qint64,bool)), this, SLOT(onAccountUnregisterDeviceAnswer(qint64,bool)));
    connect(&m_telegram, SIGNAL(accountUpdateNotifySettingsAnswer(qint64,bool)), this, SLOT(onAccountUpdateNotifySettingsAnswer(qint64,bool)));
    connect(&m_telegram, SIGNAL(accountGetNotifySettingsAnswer(qint64,PeerNotifySettings)), this, SLOT(onAccountGetNotifySettingsAnswer(qint64,PeerNotifySettings)));
    connect(&m_telegram, SIGNAL(accountResetNotifySettingsAnswer(qint64,bool)), this, SLOT(onAccountResetNotifySettingsAnswer(qint64,bool)));
    connect(&m_telegram, SIGNAL(accountUpdateProfileAnswer(qint64,User)), this, SLOT(onAccountUpdateProfileAnswer(qint64,User)));
    connect(&m_telegram, SIGNAL(accountUpdateStatusAnswer(qint64,bool)), this, SLOT(onAccountUpdateStatusAnswer(qint64,bool)));
    connect(&m_telegram, SIGNAL(accountGetWallPapersAnswer(qint64,QList<WallPaper>)), this, SLOT(onAccountGetWallPapersAnswer(qint64,QList<WallPaper>)));
    connect(&m_telegram, SIGNAL(photosUploadProfilePhotoAnswer(qint64,Photo,QList<User>)), this, SLOT(onPhotosUploadProfilePhotoAnswer(qint64,Photo,QList<User>)));
    connect(&m_telegram, SIGNAL(photosUpdateProfilePhotoAnswer(qint64,UserProfilePhoto)), this, SLOT(onPhotosUpdateProfilePhotoAnswer(qint64,UserProfilePhoto)));
    connect(&m_telegram, SIGNAL(contactsGetStatusesAnswer(qint64,QList<ContactStatus>)), this, SLOT(onContactsGetStatusesAnswer(qint64,QList<ContactStatus>)));
    connect(&m_telegram, SIGNAL(contactsGetContactsAnswer(qint64,bool,QList<Contact>,QList<User>)), this, SLOT(onContactsGetContactsAnswer(qint64,bool,QList<Contact>,QList<User>)));
    connect(&m_telegram, SIGNAL(contactsImportContactsAnswer(qint64,QList<ImportedContact>,QList<qint64>,QList<User>)), this, SLOT(onContactsImportContactsAnswer(qint64,QList<ImportedContact>,QList<qint64>,QList<User>)));
    connect(&m_telegram, SIGNAL(contactsDeleteContactAnswer(qint64,ContactsMyLink,ContactsForeignLink,User)), this, SLOT(onContactsDeleteContactAnswer(qint64,ContactsMyLink,ContactsForeignLink,User)));
    connect(&m_telegram, SIGNAL(contactsDeleteContactsAnswer(qint64,bool)), this, SLOT(onContactsDeleteContactsAnswer(qint64,bool)));
    connect(&m_telegram, SIGNAL(contactsBlockAnswer(qint64,bool)), this, SLOT(onContactsBlockAnswer(qint64,bool)));
    connect(&m_telegram, SIGNAL(contactsUnblockAnswer(qint64,bool)), this, SLOT(onContactsUnblockAnswer(qint64,bool)));
    connect(&m_telegram, SIGNAL(contactsGetBlockedAnswer(qint64,qint32,QList<ContactBlocked>,QList<User>)), this, SLOT(onContactsGetBlockedAnswer(qint64,qint32,QList<ContactBlocked>,QList<User>)));
    connect(&m_telegram, SIGNAL(usersGetUsersAnswer(qint64,QList<User>)), this, SLOT(onUsersGetUsersAnswer(qint64,QList<User>)));
    connect(&m_telegram, SIGNAL(usersGetFullUserAnswer(qint64,User,ContactsLink,Photo,PeerNotifySettings,bool,QString,QString)), this, SLOT(onUsersGetFullUserAnswer(qint64,User,ContactsLink,Photo,PeerNotifySettings,bool,QString,QString)));
    connect(&m_telegram, SIGNAL(photosGetUserPhotosAnswer(qint64,qint32,QList<Photo>,QList<User>)), this, SLOT(onPhotosGetUserPhotosAnswer(qint64,qint32,QList<Photo>,QList<User>)));
    connect(&m_telegram, SIGNAL(messagesSendMessageAnswer(qint64,qint32,qint32,qint32,qint32,QList<ContactsLink>)), this, SLOT(onMessagesSendMessageAnswer(qint64,qint32,qint32,qint32,qint32,QList<ContactsLink>)));
    connect(&m_telegram, SIGNAL(messagesSendPhotoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendPhotoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesSendVideoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendVideoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesSendGeoPointAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendGeoPointAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesSendContactAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendContactAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesSendDocumentAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendDocumentAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesSendAudioAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendAudioAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesSetTypingAnswer(qint64,bool)), this, SLOT(onMessagesSetTypingAnswer(qint64,bool)));
    connect(&m_telegram, SIGNAL(messagesGetMessagesAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)), this, SLOT(onMessagesGetMessagesAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)));
    connect(&m_telegram, SIGNAL(messagesGetDialogsAnswer(qint64,qint32,QList<Dialog>,QList<Message>,QList<Chat>,QList<User>)), this, SLOT(onMessagesGetDialogsAnswer(qint64,qint32,QList<Dialog>,QList<Message>,QList<Chat>,QList<User>)));
    connect(&m_telegram, SIGNAL(messagesGetHistoryAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)), this, SLOT(onMessagesGetHistoryAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)));
    connect(&m_telegram, SIGNAL(messagesSearchAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)), this, SLOT(onMessagesSearchAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)));
    connect(&m_telegram, SIGNAL(messagesReadHistoryAnswer(qint64,qint32,qint32,qint32)), this, SLOT(onMessagesReadHistoryAnswer(qint64,qint32,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesDeleteHistoryAnswer(qint64,qint32,qint32,qint32)), this, SLOT(onMessagesDeleteHistoryAnswer(qint64,qint32,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesDeleteMessagesAnswer(qint64,QList<qint32>)), this, SLOT(onMessagesDeleteMessagesAnswer(qint64,QList<qint32>)));
    connect(&m_telegram, SIGNAL(messagesRestoreMessagesAnswer(qint64,QList<qint32>)), this, SLOT(onMessagesRestoreMessagesAnswer(qint64,QList<qint32>)));
    connect(&m_telegram, SIGNAL(messagesReceivedMessagesAnswer(qint64,QList<qint32>)), this, SLOT(onMessagesReceivedMessagesAnswer(qint64,QList<qint32>)));
    connect(&m_telegram, SIGNAL(messagesForwardMessageAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesForwardMessageAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesForwardMessagesAnswer(qint64,QList<Message>,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesForwardMessagesAnswer(qint64,QList<Message>,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesSendBroadcastAnswer(qint64,QList<Message>,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendBroadcastAnswer(qint64,QList<Message>,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesGetChatsAnswer(qint64,QList<Chat>,QList<User>)), this, SLOT(onMessagesGetChatsAnswer(qint64,QList<Chat>,QList<User>)));
    connect(&m_telegram, SIGNAL(messagesGetFullChatAnswer(qint64,ChatFull,QList<Chat>,QList<User>)), this, SLOT(onMessagesGetFullChatAnswer(qint64,ChatFull,QList<Chat>,QList<User>)));
    connect(&m_telegram, SIGNAL(messagesEditChatTitleAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesEditChatTitleAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesEditChatPhotoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesEditChatPhotoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesAddChatUserAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesAddChatUserAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesDeleteChatUserAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesDeleteChatUserAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(messagesCreateChatAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesCreateChatAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(&m_telegram, SIGNAL(updatesGetStateAnswer(qint64,qint32,qint32,qint32,qint32,qint32)), this, SLOT(onUpdatesGetStateAnswer(qint64,qint32,qint32,qint32,qint32,qint32)));
    connect(&m_telegram, SIGNAL(updatesGetDifferenceAnswer(qint64,qint32,qint32)), this, SLOT(onUpdatesGetDifferenceAnswer(qint64,qint32,qint32)));
    connect(&m_telegram, SIGNAL(updatesGetDifferenceAnswer(qint64,QList<Message>,QList<EncryptedMessage>,QList<Update>,QList<Chat>,QList<User>,UpdatesState,bool)), this, SLOT(onUpdatesGetDifferenceAnswer(qint64,QList<Message>,QList<EncryptedMessage>,QList<Update>,QList<Chat>,QList<User>,UpdatesState,bool)));
    connect(&m_telegram, SIGNAL(uploadGetFileAnswer(qint64,StorageFileType,qint32,QByteArray,qint32,qint32,qint32)), this, SLOT(onUploadGetFileAnswer(qint64,StorageFileType,qint32,QByteArray,qint32,qint32,qint32)));
}

EventsTester::~EventsTester() {
}

void EventsTester::onError(qint64 id, qint32 errorCode, const QString &errorText) {
    qDebug() << "error SIGNAL received, id:" << id;
    qDebug() << "errorCode:" << errorCode;
    qDebug() << "errorText:" << errorText;
    optionSelector();
}

void EventsTester::onUpdatesTooLong() {
    qDebug() << "updatesTooLong SIGNAL received";
}

void EventsTester::onUpdateShortMessage(qint32 id, qint32 fromId, const QString &message, qint32 pts, qint32 date, qint32 seq) {
    qDebug() << "updateShortMessage SIGNAL received";
    qDebug() << "id:" << id;
    qDebug() << "fromId:" << fromId;
    qDebug() << "message:" << message;
    qDebug() << "pts:" << pts;
    qDebug() << "date:" << date;
    qDebug() << "seq:" << seq;
    qDebug() << "";
}

void EventsTester::onUpdateShortChatMessage(qint32 id, qint32 fromId, qint32 chatId, const QString &message, qint32 pts, qint32 date, qint32 seq) {
    qDebug() << "updateShortMessage SIGNAL received";
    qDebug() << "id:" << id;
    qDebug() << "fromId:" << fromId;
    qDebug() << "chatId:" << chatId;
    qDebug() << "message:" << message;
    qDebug() << "pts:" << pts;
    qDebug() << "date:" << date;
    qDebug() << "seq:" << seq;
    qDebug() << "";
}

void EventsTester::onUpdateShort(const Update &update, qint32 date) {
    qDebug() << "updateShort SIGNAL received";
    Dumper::dumpUpdate(update);
    qDebug() << "date:" << Dumper::unixTimeToDate(date);
    qDebug() << "";
}

void EventsTester::onUpdatesCombined(const QList<Update> &updates, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seqStart, qint32 seq) {
    qDebug() << "updatesCombined SIGNAL received";
    qDebug() << "Updates:";
    Q_FOREACH (Update update, updates) {
        Dumper::dumpUpdate(update);
    }
    qDebug() << "Users:";
    Q_FOREACH (User user, users) {
        Dumper::dumpUser(user);
    }
    qDebug() << "Chats:";
    Q_FOREACH (Chat chat, chats) {
        Dumper::dumpChat(chat);
    }
    qDebug() << "date:" << Dumper::unixTimeToDate(date);
    qDebug() << "seqStart:" << seqStart;
    qDebug() << "seq:" << seq;
}

void EventsTester::onUpdates(const QList<Update> &udts, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seq) {
    qDebug() << "updates SIGNAL received";
    qDebug() << "Updates:";
    Q_FOREACH (Update update, udts) {
        Dumper::dumpUpdate(update);
    }
    qDebug() << "Users:";
    Q_FOREACH (User user, users) {
        Dumper::dumpUser(user);
    }
    qDebug() << "Chats:";
    Q_FOREACH (Chat chat, chats) {
        Dumper::dumpChat(chat);
    }
    qDebug() << "date:" << Dumper::unixTimeToDate(date);
    qDebug() << "seq:" << seq;
}

void EventsTester::onAuthNeeded() {
    qDebug() << "authNeeded SIGNAL received";
    m_telegram.authCheckPhone();
}

void EventsTester::onAuthCheckPhoneAnswer(qint64 id, bool phoneRegistered, bool phoneInvited) {
    qDebug() << "authCheckPhone SIGNAL received, id:" << id;
    qDebug() << "phone registered:" << phoneRegistered;
    qDebug() << "phone invited:" << phoneInvited;
    m_telegram.authSendCode();
}

void EventsTester::onAuthSendCodeAnswer(qint64 id, bool phoneRegistered, qint32 sendCallTimeout) {
    qDebug() << "authSendCodeAnswer SIGNAL received, id:" << id;
    qDebug() << "phone registered:" << phoneRegistered;
    qDebug() << "time to wait until phone call:" << sendCallTimeout;

    //test msisdn 799966XYYYY, where X is the DC number and where YYYY are random numbers,
    //verification code will be XXXXX
    // in our case, the phoneNumber is 79996611111, so code will be 11111
    if (Settings::getInstance()->phoneNumber() == "79996611111") {
        if (phoneRegistered) {
            m_telegram.authSignIn("11111");
        } else {
            m_telegram.authSignUp("11111", "Richard", "Nixon");
        }
    } else {
        QTextStream cin(stdin);
        qDebug() << "Enter VERIFICATION CODE or 'call' to receive a phone call if you haven't received the sms";
        QString verificationCode = cin.readLine();
        if (verificationCode == "call") {
            m_telegram.authSendCall();
            return;
        }
        if (phoneRegistered) {
            m_telegram.authSignIn(verificationCode);
        } else {
            qDebug() << "Phone number not registered, enter your information to register:";
            qDebug() << "First Name:";
            m_firstName = cin.readLine();
            qDebug() << "Last Name:";
            m_lastName = cin.readLine();
            m_telegram.authSignUp(verificationCode, m_firstName, m_lastName);
        }
    }
}

void EventsTester::onAuthSendCallAnswer(qint64 id, bool ok) {
    qDebug() << "authSendCallAnswer SIGNAL received, id:" << id;
    qDebug() << "ok" << ok;
}

void EventsTester::onAuthLoggedIn() {
    qDebug() << "authLoggedIn SIGNAL received";
    optionSelector();
}

void EventsTester::onAuthSignInError(qint64 id, qint32 errorCode, const QString &errorText) {
    qDebug() << "authSignInError SIGNAL received, id:" << id << ",code:" << errorCode << ",error:"<< errorText;
    QTextStream cin(stdin);
    qDebug() << "Enter VERIFICATION CODE again";
    QString verificationCode = cin.readLine();
    m_telegram.authSignIn(verificationCode);
}

void EventsTester::onAuthSignUpError(qint64 id, qint32 errorCode, const QString &errorText) {
    qDebug() << "authSignUpError SIGNAL received, id:" << id << ",code:" << errorCode << ",error:"<< errorText;
    QTextStream cin(stdin);
    qDebug() << "Enter VERIFICATION CODE again";
    QString verificationCode = cin.readLine();
    m_telegram.authSignUp(verificationCode, m_firstName, m_lastName);
}



void EventsTester::onAccountRegisterDeviceAnswer(qint64 id, bool ok) {
    qDebug() << "accountRegisterDeviceAnswer SIGNAL received, id:" << id;
    qDebug() << "ok:" << ok;
    optionSelector();
}

void EventsTester::onAccountUnregisterDeviceAnswer(qint64 id, bool ok) {
    qDebug() << "accountUnregisterDeviceAnswer SIGNAL received, id:" << id;
    qDebug() << "ok:" << ok;
    optionSelector();
}

void EventsTester::onAccountUpdateNotifySettingsAnswer(qint64 id, bool ok) {
    qDebug() << "accountUpdateNotifySettingsAnswer SIGNAL received, id:" << id;
    qDebug() << "ok:" << ok;
    optionSelector();
}

void EventsTester::onAccountGetNotifySettingsAnswer(qint64 id, PeerNotifySettings peerNotifySettings) {
    qDebug() << "accountGetNotifySettingsAnswer SIGNAL received, id:" << id;
    Dumper::dumpPeerNotifySettings(peerNotifySettings);
    optionSelector();
}

void EventsTester::onAccountResetNotifySettingsAnswer(qint64 id, bool ok) {
    qDebug() << "accountResetNotifySettingsAnswer SIGNAL received, id:" << id;
    qDebug() << "ok:" << ok;
    optionSelector();
}

void EventsTester::onAccountUpdateProfileAnswer(qint64 id, User user) {
    qDebug() << "accountUpdateProfileAnswer SIGNAL received, id:" << id;
    Dumper::dumpUser(user);
    optionSelector();
}

void EventsTester::onAccountUpdateStatusAnswer(qint64 id, bool ok) {
    qDebug() << "accountUpdateStatusAnswer SIGNAL received, id:" << id;
    qDebug() << "ok:" << ok;
    optionSelector();
}

void EventsTester::onAccountGetWallPapersAnswer(qint64 id, QList<WallPaper> wallPapers) {
    qDebug() << "accountGetWallPapersAnswer SIGNAL received, id:" << id;
    Q_FOREACH (WallPaper wallPaper, wallPapers) {
        Dumper::dumpWallPaper(wallPaper);
        qDebug() << "";
    }
    optionSelector();
}

void EventsTester::onPhotosUploadProfilePhotoAnswer(qint64 id, Photo photo, QList<User> users) {
    qDebug() << "photosUploadProfilePhotoAnswer SIGNAL received, id:" << id;
    Dumper::dumpPhoto(photo);
    qDebug() << "Users mentioned in description of the photo:";
    Q_FOREACH (User user, users) {
        Dumper::dumpUser(user);
        qDebug() << "";
    }
    optionSelector();
}

void EventsTester::onPhotosUpdateProfilePhotoAnswer(qint64 id, UserProfilePhoto userProfilePhoto) {
    qDebug() << "photosUpdateProfilePhotoAnswer SIGNAL received, id:" << id;
    Dumper::dumpUserProfilePhoto(userProfilePhoto);
    optionSelector();
}

void EventsTester::onUsersGetUsersAnswer(qint64 id, const QList<User> &users) {
    qDebug() << "usersGetUsersAnswer SIGNAL received, id:" << id;
    qDebug() << "users retrieved:" << users.size();
    qDebug() << "---";
    Q_FOREACH (User user, users) {
        Dumper::dumpUser(user);
    }
    optionSelector();
}

void EventsTester::onUsersGetFullUserAnswer(qint64 id, const User &user, const ContactsLink &link, const Photo &profilePhoto, const PeerNotifySettings &notifySettings, bool blocked, const QString &realFirstName, const QString &realLastName) {
    qDebug() << "usersGetFullUsersAnswer SIGNAL received, id:" << id;
    Dumper::dumpUser(user);
    Dumper::dumpContactsLink(link);
    Dumper::dumpPhoto(profilePhoto);
    Dumper::dumpPeerNotifySettings(notifySettings);
    qDebug() << "blocked:" << blocked;
    qDebug() << "realFirstName:" << realFirstName;
    qDebug() << "realLastName:" << realLastName;
    optionSelector();
}

void EventsTester::onPhotosGetUserPhotosAnswer(qint64 id, qint32 count, const QList<Photo> &photos, const QList<User> &users) {
    qDebug() << "photosGetUserPhotos SIGNAL received, id:" << id;
    qDebug() << "count:" << count;
    Q_FOREACH (Photo photo, photos) {
        Dumper::dumpPhoto(photo);
    }
    Q_FOREACH (User user, users) {
        Dumper::dumpUser(user);
    }
    optionSelector();
}

void EventsTester::onContactsGetStatusesAnswer(qint64 id, const QList<ContactStatus> &statuses) {
    qDebug() << "contactsGetStatuses SIGNAL received, id:" << id;
    Q_FOREACH (ContactStatus status, statuses) {
        Dumper::dumpContactStatus(status);
        qDebug() << "";
    }
    optionSelector();
}

void EventsTester::onContactsGetContactsAnswer(qint64 id, bool modified, const QList<Contact> &contacts, const QList<User> &users) {
    qDebug() << "contactsGetContactsAnswer SIGNAL received, id:" << id;
    qDebug() << "modified from last requested time:" << modified;
    qDebug() << "contacts:" << contacts.size();
    qDebug() << "users:" << users.size();
    qDebug() << "---";

    qDebug() << endl << "#Contacts:" << endl;
    Q_FOREACH (Contact contact, contacts) {
        Dumper::dumpContact(contact);
        qDebug() <<  "";
    }

    qDebug() << endl << "#Users:" << endl;
    Q_FOREACH (User user, users) {
        Dumper::dumpUser(user, false);
        qDebug() << "";
    }
    optionSelector();
}

void EventsTester::onContactsImportContactsAnswer(qint64 id, const QList<ImportedContact> &imported, const QList<qint64> &retryContacts, const QList<User> &users) {
    qDebug() << "contactsImportContactsAnswer SIGNAL received, id:" << id;
    qDebug() << "#ImportedContacts:";
    Q_FOREACH (ImportedContact imp, imported) {
        Dumper::dumpImportedContact(imp);
        qDebug() << "";
    }
    qDebug() << "#RetryContacts:";
    Q_FOREACH (qint64 retryContact, retryContacts) {
        qDebug() << "retryContact:" << retryContact;
    }
    qDebug() << "#Users:";
    Q_FOREACH (User user, users) {
        Dumper::dumpUser(user);
        qDebug() << "";
    }
    optionSelector();
}

void EventsTester::onContactsDeleteContactAnswer(qint64 id, const ContactsMyLink &myLink, const ContactsForeignLink &foreignLink, const User &user) {
    qDebug() << "contactsDeleteContactAnswer SIGNAL received, id:" << id;
    Dumper::dumpContactsMyLink(myLink);
    Dumper::dumpContactsForeignLink(foreignLink);
    Dumper::dumpUser(user);
    optionSelector();
}

void EventsTester::onContactsDeleteContactsAnswer(qint64 id, bool ok) {
    qDebug() << "contactsDeleteContactsAnswer SIGNAL received, id:" << id;
    qDebug() << "ok:" << ok;
    optionSelector();
}

void EventsTester::onContactsBlockAnswer(qint64 id, bool ok) {
    qDebug() << "contactsBlockAnswer SIGNAL received, id:" << id;
    qDebug() << "ok:" << ok;
    optionSelector();
}

void EventsTester::onContactsUnblockAnswer(qint64 id, bool ok) {
    qDebug() << "contactsUnblockAnswer SIGNAL received, id:" << id;
    qDebug() << "ok:" << ok;
    optionSelector();
}

void EventsTester::onContactsGetBlockedAnswer(qint64 id, qint32 count, const QList<ContactBlocked> &blocked, const QList<User> &users) {
    qDebug() << "contactsGetBlockedAnswer SIGNAL received, id:" << id;
    qDebug() << "count:" << count;
    qDebug() << "#Blocked:";
    Q_FOREACH (ContactBlocked b, blocked) {
        Dumper::dumpContactBlocked(b);
        qDebug() << "";
    }
    qDebug() << "#Users:";
    Q_FOREACH (User user, users) {
        Dumper::dumpUser(user);
        qDebug() << "";
    }
    optionSelector();
}

void EventsTester::onMessagesSendMessageAnswer(qint64 id, qint32 messageId, qint32 date, qint32 pts, qint32 seq, const QList<ContactsLink> &links) {
    qDebug() << "messagesSendMessageAnswer SIGNAL received, id:" << id;
    qDebug() << "messageId:" << messageId;
    qDebug() << "date:" << date;
    qDebug() << "pts:" << pts;
    qDebug() << "seq:" << seq;
    Q_FOREACH (ContactsLink link, links) {
        Dumper::dumpContactsLink(link);
    }
    optionSelector();
}

void EventsTester::onMessagesSendPhotoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesSendPhotoAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessage(message, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onMessagesSendVideoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesSendVideoAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessage(message, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onMessagesSendGeoPointAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesSendGeoPointAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessage(message, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onMessagesSendContactAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesSendContactAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessage(message, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onMessagesSendDocumentAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesSendDocumentAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessage(message, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onMessagesSendAudioAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesSendAudioAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessage(message, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onMessagesSetTypingAnswer(qint64 id, bool ok) {
    qDebug() << "messagesSetTypingAnswer SIGNAL received, id:" << id;
    qDebug() << "ok:" << ok;
    optionSelector();
}

void EventsTester::onMessagesGetMessagesAnswer(qint64 id, qint32 count, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users) {
    qDebug() << "messagesGetMessagesAnswer SIGNAL received, id:" << id;
    qDebug() << "count:" << count;
    qDebug() << "messages:" << messages.size();
    qDebug() << "chats mentioned in messages:" << chats.size();
    qDebug() << "users mentioned in messages and chats:" << users.size();
    qDebug() << "-----";
    QMap<qint32, const User *> mentionedUsers;
    for (qint32 i=0; i<users.size(); i++) {
        mentionedUsers.insert(users.at(i).id(), &(users.at(i)));
    }
    QMap<qint32, const Chat *> mentionedChats;
    for (qint32 i=0; i<chats.size(); i++) {
        mentionedChats.insert(chats.at(i).id(), &(chats.at(i)));
    }

    Q_FOREACH (Message m, messages) {
        const User *u = mentionedUsers.value(m.fromId());
        QString from = u->firstName() + " " + u->lastName();
        QString to;
        if (m.toId().classType() == Peer::typePeerUser) {
            const User *u = mentionedUsers.value(m.toId().userId());
            to = u->firstName() + " " + u->lastName();
        } else {
            const Chat *c = mentionedChats.value(m.toId().chatId());
            to = c->title();
        }
        qDebug() << from << " -> " << to << ":" << m.message();
    }
    optionSelector();
}


void EventsTester::onMessagesGetDialogsAnswer(qint64 id, qint32 count, const QList<Dialog> &dialogs, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users) {
    qDebug() << "messagesGetDialogsAnswer SIGNAL received, id:" << id;
    qDebug() << "count:" << count;
    qDebug() << "dialogs:" << dialogs.size();
    qDebug() << "last messages from dialogs:" << messages.size();
    qDebug() << "chats mentioned in dialogs:" << chats.size();
    qDebug() << "users mentioned in messages and chats:" << users.size();
    qDebug() << "-----";
    QMap<qint32, const User*> mentionedUsers;
    for (qint32 i=0; i<users.size(); i++) {
        mentionedUsers.insert(users.at(i).id(), &(users.at(i)));
    }

    qint32 chatCounter = 0;
    for (qint32 i = 0; i < dialogs.size(); ++i) {
        Dialog thisdialog = dialogs.at(i);
        Message thismessage = messages.at(i);
        Peer peer = thisdialog.peer();
        if (peer.classType() == Peer::typePeerChat) {
            qDebug() << "chatId:" << peer.chatId();
            if (chats.size() > chatCounter) {
                Chat thischat = chats.at(chatCounter++);
                qDebug() <<"Chat Name: " << thischat.title();
            }
        } else if (peer.classType() == Peer::typePeerUser) {
            qDebug() << "userId:" << peer.userId();
            const User *u = mentionedUsers.value(peer.userId());
            qDebug() << "Contact Name: " << u->firstName() + " " + u->lastName();
        }
        qDebug() << "Unread: " << thisdialog.unreadCount();
        if (thismessage.classType() != Message::typeMessageService) {
            qDebug() << "Last message:" << thismessage.message();
            qDebug() << "Date:" << Dumper::unixTimeToDate(thismessage.date());
        }
        if (thismessage.media().classType() != MessageMedia::typeMessageMediaEmpty) {
            qDebug() << "Has media";
        }
        qDebug() << "";
    }
    optionSelector();
}

void EventsTester::onMessagesGetHistoryAnswer(qint64 id, qint32 count, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users) {
    qDebug() << "messagesGetHistoryAnswer SIGNAL received, id:" << id;
    qDebug() << "count:" << count;
    qDebug() << "messages:" << messages.size();
    qDebug() << "chats mentioned in messages:" << chats.size();
    qDebug() << "users mentioned in messages and chats:" << users.size();
    qDebug() << "-----";
    QMap<qint32, const User *> mentionedUsers;
    for (qint32 i=0; i<users.size(); i++) {
        mentionedUsers.insert(users.at(i).id(), &(users.at(i)));
    }
    QMap<qint32, const Chat *> mentionedChats;
    for (qint32 i=0; i<chats.size(); i++) {
        mentionedChats.insert(chats.at(i).id(), &(chats.at(i)));
    }

    for (qint32 i = messages.size() - 1; i >= 0; i--) {
        const Message m = messages.at(i);
        const User *u = mentionedUsers.value(m.fromId());
        QString from = u->firstName() + " " + u->lastName();
        QString to;
        if (m.toId().classType() == Peer::typePeerUser) {
            const User *u = mentionedUsers.value(m.toId().userId());
            to = u->firstName() + " " + u->lastName();
        } else {
            const Chat *c = mentionedChats.value(m.toId().chatId());
            to = c->title();
        }
        QString msg = m.message();
        if (m.media().classType() != MessageMedia::typeMessageMediaEmpty) {
            QString type = "photo";
            qint64 id;
            qint64 accessHash;
            switch (m.media().classType()) {
            case MessageMedia::typeMessageMediaVideo:
                type = "video";
                id = m.media().video().id();
                accessHash = m.media().video().accessHash();
                break;
            case MessageMedia::typeMessageMediaGeo:
                type = "geopoint";
                break;
            case MessageMedia::typeMessageMediaContact:
                type = "contact";
                break;
            case MessageMedia::typeMessageMediaUnsupported:
                type= "unsupported";
                break;
            case MessageMedia::typeMessageMediaDocument:
                type = "document";
                id = m.media().document().id();
                accessHash = m.media().document().accessHash();
                break;
            case MessageMedia::typeMessageMediaAudio:
                type = "audio";
                id = m.media().audio().id();
                accessHash = m.media().audio().accessHash();
                break;
            default:
                type = "photo";
                id = m.media().photo().id();
                accessHash = m.media().photo().accessHash();
                break;
            }

            msg = "MEDIA | type=" + type;
            if (id && accessHash) {
                msg = msg + ", id=" + QString::number(id) + ", access_hash=" + QString::number(accessHash);
            }
            Dumper::dumpMessageMedia(m.media());
        }
        qDebug() << "msgId:" << m.id() << "-" << from << " -> " << to << ":" << msg;
    }
    optionSelector();
}

void EventsTester::onMessagesSearchAnswer(qint64 id, qint32 count, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users) {
    qDebug() << "messagesSearchAnswer SIGNAL received, id:" << id;
    qDebug() << "count:" << count;
    qDebug() << "messages:" << messages.size();
    qDebug() << "chats mentioned in messages:" << chats.size();
    qDebug() << "users mentioned in messages and chats:" << users.size();
    qDebug() << "-----";
    QMap<qint32, const User *> mentionedUsers;
    for (qint32 i=0; i<users.size(); i++) {
        mentionedUsers.insert(users.at(i).id(), &(users.at(i)));
    }
    QMap<qint32, const Chat *> mentionedChats;
    for (qint32 i=0; i<chats.size(); i++) {
        mentionedChats.insert(chats.at(i).id(), &(chats.at(i)));
    }

    Q_FOREACH (Message m, messages) {
        const User *u = mentionedUsers.value(m.fromId());
        QString from = u->firstName() + " " + u->lastName();
        QString to;
        if (m.toId().classType() == Peer::typePeerUser) {
            const User *u = mentionedUsers.value(m.toId().userId());
            to = u->firstName() + " " + u->lastName();
        } else {
            const Chat *c = mentionedChats.value(m.toId().chatId());
            to = c->title();
        }
        qDebug() << "msgId:" << m.id() << "-" << from << " -> " << to << ":" << m.message();
    }
    optionSelector();
}

void EventsTester::onMessagesReadHistoryAnswer(qint64 id, qint32 pts, qint32 seq, qint32 offset) {
    qDebug() << "messagesReadHistoryAnswer SIGNAL received, id:" << id;
    qDebug() << "pts:" << pts;
    qDebug() << "seq:" << seq;
    qDebug() << "offset:" << offset;
    optionSelector();
}

void EventsTester::onMessagesDeleteHistoryAnswer(qint64 id, qint32 pts, qint32 seq, qint32 offset) {
    qDebug() << "messagesDeleteHistoryAnswer SIGNAL received, id:" << id;
    qDebug() << "pts:" << pts;
    qDebug() << "seq:" << seq;
    qDebug() << "offset:" << offset;
    optionSelector();
}

void EventsTester::onMessagesDeleteMessagesAnswer(qint64 id, const QList<qint32> &deletedIds) {
    qDebug() << "messagesDeleteMessagesAnswer SIGNAL received, id:" << id;
    Q_FOREACH (qint32 deletedId, deletedIds) {
        qDebug() << "deletedId:" << deletedId;
    }
    optionSelector();
}

void EventsTester::onMessagesRestoreMessagesAnswer(qint64 id, const QList<qint32> &restoredIds) {
    qDebug() << "messagesRestoreMessagesAnswer SIGNAL received, id:" << id;
    Q_FOREACH (qint32 restoredId, restoredIds) {
        qDebug() << "restoredId:" << restoredId;
    }
    optionSelector();
}

void EventsTester::onMessagesReceivedMessagesAnswer(qint64 id, const QList<qint32> &confirmedIds) {
    qDebug() << "messagesReceivedMessagesAnswer SIGNAL received, id:" << id;
    Q_FOREACH (qint32 confirmedId, confirmedIds) {
        qDebug() << "confirmedId:" << confirmedId;
    }
    optionSelector();
}

void EventsTester::onMessagesForwardMessageAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesForwardMessageAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessage(message, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onMessagesForwardMessagesAnswer(qint64 id, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesForwardMessagesAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessages(messages, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onMessagesSendBroadcastAnswer(qint64 id, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesSendBroadcastAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessages(messages, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onMessagesGetChatsAnswer(qint64 id, const QList<Chat> &chats, const QList<User> &users) {
    qDebug() << "messagesGetChatsAnswer SIGNAL received, id:" << id;
    qDebug() << "number of chats: " << chats.length();
    qDebug() << "number of users mentioned in these chats:" << users.length();
    qDebug() << "---------";
    qDebug() << "Chats:";
    Q_FOREACH (Chat chat, chats) {
        Dumper::dumpChat(chat);
        qDebug() << "";
    }
    qDebug() << "Users:";
    Q_FOREACH (User user, users) {
        Dumper::dumpUser(user);
        qDebug() << "";
    }
    optionSelector();
}

void EventsTester::onMessagesGetFullChatAnswer(qint64 id, const ChatFull &fullChat, const QList<Chat> &chats, const QList<User> &users) {
    qDebug() << "messagesGetFullChatAnswer SIGNAL received, id:" << id;
    qDebug() << "number of chats: " << chats.length();
    qDebug() << "number of users mentioned in these chats:" << users.length();
    qDebug() << "---------";
    Dumper::dumpChatFull(fullChat);
    qDebug() << "";
    qDebug() << "Chats:";
    Q_FOREACH (Chat chat, chats) {
        Dumper::dumpChat(chat);
        qDebug() << "";
    }
    qDebug() << "Users:";
    Q_FOREACH (User user, users) {
        Dumper::dumpUser(user);
        qDebug() << "";
    }
    optionSelector();
}

void EventsTester::onMessagesEditChatTitleAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesEditChatTitleAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessage(message, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onMessagesEditChatPhotoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesEditChatPhotoAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessage(message, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onMessagesAddChatUserAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesAddChatUserAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessage(message, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onMessagesDeleteChatUserAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesDeleteChatUserAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessage(message, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onMessagesCreateChatAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    qDebug() << "messagesCreateChatAnswer SIGNAL received, id:" << id;
    Dumper::dumpStatedMessage(message, chats, users, links, pts, seq);
    optionSelector();
}

void EventsTester::onUpdatesGetStateAnswer(qint64 id, qint32 pts, qint32 qts, qint32 date, qint32 seq, qint32 unreadCount) {
    qDebug() << "updatesGetStateAnswer SIGNAL received, id:" << id;
    qDebug() << "pts:" << pts;
    qDebug() << "qts:" << qts;
    qDebug() << "date:" << date;
    qDebug() << "seq:" << seq;
    qDebug() << "unreadCount:" << unreadCount;
    optionSelector();
}

void EventsTester::onUpdatesGetDifferenceAnswer(qint64 id, qint32 date, qint32 seq) {
    qDebug() << "updatesGetDifferenceAnswer SIGNAL received, id:" << id;
    qDebug() << "date:" << date;
    qDebug() << "seq:" << seq;
    optionSelector();
}

void EventsTester::onUpdatesGetDifferenceAnswer(qint64 id, QList<Message> messages, QList<EncryptedMessage> newEncryptedMessages, QList<Update> otherUpdates, QList<Chat> chats, QList<User> users, UpdatesState state, bool isIntermediateState) {
    qDebug() << "updatesGetDifferenceAnswer SIGNAL received, id:" << id;
    qDebug() << "NewMessages:";
    Q_FOREACH (Message message, messages) {
        Dumper::dumpMessage(message);
        qDebug() << "";
    }
    qDebug() << "NewEncryptedMessages:";
    Q_FOREACH (EncryptedMessage enc, newEncryptedMessages) {
        Dumper::dumpEncryptedMessage(enc);
        qDebug() << "";
    }
    qDebug() << "OtherUpdates:";
    Q_FOREACH (Update update, otherUpdates) {
        Dumper::dumpUpdate(update);
        qDebug() << "";
    }
    qDebug() << "Chats:";
    Q_FOREACH (Chat chat, chats) {
        Dumper::dumpChat(chat);
        qDebug() << "";
    }
    qDebug() << "Users:";
    Q_FOREACH (User user, users) {
        Dumper::dumpUser(user);
        qDebug() << "";
    }
    Dumper::dumpUpdatesState(state);
    qDebug() << "isIntermediateState:" << isIntermediateState;
    optionSelector();
}

void EventsTester::onUploadGetFileAnswer(qint64 id, StorageFileType type, qint32 mtime, QByteArray bytes, qint32 partId, qint32 downloaded, qint32 total) {
    qDebug() << "uploadGetFileAnswer SIGNAL received, id:" << id;
    qDebug() << "modificationTime:" << Dumper::unixTimeToDate(mtime);
    qDebug() << "partId:" << partId;
    qDebug() << "size:" << bytes.length();
    qDebug() << "downloaded:" << downloaded;
    qDebug() << "total:" << total;
    qDebug() << "calculated percentage:" << (downloaded * 100 / total) << "%";
    Dumper::dumpStorageFileType(type);

    if (!m_downloadFile.isOpen()) {
        QTextStream cin(stdin);
        qDebug() << "File received, enter name for it:";
        QString name = cin.readLine();
        qDebug() << "Enter path where store the file:";
        QString path = cin.readLine();
        QString absPath = path + "/" + name;
        m_downloadFile.setFileName(absPath);
        m_downloadFile.open(QIODevice::WriteOnly);
    }

    m_downloadFile.write(bytes);

    if ((type.classType() != StorageFileType::typeStorageFilePartial) ||
            (m_downloadFile.size() >= m_downloadFileExpectedLength)) {
        m_downloadFile.close();
        qDebug() << "file write completed!";
        optionSelector();
    }
}

InputPeer EventsTester::askForInputPeer() {
    bool valid = false;
    InputPeer::InputPeerType type;
    QString sType;
    while (!valid) {
        qDebug() << "Is destination 'contact', 'self' or 'chat'?";
        QTextStream cin(stdin);
        sType = cin.readLine();
        if (sType == "contact") {
            type = InputPeer::typeInputPeerContact;
            valid = true;
        } else if (sType == "chat") {
            type = InputPeer::typeInputPeerChat;
            valid = true;
        } else if (sType == "self") {
            type = InputPeer::typeInputPeerSelf;
            valid = true;
        }
    }
    QString id = "-1";
    if (type != InputPeer::typeInputPeerSelf) {
        qDebug() << "Enter ID of the destination contact or chat";
        QTextStream cin(stdin);
        id = cin.readLine();
    }
    InputPeer peer(type);
    if (type == InputPeer::typeInputPeerContact) {
        peer.setUserId(id.toInt());
    } else if (type == InputPeer::typeInputPeerChat) {
        peer.setChatId(id.toInt());
    }
    return peer;
}

InputGeoChat EventsTester::askForInputGeoChat() {
    qDebug() << "Enter chat id:";
    qint32 chatId = askForInt();
    qDebug() << "Enter access hash:";
    long accessHash = askForLong();
    InputGeoChat geoChat;
    geoChat.setChatId(chatId);
    geoChat.setAccessHash(accessHash);
    return geoChat;
}

InputNotifyPeer EventsTester::askForInputNotifyPeer() {
    InputNotifyPeer inputNotifyPeer(InputNotifyPeer::typeInputNotifyAll);
    bool valid = false;
    QString sType;
    while (!valid) {
        qDebug() << "Is notification settings affected 'peer', 'users', 'chats', 'all' or 'geochat'?";
        QTextStream cin(stdin);
        sType = cin.readLine();
        if (sType == "peer") {
            inputNotifyPeer.setClassType(InputNotifyPeer::typeInputNotifyPeer);
            valid = true;
            qDebug() << "Enter peer:";
            InputPeer peer = askForInputPeer();
            inputNotifyPeer.setPeer(peer);
        } else if (sType == "users") {
            inputNotifyPeer.setClassType(InputNotifyPeer::typeInputNotifyUsers);
            valid = true;
        } else if (sType == "chats") {
            inputNotifyPeer.setClassType(InputNotifyPeer::typeInputNotifyChats);
            valid = true;
        } else if (sType == "all") {
            inputNotifyPeer.setClassType(InputNotifyPeer::typeInputNotifyAll);
            valid = true;
        } else if (sType == "geochat") {
            inputNotifyPeer.setClassType(InputNotifyPeer::typeInputNotifyGeoChatPeer);
            valid = true;
            qDebug() << "Enter geo chat:";
            InputGeoChat geoChat = askForInputGeoChat();
            inputNotifyPeer.setGeoChatPeer(geoChat);
        }
    }
    return inputNotifyPeer;
}

InputUser EventsTester::askForInputUser() {
    bool valid = false;
    InputUser::InputUserType type;
    QString sType;
    while (!valid) {
        qDebug() << "enter type of user: 'contact', 'self' or 'foreign'?";
        QTextStream cin(stdin);
        sType = cin.readLine();
        if (sType == "contact") {
            type = InputUser::typeInputUserContact;
            valid = true;
        } else if (sType == "foreign") {
            type = InputUser::typeInputUserForeign;
            valid = true;
        } else if (sType == "self") {
            type = InputUser::typeInputUserSelf;
            valid = true;
        }
    }
    QString sId;
    QString sAccessHash;
    qint32 id;
    qint64 accessHash;
    InputUser user(type);
    if (type != InputUser::typeInputUserSelf) {
        valid = false;
        QTextStream cin(stdin);
        while (!valid) {
            qDebug() << "Enter user ID";
            sId = cin.readLine();
            id = sId.toInt(&valid);
        }
        user.setUserId(id);
        if (type == InputUser::typeInputUserForeign) {
            valid = false;
            QTextStream cin(stdin);
            while (!valid){
                qDebug() << "Enter access hash:";
                sAccessHash = cin.readLine();
                accessHash = sAccessHash.toLongLong(&valid);
            }
            user.setAccessHash(accessHash);
        }
    }
    return user;
}

InputContact EventsTester::askForInputContact() {
    InputContact ic;
    bool valid = false;
    qDebug() << "Enter data for input contact:";
    while (!valid) {
        qDebug() << "ClientId?:";
        QTextStream cin(stdin);
        QString in = cin.readLine();
        qint32 clientId = in.toInt(&valid);
        if (valid) {
            ic.setClientId(clientId);
        }
    }
    qDebug() << "Phone?:";
    QTextStream cin(stdin);
    ic.setPhone(cin.readLine());
    qDebug() << "FirstName?:";
    ic.setFirstName(cin.readLine());
    qDebug() << "LastName?:";
    ic.setLastName(cin.readLine());
    return ic;
}

qint32 EventsTester::askForInt() {
    qDebug() << "Enter integer value";
    QTextStream cin(stdin);
    bool valid = false;
    qint32 value;
    while (!valid) {
        QString in = cin.readLine();
        value = in.toInt(&valid);
    }
    return value;
}

qint64 EventsTester::askForLong() {
    qDebug() << "Enter long id value";
    QTextStream cin(stdin);
    bool valid = false;
    qint64 value;
    while (!valid) {
        QString in = cin.readLine();
        value = in.toLongLong(&valid);
    }
    return value;
}

InputGeoPoint EventsTester::askForGeoPoint() {
    InputGeoPoint geo(InputGeoPoint::typeInputGeoPoint);
    QTextStream cin(stdin);
    bool valid = false;
    while (!valid) {
        qDebug() << "Enter longitude";
        QString sLong = cin.readLine();
        geo.setLongitude(sLong.toDouble(&valid));
    }
    valid = false;
    while (!valid) {
        qDebug() << "Enter latitude";
        QString sLat = cin.readLine();
        geo.setLat(sLat.toDouble(&valid));
    }
    return geo;
}

InputPeerNotifySettings EventsTester::askForInputPeerNotifySettings() {
    InputPeerNotifySettings settings;
    qDebug() << "Enter mute until:";
    settings.setMuteUntil(askForInt());
    qDebug() << "Enter sound:";
    QTextStream cin(stdin);
    settings.setSound(cin.readLine());
    qDebug() << "Show previews? (y/N)";
    bool showPreviews = false;
    QString sShowPreviews = cin.readLine();
    sShowPreviews == "y" ? showPreviews = true : showPreviews = false;
    settings.setShowPreviews(showPreviews);
    qDebug() << "Enter events mask:";
    settings.setEventsMask(askForInt());
    return settings;
}

QByteArray EventsTester::readFile(const QString &path) {
    QFile f(path);
    QByteArray bytes;
    if (!f.open(QIODevice::ReadOnly)) {
        qWarning() << "Error opening file";
    } else {
        bytes = f.readAll();
    }
    f.close();
    return bytes;
}


template<typename T>
QList<T> EventsTester::askForArray(T (EventsTester::*askForElement)()) {
    QTextStream cin(stdin);
    QList<T> array;
    qDebug() << "enter array elements, one per row. Type 'end' to finish list";
    QString line;
    while (line != "end") {
        T t = (this->*askForElement)();
        array.append(t);
        qDebug() << "element added to array. Type any key to continue or 'end' to finish";
        line = cin.readLine();
    }
    return array;
}

QString EventsTester::getMimeType(const QString& path) {
    QFile file(path);
    QMimeDatabase mimeDatabase;
    return mimeDatabase.mimeTypeForFile(QFileInfo(file)).name();
}

void EventsTester::optionSelector() {
    qDebug() << "Select option:";
    qDebug() << "0.- listen";
    qDebug() << "NOTIFICATIONS/SETTINGS:";
    qDebug() << "100.- register a device to receive PUSH notifications";
    qDebug() << "101.- unregister a device to stop receiving PUSH notifications";
    qDebug() << "102.- update notification settings";
    qDebug() << "103.- get notification settings";
    qDebug() << "104.- reset all notification settings";
    qDebug() << "105.- update user profile";
    qDebug() << "106.- update user status";
    qDebug() << "107.- get wallpapers";
    qDebug() << "108.- upload profile photo";
    qDebug() << "109.- forward photo to profile photo";
    qDebug() << "USERS:";
    qDebug() << "200.- get user info";
    qDebug() << "201.- get full user info";
    qDebug() << "202.- get user photos";
    qDebug() << "CONTACTS:";
    qDebug() << "300.- get contacts statuses";
    qDebug() << "301.- get contacts";
    qDebug() << "302.- add contacts";
    qDebug() << "303.- delete contact";
    qDebug() << "304.- delete contacts";
    qDebug() << "BLACKLIST:";
    qDebug() << "400.- block contact";
    qDebug() << "401.- unblock contact";
    qDebug() << "402.- get blocked contacts";
    qDebug() << "MESSAGES:";
    qDebug() << "501.- send message";
    qDebug() << "502.- send photo file";
    qDebug() << "503.- send video file";
    qDebug() << "504.- send geo point";
    qDebug() << "505.- send contact";
    qDebug() << "506.- send document file";
    qDebug() << "507.- send audio file";
    qDebug() << "508.- forward photo";
    qDebug() << "509.- forward video";
    qDebug() << "510.- forward document";
    qDebug() << "511.- forward audio";
    qDebug() << "512.- send set typing signal";
    qDebug() << "513.- get messages by messageId";
    qDebug() << "514.- get dialogs";
    qDebug() << "515.- get dialog detail";
    qDebug() << "516.- search";
    qDebug() << "517.- mark as read dialog messages";
    qDebug() << "518.- delete dialog";
    qDebug() << "519.- delete messages by messageId";
    qDebug() << "520.- restore messages by messageId";
    qDebug() << "521.- confirm receipt of messages";
    qDebug() << "522.- forward message";
    qDebug() << "523.- forward messages";
    qDebug() << "524.- send broadcast message [TBD]";
    qDebug() << "CHATS:";
    qDebug() << "600.- get chats";
    qDebug() << "601.- get chat full info";
    qDebug() << "602.- edit chat title";
    qDebug() << "603.- edit chat photo (upload)";
    qDebug() << "604.- forward photo to chat photo";
    qDebug() << "605.- add user to a chat";
    qDebug() << "606.- kick user from a chat";
    qDebug() << "607.- create a chat";
    qDebug() << "UPDATES:";
    qDebug() << "900.- get state of updates";
    qDebug() << "901.- get difference between current state and transmitted updates";
    qDebug() << "FILES:";
    qDebug() << "1000.- download a file";
    qDebug() << endl << "enter option...";
    QTextStream cin(stdin);
    QString option = cin.readLine();
    if (option == "0") {
        // listen for updates
    } else if (option == "100") {
        qDebug() << "Enter device token:";
        QString token = cin.readLine();
        m_telegram.accountRegisterDevice(token, true);
    } else if (option == "101") {
        qDebug() << "Enter device token:";
        QString token = cin.readLine();
        m_telegram.accountUnregisterDevice(token);
    } else if (option == "102") {
        InputNotifyPeer peer = askForInputNotifyPeer();
        InputPeerNotifySettings settings = askForInputPeerNotifySettings();
        m_telegram.accountUpdateNotifySettings(peer,settings);
    } else if (option == "103") {
        InputNotifyPeer peer = askForInputNotifyPeer();
        m_telegram.accountGetNotifySettings(peer);
    } else if (option == "104") {
        m_telegram.accountResetNotifySettings();
    } else if (option == "105") {
        qDebug() << "Enter first name:";
        QString firstName = cin.readLine();
        qDebug() << "Enter last name:";
        QString lastName = cin.readLine();
        m_telegram.accountUpdateProfile(firstName, lastName);
    } else if (option == "106") {
        qDebug() << "Offline? (y/N):";
        QString sOffline = cin.readLine();
        bool offline = false;
        sOffline == "y" ? offline = true : offline = false;
        m_telegram.accountUpdateStatus(offline);
    } else if (option == "107") {
        m_telegram.accountGetWallPapers();
    } else if (option == "108") {
        qDebug() << "Enter path for photo file";
        QString path = cin.readLine();
        QByteArray bytes = readFile(path);
        QString fileName = path.mid(path.lastIndexOf("/")+1);
        qDebug() << "Enter caption:";
        QString caption = cin.readLine();
        m_telegram.photosUploadProfilePhoto(bytes, fileName, caption);
    } else if (option == "109") {
        qDebug() << "Enter photo id to set:";
        qint64 id = askForLong();
        qDebug() << "Enter access hash:";
        qint64 hash = askForLong();
        m_telegram.photosUpdateProfilePhoto(id, hash);
    } else if (option == "200") {
        QList<InputUser> users = askForArray(&EventsTester::askForInputUser);
        m_telegram.usersGetUsers(users);
    } else if (option == "201") {
        InputUser inputUser = askForInputUser();
        m_telegram.usersGetFullUser(inputUser);
    } else if (option == "202") {
        InputUser inputUser = askForInputUser();
        m_telegram.photosGetUserPhotos(inputUser);
    } else if (option == "300") {
        m_telegram.contactsGetStatuses();
    } else if (option == "301") {
        m_telegram.contactsGetContacts();
    } else if (option == "302") {
        QList<InputContact> contacts = askForArray(&EventsTester::askForInputContact);
        m_telegram.contactsImportContacts(contacts, false);
    } else if (option == "303") {
        InputUser inputUser = askForInputUser();
        m_telegram.contactsDeleteContact(inputUser);
    } else if (option == "304") {
        QList<InputUser> users = askForArray(&EventsTester::askForInputUser);
        m_telegram.contactsDeleteContacts(users);
    } else if (option == "400") {
        InputUser inputUser = askForInputUser();
        m_telegram.contactsBlock(inputUser);
    } else if (option == "401") {
        InputUser inputUser = askForInputUser();
        m_telegram.contactsUnblock(inputUser);
    } else if (option == "402") {
        m_telegram.contactsGetBlocked();
    } else if (option == "501") {
        InputPeer peer = askForInputPeer();
        qDebug() << "Enter message to send";
        QString message = cin.readLine();
        m_telegram.messagesSendMessage(peer, message);
    } else if (option == "502") {
        InputPeer peer = askForInputPeer();
        qDebug() << "Enter path for photo file";
        if (m_photoFilePath.length() > 0) {
            qDebug() << "press enter for using" << m_photoFilePath;
            QString path = cin.readLine();
            if (path.length() > 0) {
                m_photoFilePath = path;
            }
        } else {
            m_photoFilePath = cin.readLine();
        }
        qDebug() << "Send file by parts? (Y/n)?";
        QString answer = cin.readLine();
        if (answer == "n") {
            QByteArray bytes = readFile(m_photoFilePath);
            QString fileName = m_photoFilePath.mid(m_photoFilePath.lastIndexOf("/")+1);
            m_telegram.messagesSendPhoto(peer, bytes, fileName);
        } else {
            m_telegram.messagesSendPhoto(peer, m_photoFilePath);
        }
    } else if (option == "503") {
        InputPeer peer = askForInputPeer();
        qDebug() << "Enter path for video file";
        if (m_videoFilePath.length() > 0) {
            qDebug() << "press enter for using" << m_videoFilePath;
            QString path = cin.readLine();
            if (path.length() > 0) {
                m_videoFilePath = path;
            }
        } else {
            m_videoFilePath = cin.readLine();
        }
        QString mimeType = getMimeType(m_videoFilePath);
        qDebug() << "readed mime type for video:" << mimeType;
        qDebug() << "Send file by parts? (Y/n)?";
        QString answer = cin.readLine();
        if (answer == "n") {
            QByteArray bytes = readFile(m_videoFilePath);
            QString fileName = m_videoFilePath.mid(m_videoFilePath.lastIndexOf("/")+1);
            qDebug() << "Has it thumb? (y/N)?";
            QString thumb = cin.readLine();
            QByteArray thumbBytes;
            QString thumbName;
            if (thumb == "y") {
                qDebug() << "Enter path for thumbnail file";
                if (m_videoThumbPath.length() > 0) {
                    qDebug() << "press enter for using" << m_videoThumbPath;
                    QString path = cin.readLine();
                    if (path.length() > 0) {
                        m_videoThumbPath = path;
                    }
                } else {
                    m_videoThumbPath = cin.readLine();
                }
            } else {
                // calculate thumb from file
                m_videoThumbPath = MediaUtils::createThumbnail(m_videoFilePath);
                qDebug() << "created thumbnail at" << m_videoThumbPath;
            }
            thumbBytes = readFile(m_videoThumbPath);
            thumbName = m_videoThumbPath.mid(m_videoThumbPath.lastIndexOf("/")+1);
            MediaMetadata metadata = MediaUtils::readMediaMetadata(m_videoFilePath, mimeType);
            qint32 duration = metadata.duration();
            qint32 width = metadata.width();
            qint32 height = metadata.height();
            m_telegram.messagesSendVideo(peer, bytes, fileName, duration, width, height, mimeType, thumbBytes, thumbName);
        } else {
            qDebug() << "Has it thumb? (y/N)?";
            QString thumb = cin.readLine();
            if (thumb == "y") {
                qDebug() << "Enter path for thumbnail file";
                if (m_videoThumbPath.length() > 0) {
                    qDebug() << "press enter for using" << m_videoThumbPath;
                    QString path = cin.readLine();
                    if (path.length() > 0) {
                        m_videoThumbPath = path;
                    }
                } else {
                    m_videoThumbPath = cin.readLine();
                }
            } else {
                // calculate thumb from file
                m_videoThumbPath = MediaUtils::createThumbnail(m_videoFilePath);
                qDebug() << "created thumbnail at" << m_videoThumbPath;
            }
            MediaMetadata metadata = MediaUtils::readMediaMetadata(m_videoFilePath, mimeType);
            qint32 duration = metadata.duration();
            qint32 width = metadata.width();
            qint32 height = metadata.height();
            m_telegram.messagesSendVideo(peer, m_videoFilePath, duration, width, height, m_videoThumbPath);
        }
    } else if (option == "504") {
        InputPeer peer = askForInputPeer();
        InputGeoPoint geo = askForGeoPoint();
        m_telegram.messagesSendGeoPoint(peer, geo);
    } else if (option == "505") {
        InputPeer peer = askForInputPeer();
        qDebug() << "enter contact phone number";
        QString phoneNumber = cin.readLine();
        qDebug() << "enter first name";
        QString firstName = cin.readLine();
        qDebug() << "enter last name";
        QString lastName = cin.readLine();
        m_telegram.messagesSendContact(peer, phoneNumber, firstName, lastName);
    } else if (option == "506") {
        InputPeer peer = askForInputPeer();
        qDebug() << "Enter path for document file";
        if (m_docFilePath.length() > 0) {
            qDebug() << "press enter for using" << m_docFilePath;
            QString path = cin.readLine();
            if (path.length() > 0) {
                m_docFilePath = path;
            }
        } else {
            m_docFilePath = cin.readLine();
        }
        qDebug() << "Send file by parts? (Y/n)?";
        QString answer = cin.readLine();
        if (answer == "n") {
            QString mimeType = getMimeType(m_docFilePath);
            qDebug() << "readed mime type for document:" << mimeType;
            QByteArray bytes = readFile(m_docFilePath);
            QString fileName = m_docFilePath.mid(m_docFilePath.lastIndexOf("/")+1);
            qDebug() << "Has it thumb? (y/N)?";
            QString thumb = cin.readLine();
            QByteArray thumbBytes;
            QString thumbName;
            if (thumb == "y") {
                qDebug() << "Enter path for thumbnail file";
                if (m_docThumbPath.length() > 0) {
                    qDebug() << "press enter for using" << m_docThumbPath;
                    QString path = cin.readLine();
                    if (path.length() > 0) {
                        m_docThumbPath = path;
                    }
                } else {
                    m_docThumbPath = cin.readLine();
                }
                thumbBytes = readFile(m_docThumbPath);
                thumbName = m_docThumbPath.mid(m_docThumbPath.lastIndexOf("/")+1);
            }
            //m_telegram.messagesSendDocument(peer, bytes, fileName, mimeType, thumbBytes);
            m_telegram.messagesSendDocument(peer, bytes, fileName, mimeType, thumbBytes, thumbName);
        } else {
            qDebug() << "Has it thumb? (y/N)?";
            QString thumb = cin.readLine();
            if (thumb == "y") {
                qDebug() << "Enter path for thumbnail file";
                if (m_docThumbPath.length() > 0) {
                    qDebug() << "press enter for using" << m_docThumbPath;
                    QString path = cin.readLine();
                    if (path.length() > 0) {
                        m_docThumbPath = path;
                    }
                } else {
                    m_docThumbPath = cin.readLine();
                }
            }
            m_telegram.messagesSendDocument(peer, m_docFilePath, m_docThumbPath);
        }
    } else if (option == "507") {
        InputPeer peer = askForInputPeer();
        qDebug() << "Enter path for audio file";
        if (m_audioFilePath.length() > 0) {
            qDebug() << "press enter for using" << m_audioFilePath;
            QString path = cin.readLine();
            if (path.length() > 0) {
                m_audioFilePath = path;
            }
        } else {
            m_audioFilePath = cin.readLine();
        }
        QString mimeType = getMimeType(m_audioFilePath);
        MediaMetadata metadata = MediaUtils::readMediaMetadata(m_audioFilePath, mimeType);
        qint32 duration = metadata.duration();
        qDebug() << "Send file by parts? (Y/n)?";
        QString answer = cin.readLine();
        if (answer == "n") {
            qDebug() << "readed mime type for audio:" << mimeType;
            QByteArray bytes = readFile(m_audioFilePath);
            QString fileName = m_audioFilePath.mid(m_audioFilePath.lastIndexOf("/")+1);
            //m_telegram.messagesSendAudio(peer, bytes, fileName, duration, mimeType);
            m_telegram.messagesSendAudio(peer, bytes, fileName, duration, mimeType);
        } else {
            m_telegram.messagesSendAudio(peer, m_audioFilePath, duration);
        }
    } else if (option == "508") {
        InputPeer peer = askForInputPeer();
        qDebug() << "Enter photo id to forward:";
        qint64 id = askForLong();
        qDebug() << "Enter access hash:";
        qint64 hash = askForLong();
        m_telegram.messagesForwardPhoto(peer, id, hash);
    } else if (option == "509") {
        InputPeer peer = askForInputPeer();
        qDebug() << "Enter video id to forward:";
        qint64 id = askForLong();
        qDebug() << "Enter access hash:";
        qint64 hash = askForLong();
        m_telegram.messagesForwardVideo(peer, id, hash);
    } else if (option == "510") {
        InputPeer peer = askForInputPeer();
        qDebug() << "Enter document id to forward:";
        qint64 id = askForLong();
        qDebug() << "Enter access hash:";
        qint64 hash = askForLong();
        m_telegram.messagesForwardDocument(peer, id, hash);
    } else if (option == "511") {
        InputPeer peer = askForInputPeer();
        qDebug() << "Enter audio id to forward:";
        qint64 id = askForLong();
        qDebug() << "Enter access hash:";
        qint64 hash = askForLong();
        m_telegram.messagesForwardAudio(peer, id, hash);
    } else if (option == "512") {
        InputPeer peer = askForInputPeer();
        m_telegram.messagesSetTyping(peer, true);
    } else if (option == "513") {
        QList<qint32> ids = askForArray(&EventsTester::askForInt);
        m_telegram.messagesGetMessages(ids);
    } else if (option == "514") {
        m_telegram.messagesGetDialogs();
    } else if (option == "515") {
        InputPeer peer = askForInputPeer();
        m_telegram.messagesGetHistory(peer);
    } else if (option == "516") {
        qDebug() << "First of all, the peer where search";
        InputPeer peer = askForInputPeer();
        qDebug() << "Enter query string to search";
        QTextStream in(stdin);
        QString query = in.readLine();
        bool valid = false;
        MessagesFilter f(MessagesFilter::typeInputMessagesFilterEmpty);
        while (!valid) {
            qDebug() << "Enter filter (return for no filter): 'photos', 'video', 'photo and video', 'document'";
            QString filter = in.readLine();
            if (filter.length() > 0) {
                if (filter == "photos") {
                    f.setClassType(MessagesFilter::typeInputMessagesFilterPhotos); valid = true;
                } else if (filter == "video") {
                    f.setClassType(MessagesFilter::typeInputMessagesFilterVideo); valid = true;
                } else if (filter == "photo and video") {
                    f.setClassType(MessagesFilter::typeInputMessagesFilterPhotoVideo); valid = true;
                } else if (filter == "document") {
                    f.setClassType(MessagesFilter::typeInputMessagesFilterDocument); valid = true;
                }
            } else {
                valid = true;
            }
        }
        m_telegram.messagesSearch(peer, query, f);
    } else if (option == "517") {
        InputPeer peer = askForInputPeer();
        m_telegram.messagesReadHistory(peer);
    } else if (option == "518") {
        InputPeer peer = askForInputPeer();
        m_telegram.messagesDeleteHistory(peer);
    } else if (option == "519") {
        QList<qint32> ids = askForArray(&EventsTester::askForInt);
        m_telegram.messagesDeleteMessages(ids);
    } else if (option == "520") {
        QList<qint32> ids = askForArray(&EventsTester::askForInt);
        m_telegram.messagesRestoreMessages(ids);
    } else if (option == "521") {
        qDebug() << "Enter the higher identifier of the messages to confirm";
        qint32 maxId = askForInt();
        m_telegram.messagesReceivedMessages(maxId);
    } else if (option == "522") {
        InputPeer peer = askForInputPeer();
        qDebug() << "Enter id of the message to forward";
        qint32 id = askForInt();
        m_telegram.messagesForwardMessage(peer, id);
    } else if (option == "523") {
        InputPeer peer = askForInputPeer();
        qDebug() << "Enter ids of the message to forward";
        QList<qint32> ids = askForArray(&EventsTester::askForInt);
        m_telegram.messagesForwardMessages(peer, ids);
    } else if (option == "524") {
        QList<InputUser> users = askForArray(&EventsTester::askForInputUser);
        qDebug() << "Enter message to broadcast:";
        QTextStream in(stdin);
        QString message = in.readLine();
        //XXX TODO COMPLETE ASKING FOR MEDIA (ask for path and media type, for instance)
    } else if (option == "600") {
        qDebug() << "Enter the ids of the chats to retrieve";
        QList<qint32> chatIds = askForArray(&EventsTester::askForInt);
        m_telegram.messagesGetChats(chatIds);
    } else if (option == "601") {
        qDebug() << "Enter chatId";
        qint32 chatId = askForInt();
        m_telegram.messagesGetFullChat(chatId);
    } else if (option == "602") {
        qDebug() << "Enter chatId";
        qint32 chatId = askForInt();
        qDebug() << "Enter new title:";
        QString title = cin.readLine();
        m_telegram.messagesEditChatTitle(chatId, title);
    } else if (option == "603") {
        qDebug() << "Enter chatId";
        qint32 chatId = askForInt();
        qDebug() << "Enter path for photo file";
        QString path = cin.readLine();
        QByteArray bytes = readFile(path);
        QString fileName = path.mid(path.lastIndexOf("/")+1);
        m_telegram.messagesEditChatPhoto(chatId, bytes, fileName);
    } else if (option == "604") {
        qDebug() << "Enter chatId";
        qint32 chatId = askForInt();
        qDebug() << "Enter photo id to forward:";
        qint64 id = askForLong();
        qDebug() << "Enter access hash:";
        qint64 hash = askForLong();
        m_telegram.messagesEditChatPhoto(chatId, id, hash);
    } else if (option == "605") {
        qDebug() << "Enter chatId";
        qint32 chatId = askForInt();
        InputUser user = askForInputUser();
        m_telegram.messagesAddChatUser(chatId, user, 0);
    } else if (option == "606") {
        qDebug() << "Enter chatId";
        qint32 chatId = askForInt();
        InputUser user = askForInputUser();
        m_telegram.messagesDeleteChatUser(chatId, user);
    } else if (option == "607") {
        qDebug() << "Enter users included in new chat:";
        QList<InputUser> users = askForArray(&EventsTester::askForInputUser);
        qDebug() << "Enter chat title";
        QString title = cin.readLine();
        m_telegram.messagesCreateChat(users, title);
    } else if (option == "900") {
        m_telegram.updatesGetState();
    } else if (option == "901") {
        qDebug() << "Enter pts";
        qint32 pts = askForInt();
        qDebug() << "Enter date";
        qint32 date = askForInt();
        qDebug() << "Enter qts";
        qint32 qts = askForInt();
        m_telegram.updatesGetDifference(pts, date, qts);
    } else if (option == "1000") {
        qDebug() << "What type of file are you going to download? 'video', 'audio', 'document', 'other' (default)";
        QString type = cin.readLine();
        qDebug() << "Enter the size in bytes:";
        m_downloadFileExpectedLength = askForLong();
        qDebug() << "Enter the DC number where the file is (Enter if you don't know):";
        QString dc = cin.readLine();
        qint32 dcNum = 0;
        if (dc != "") dcNum = dc.toInt();
        InputFileLocation location(InputFileLocation::typeInputFileLocation);
        if (type == "video") {
            location.setClassType(InputFileLocation::typeInputVideoFileLocation);
        } else if (type == "audio") {
            location.setClassType(InputFileLocation::typeInputAudioFileLocation);
        } else if (type == "document") {
            location.setClassType(InputFileLocation::typeInputDocumentFileLocation);
        }
        if (location.classType() == InputFileLocation::typeInputFileLocation) {
            qDebug() << "Enter volumeId";
            location.setVolumeId(askForLong());
            qDebug() << "Enter localId";
            location.setLocalId(askForInt());
            qDebug() << "Enter secret";
            location.setSecret(askForLong());
        } else {
            qDebug() << "Enter id:";
            location.setId(askForLong());
            qDebug() << "Enter access hash:";
            location.setAccessHash(askForLong());
        }
        m_telegram.uploadGetFile(location, m_downloadFileExpectedLength, dcNum);
    } else {
        optionSelector();
    }
}
