#ifndef EVENTSTESTER_H
#define EVENTSTESTER_H

#include <QObject>

#include "telegram.h"

#include "types/inputpeer.h"
#include "types/dialog.h"
#include "types/message.h"
#include "types/chat.h"
#include "types/user.h"
#include "types/inputuser.h"
#include "types/inputcontact.h"

class EventsTester : public QObject
{
    Q_OBJECT
public:
    explicit EventsTester(Telegram &telegram, QObject *parent = 0);
    ~EventsTester();

private:
    Telegram &m_telegram;
    QString m_firstName;
    QString m_lastName;

    QString m_videoFilePath;
    QString m_videoThumbPath;
    QString m_audioFilePath;
    QString m_photoFilePath;
    QString m_docFilePath;
    QString m_docThumbPath;
    QFile m_downloadFile;
    qint64 m_downloadFileExpectedLength;

    InputPeer askForInputPeer();
    InputContact askForInputContact();
    InputUser askForInputUser();
    qint32 askForInt();
    qint64 askForLong();
    InputGeoPoint askForGeoPoint();
    InputNotifyPeer askForInputNotifyPeer();
    InputGeoChat askForInputGeoChat();
    InputPeerNotifySettings askForInputPeerNotifySettings();
    QByteArray readFile(const QString &path);
    template<typename T>
    QList<T> askForArray(T (EventsTester::*)());
    QString getMimeType(const QString& path);
    void optionSelector();

private Q_SLOTS:
    void onError(qint64 id, qint32 errorCode, const QString &errorText);
    // Registration / authorization
    void onAuthNeeded();
    void onAuthCheckPhoneAnswer(qint64 id, bool phoneRegistered, bool phoneInvited);
    void onAuthSendCodeAnswer(qint64 id, bool phoneRegistered, qint32 sendCallTimeout);
    void onAuthSendCallAnswer(qint64 id, bool ok);
    void onAuthLoggedIn();
    void onAuthSignInError(qint64 id, qint32 errorCode, const QString &errorText);
    void onAuthSignUpError(qint64 id, qint32 errorCode, const QString &errorText);
    // Notifications / settings
    void onAccountRegisterDeviceAnswer(qint64 id, bool ok); //result
    void onAccountUnregisterDeviceAnswer(qint64 id, bool ok); //result
    void onAccountUpdateNotifySettingsAnswer(qint64 id, bool ok); //result
    void onAccountGetNotifySettingsAnswer(qint64 id, PeerNotifySettings peerNotifySettings);
    void onAccountResetNotifySettingsAnswer(qint64 id, bool ok); //result
    void onAccountUpdateProfileAnswer(qint64 id, User user); //accountUpdatedProfile
    void onAccountUpdateStatusAnswer(qint64 id, bool ok); //result
    void onAccountGetWallPapersAnswer(qint64 id, QList<WallPaper> wallPapers);
    void onPhotosUploadProfilePhotoAnswer(qint64 id, Photo photo, QList<User> users);
    void onPhotosUpdateProfilePhotoAnswer(qint64 id, UserProfilePhoto userProfilePhoto);
    // Users
    void onUsersGetUsersAnswer(qint64 id, const QList<User> &users);
    void onUsersGetFullUserAnswer(qint64 id, const User &user, const ContactsLink &link, const Photo &profilePhoto, const PeerNotifySettings &notifySettings, bool blocked, const QString &realFirstName, const QString &realLastName);
    void onPhotosGetUserPhotosAnswer(qint64 id, qint32 count, const QList<Photo> &photo, const QList<User> &users);
    // Contacts
    void onContactsGetStatusesAnswer(qint64 id, const QList<ContactStatus> &statuses);
    void onContactsGetContactsAnswer(qint64 id, bool modified, const QList<Contact> &contacts, const QList<User> &users);
    void onContactsImportContactsAnswer(qint64 id, const QList<ImportedContact> &imported, const QList<qint64> &retryContacts, const QList<User> &users);
    void onContactsDeleteContactAnswer(qint64 id, const ContactsMyLink &myLink, const ContactsForeignLink &foreignLink, const User &user);
    void onContactsDeleteContactsAnswer(qint64 id, bool ok);
    // Blacklist
    void onContactsBlockAnswer(qint64 id, bool ok);
    void onContactsUnblockAnswer(qint64 id, bool ok);
    void onContactsGetBlockedAnswer(qint64 id, qint32 count, const QList<ContactBlocked> &blocked, const QList<User> &users);
    // Messages
    void onMessagesSendMessageAnswer(qint64 id, qint32 messageId, qint32 date, qint32 pts, qint32 seq, const QList<ContactsLink> &links);
    void onMessagesSendPhotoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendVideoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendGeoPointAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendContactAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendDocumentAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendAudioAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSetTypingAnswer(qint64 id, bool ok);
    void onMessagesGetMessagesAnswer(qint64 id, qint32 count, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesGetDialogsAnswer(qint64 id, qint32 count, const QList<Dialog> &dialogs, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesGetHistoryAnswer(qint64 id, qint32 count, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesSearchAnswer(qint64 id, qint32 count, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesReadHistoryAnswer(qint64 id, qint32 pts, qint32 seq, qint32 offset);
    void onMessagesDeleteHistoryAnswer(qint64 id, qint32 pts, qint32 seq, qint32 offset);
    void onMessagesDeleteMessagesAnswer(qint64 id, const QList<qint32> &deletedIds);
    void onMessagesRestoreMessagesAnswer(qint64 id, const QList<qint32> &restoredIds);
    void onMessagesReceivedMessagesAnswer(qint64 id, const QList<qint32> &confirmedIds);
    void onMessagesForwardMessageAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesForwardMessagesAnswer(qint64 id, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendBroadcastAnswer(qint64 id, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    // Chats
    void onMessagesGetChatsAnswer(qint64 id, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesGetFullChatAnswer(qint64 id, const ChatFull &fullChat, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesEditChatTitleAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesEditChatPhotoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesAddChatUserAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesDeleteChatUserAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesCreateChatAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    // Updates
    void onUpdatesGetStateAnswer(qint64 id, qint32 pts, qint32 qts, qint32 date, qint32 seq, qint32 unreadCount);
    void onUpdatesGetDifferenceAnswer(qint64 id, qint32 date, qint32 seq);
    void onUpdatesGetDifferenceAnswer(qint64 id, QList<Message> messages, QList<EncryptedMessage> newEncryptedMessages, QList<Update> otherUpdates, QList<Chat> chats, QList<User> users, UpdatesState state, bool isIntermediateState);
    // Files
    void onUploadGetFileAnswer(qint64 id, StorageFileType type, qint32 mtime, QByteArray bytes, qint32 partId, qint32 downloaded, qint32 total);

    void onUpdatesTooLong();
    void onUpdateShortMessage(qint32 id, qint32 fromId, const QString &message, qint32 pts, qint32 date, qint32 seq);
    void onUpdateShortChatMessage(qint32 id, qint32 fromId, qint32 chatId, const QString &message, qint32 pts, qint32 date, qint32 seq);
    void onUpdateShort(const Update &update, qint32 date);
    void onUpdatesCombined(const QList<Update> &updates, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seqStart, qint32 seq);
    void onUpdates(const QList<Update> &udts, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seq);
};

#endif // EVENTSTESTER_H
