#include "mediautils.h"

#include <QtMultimedia/QMediaPlayer>
#include <QtMultimedia/QMediaResource>
#include <QtMultimedia/QMediaContent>
#include <QEventLoop>
#include <QSize>
#include <QProcess>

MediaMetadata MediaUtils::readMediaMetadata(const QString &resource, const QString &type) {
    QMediaPlayer p;
    QMediaResource mediaResource(QUrl("file://" + resource), type);
    QMediaContent media(mediaResource);
    p.setVolume(0);
    p.setMedia(media);
    p.play();
    QEventLoop loop;
    connect(&p, SIGNAL(durationChanged(qint64)), &loop, SLOT(quit()));
    loop.exec();
    // once durationChanged signal, duration and metadata are available
    QVariant variant = p.metaData("Resolution");
    QSize resolutionSize = qvariant_cast<QSize>(variant);
    // p.duration() returns number of milliseconds but seconds is needed in telegram api, so divide by 1000
    MediaMetadata metadata(resolutionSize.width(), resolutionSize.height(), p.duration() / 1000);
    return metadata;
}

QString MediaUtils::createThumbnail(const QString &resource) {
    QProcess ffmpeg;
    QString command = "ffmpeg";
    QStringList params;
    qint32 dotIndex = resource.indexOf(".");
    QString thumb = resource.mid(0, dotIndex) + "_thumb.jpg";
    params << "-i" << resource << "-vframes" << "1" << thumb;
    ffmpeg.start(command,params);
    if (!ffmpeg.waitForFinished()) {
        qWarning() << "thumbnail creation failed";
        thumb = "";
    }
    return thumb;
}
