/*
    This file is part of libqtelegram library.

    Telegram-client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Telegram-client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this telegram-client.  If not, see <http://www.gnu.org/licenses/>.

    Copyright Roberto Mier & Tiago Herrmann & Vitaly Valtman 2014
*/
#ifndef CONNECTION_H
#define CONNECTION_H

#include <QtNetwork/QTcpSocket>
#include "endpoint.h"

class Connection : public QTcpSocket, public Endpoint
{
    Q_OBJECT
public:
    explicit Connection(QObject *parent = 0);
    explicit Connection(const QString &host, qint32 port, QObject *parent = 0);
    ~Connection();

    void connectToServer();

protected:
    qint64 writeOut(const void *data, qint64 length);
    QByteArray readAll();
    QByteArray readIn(qint32 len);
    qint32 readIn(void *data, qint32 len);
    qint32 peekIn(void *data, qint32 len);
    virtual void processRpcAnswer(qint32 op, qint32 len) = 0;
    virtual void processConnected() = 0;

protected Q_SLOTS:
    void onConnected();
    void onReadyRead();
};

#endif // CONNECTION_H
