/*
    This file is part of libqtelegram library.

    Telegram-client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Telegram-client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this telegram-client.  If not, see <http://www.gnu.org/licenses/>.

    Copyright Roberto Mier & Tiago Herrmann & Vitaly Valtman 2014
*/
#ifndef CRYPTOUTILS_H
#define CRYPTOUTILS_H

#include <QObject>
#include "outboundpkt.h"

#include <openssl/bn.h>
#include <openssl/aes.h>

class CryptoUtils : public QObject
{
    Q_OBJECT
public:
    static CryptoUtils *getInstance();

    qint32 encryptPacketBuffer(OutboundPkt &p, void *encryptBuffer);
    qint32 encryptPacketBufferAESUnAuth(const char serverNonce[16], const char hiddenClientNonce[32], OutboundPkt &p, void *encryptBuffer);
    qint32 padRsaEncrypt(char *from, qint32 from_len, char *to, qint32 size, BIGNUM *N, BIGNUM *E);

    void initAESAuth (char authKey[192], char msgKey[16], qint32 encrypt);
    void initAESUnAuth(const char serverNonce[16], const char hiddenClientNonce[32], qint32 encrypt);
    qint32 padAESDecrypt(const char *from, qint32 fromLen, char *to, qint32 size);
    qint32 padAESEncrypt(const char *from, qint32 fromLen, char *to, qint32 size);

    qint32 checkPrime(BIGNUM *p);
    qint32 checkDHParams(BIGNUM *p, qint32 g);

    qint32 BNModExp(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m);

private:
    CryptoUtils();
    ~CryptoUtils();
    CryptoUtils(const CryptoUtils &);
    CryptoUtils &operator=(const CryptoUtils &);

    static CryptoUtils *m_instance;

    BN_CTX *BN_ctx;
    uchar aes_key_raw[32], aes_iv[32];
    AES_KEY aes_key;
};

#endif // CRYPTOUTILS_H
