/*
    This file is part of libqtelegram library.

    Telegram-client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Telegram-client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this telegram-client.  If not, see <http://www.gnu.org/licenses/>.

    Copyright Roberto Mier & Tiago Herrmann & Vitaly Valtman 2014
*/
#ifndef DCPROVIDER_H
#define DCPROVIDER_H

#include <QObject>

#include "dc.h"
#include "dcauth.h"
#include "api.h"

class DcProvider : public QObject
{
    Q_OBJECT
public:
    DcProvider();
    ~DcProvider();

    void initialize();
    DC *getDc(qint32 dcNum) const;
    DC *getWorkingDc() const;
    QList<DC *> getDcs() const;
    Api *getApi() const;
    void transferAuth();

Q_SIGNALS:
    void dcProviderReady();
    void authNeeded();
    void authTransferCompleted();

private:
    void processDcReady(DC *dc);

    QMap<qint32, DC *> m_dcs;
    QMap<qint32, DCAuth *> m_dcAuths;
    //api instance for "internal" operations (config, getNearestDc, etc...). This object could be received
    // from outside, as parameter, when completed external public layer
    Api *m_api;

    // counter of the dc's pending to be authenticated. When this counter is zero, all available DCs are
    // authenticated and a signal dcProviderReady() is emitted
    qint32 m_pendingDcs;

    // this list is filled with a session to every dc receiving auth transfer in transferAuth() operation.
    // Working session exports auth bytes and then is taken the first session in this list for the import.
    // When the list is empty, all the
    // an element is deleted until leave it empty. When it happens, all dcs have the auth transferred.
    QList<Session *> m_transferSessions;
    qint32 m_pendingTransferSessions;

    // main session. The one linked to working dc. Must be stored mean transferring auth to other dcs because
    // is the reference for exporting data to any dc transfer auth receipt
    Session *m_workingDcSession;


private Q_SLOTS:
    void onDcReady(DC *dc);
    void onDcAuthDisconnected();
    void onApiReady(DC*);
    void onConfigReceived(qint64 msgId, qint32 date, bool testMode, qint32 thisDc, const QList<DcOption> &dcOptions, qint32 chatMaxSize, qint32 broadcastMaxSize);
    void onTransferSessionReady(DC*);
    void onAuthExportedAuthorization(qint64, qint32 ourId, const QByteArray &bytes);
    void onAuthImportedAuthorization(qint64, qint32 expires, const User &);
};

#endif // DCPROVIDER_H
