#include "sessionmanager.h"

SessionManager::SessionManager(Session *session, QObject *parent) :
    QObject(parent), m_mainSession(session) {
}

SessionManager::~SessionManager() {
    if (m_mainSession) {
        if (m_mainSession->state() == QAbstractSocket::ConnectedState) {
            m_mainSession->disconnectFromHost();
        }
        m_mainSession->deleteLater();
    }
    Q_FOREACH (Session *session, m_sessions) {
        if (session->state() == QAbstractSocket::ConnectedState) {
            session->disconnectFromHost();
        }
        session->deleteLater();
    }
}

Session *SessionManager::createSession(DC *dc) {
    Session *session = new Session(dc, this);
    connectResponsesSignals(session);
    m_sessions.insert(session->sessionId(), session);
    return session;
}

Session *SessionManager::mainSession() {
    return m_mainSession;
}

void SessionManager::setMainSession(Session *session) {
    m_mainSession = session;
}

void SessionManager::changeMainSessionToDc(DC *dc) {
    // disconnect main session if connected
    if (m_mainSession->state() == QAbstractSocket::ConnectedState) {
        Session *tmp = m_mainSession;
        tmp->disconnectFromHost();
        tmp->deleteLater();
    }
    // create session to new dc and connect to dc
    m_mainSession = createSession(dc);
    connect(m_mainSession, SIGNAL(sessionReady(DC*)), this, SIGNAL(mainSessionDcChanged(DC*)));
    connectUpdatesSignals(m_mainSession);
    m_mainSession->connectToServer();
}
