/*
    This file is part of libqtelegram library.

    Telegram-client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Telegram-client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this telegram-client.  If not, see <http://www.gnu.org/licenses/>.

    Copyright Roberto Mier & Tiago Herrmann & Vitaly Valtman 2014
*/
#ifndef SETTINGS_H
#define SETTINGS_H

#define PROG_NAME "libqtelegram"
#define CONFIG_DIRECTORY "." PROG_NAME
#define CONFIG_FILE "config"
#define AUTH_KEY_FILE "auth"
#define STATE_FILE "state"
#define SECRET_CHAT_FILE "secret"
#define DOWNLOADS_DIRECTORY "downloads"
#define DEFAULT_CONFIG_CONTENTS     \
  "# This is an empty config file\n" \
  "# Feel free to put something here\n"
#define ST_DC_NUM "dcNum"
#define ST_AUTH_FILE "authFile"
#define ST_SECRET "secret"
#define ST_DOWNLOADS "downloads"
#define ST_STATE_FILE "stateFile"
#define ST_TEST_MODE "testMode"
#define ST_MANAGED_DOWNLOADS "managedDownloads"
#define ST_LANG_CODE "lang"

#define ST_WORKING_DC_NUM "workingDcNum"
#define ST_DC_STATE "state"
#define ST_OUR_ID "ourId"
#define ST_DCS_ARRAY "dcs"
#define ST_HOST "host"
#define ST_PORT "port"
#define ST_AUTH_KEY_ID "authKeyId"
#define ST_AUTH_KEY "authKey"
#define ST_SERVER_SALT "serverSalt"
#define ST_EXPIRES "expires"
#define ST_PRODUCTION "production"
#define ST_TEST "test"

#define DEFAULT_CONFIG_PATH "~/.libqtelegram"
#define DEFAULT_PUBLIC_KEY_FILE "qtelegram.pub"

#include <QObject>
#include <QList>
#include <openssl/rsa.h>

class DC;

class Settings : public QObject
{
    Q_OBJECT
public:
    static Settings *getInstance();
    bool loadSettings(const QString &phoneNumber, const QString &configPath = DEFAULT_CONFIG_PATH, const QString &publicKeyFile = DEFAULT_PUBLIC_KEY_FILE);
    void saveSettings();

    qint64 pkFingerprint() const { return m_pkFingerprint; }
    RSA* pubKey() { return m_pubKey; }
    qint32 workingDcNum() const { return m_workingDcNum; }
    qint32 ourId() const { return m_ourId; }
    QList<DC *> &dcsList() { return m_dcsList; }
    void setWorkingDcNum(qint32 workingDcNum) { m_workingDcNum = workingDcNum; }
    void setOurId(qint32 ourId) { m_ourId = ourId; }
    void setDcsList(const QList<DC *> &dcsList) { m_dcsList = dcsList; }
    QString phoneNumber() const { return m_phoneNumber; }
    bool testMode() const { return m_testMode; }
    bool managedDownloads() const { return m_managedDownloads; }
    QString langCode() const { return m_langCode; }

private:
    Settings();
    ~Settings();
    Settings(const Settings &); // hide copy constructor
    Settings& operator=(const Settings &); // hide asignment

    void writeAuthFile();
    void readAuthFile();

    static Settings* m_instance;

    qint64 m_pkFingerprint;
    RSA *m_pubKey;
    QString m_authFilename;
    QString m_secretChatFilename;
    QString m_downloadsDirectory;
    QString m_stateFilename;
    qint32 m_workingDcNum;
    qint32 m_ourId;
    QList<DC *> m_dcsList;
    QString m_phoneNumber;
    // if true, all operations are performed against telegram test servers
    bool m_testMode;
    // if true, downloading a big file will be stored in byte array until completion and only one signal is emited when download finish
    // if false, every downloaded part is notified as a signal with the partial data. Default is false
    bool m_managedDownloads;
    // default is 'en'
    QString m_langCode;

};

#endif // SETTINGS_H
