/*
    This file is part of libqtelegram library.

    Telegram-client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Telegram-client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this telegram-client.  If not, see <http://www.gnu.org/licenses/>.

    Copyright Roberto Mier & Tiago Herrmann & Vitaly Valtman 2014
*/
#ifndef DOWNLOADFILE_H
#define DOWNLOADFILE_H

#include <QString>
#include <QByteArray>
#include "core/constants.h"
#include "types/inputfilelocation.h"
#include "file.h"
#include "core/session.h"

class DownloadFile : public File
{
    Q_OBJECT
public:
    DownloadFile(Session *session, InputFileLocation fileLocation, qint32 expectedSize, QObject *parent = 0);
    ~DownloadFile();

    void appendBytes(const QByteArray &bytes);
    QByteArray bytes() const;
    qint32 offset() const;
    void setOffset(qint32 offset);
    void setMtime(qint32 mtime);
    qint32 mtime() const;
    InputFileLocation fileLocation() const;
    qint32 expectedSize() const ;
    qint32 partId() const;
    void incrementPartId();

private:
    InputFileLocation m_fileLocation;
    QString m_name;
    QByteArray m_bytes;
    qint32 m_offset;
    qint32 m_mtime;
    qint32 m_expectedSize;
    qint32 m_partId;
};

#endif // DOWNLOADFILE_H
