/*
    This file is part of libqtelegram library.

    Telegram-client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Telegram-client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this telegram-client.  If not, see <http://www.gnu.org/licenses/>.

    Copyright Roberto Mier & Tiago Herrmann & Vitaly Valtman 2014
*/
#ifndef FILE_H
#define FILE_H

#include <QObject>
#include "core/constants.h"
#include "core/utils.h"
#include "core/session.h"

class File : public QObject
{
    Q_OBJECT
public:
    explicit File(Session *session, qint64 length, QObject *parent = 0) : QObject(parent), m_session(session), m_length(length) {
        // generate random file id
        Utils::randomBytes(&m_id, 8);
    }
    ~File() {
        if (m_session) {
            m_session->disconnectFromHost();
            m_session->deleteLater();
        }
    }

    void setSession(Session *session) { m_session = session; }
    Session *session() { return m_session; }
    qint64 id() const { return m_id; }
    qint64 length() const { return m_length; }
    qint64 partLength() const { return BLOCK; }

protected:
    Session *m_session;
    qint64 m_id;
    qint64 m_length;
};

#endif // FILE_H
