
/*
    This file is part of libqtelegram library.

    Telegram-client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Telegram-client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this telegram-client.  If not, see <http://www.gnu.org/licenses/>.

    Copyright Roberto Mier & Tiago Herrmann & Vitaly Valtman 2014
*/
#ifndef UPDATE_H
#define UPDATE_H

#include "dcoption.h"
#include "contactsmylink.h"
#include "peernotifysettings.h"
#include "userprofilephoto.h"
#include "chatparticipants.h"
#include "userstatus.h"
#include "message.h"
#include "contactsforeignlink.h"
#include "notifypeer.h"
#include "geochatmessage.h"
#include "encryptedchat.h"

class Update
{
public:

    enum UpdateType {
       typeUpdateUserStatus = 0x1bfbd823,
       typeUpdateNotifySettings = 0xbec268ef,
       typeUpdateMessageID = 0x4e90bfd6,
       typeUpdateChatUserTyping = 0x3c46cfe6,
       typeUpdateActivation = 0x6f690963,
       typeUpdateRestoreMessages = 0xd15de04d,
       typeUpdateEncryption = 0xb4a2e88d,
       typeUpdateUserName = 0xda22d9ad,
       typeUpdateUserBlocked = 0x80ece81a,
       typeUpdateNewMessage = 0x13abdb3,
       typeUpdateContactLink = 0x51a48a9a,
       typeUpdateChatParticipantDelete = 0x6e5f8c22,
       typeUpdateNewAuthorization = 0x8f06529a,
       typeUpdateChatParticipantAdd = 0x3a0eeb22,
       typeUpdateDcOptions = 0x8e5e9873,
       typeUpdateDeleteMessages = 0xa92bfe26,
       typeUpdateUserTyping = 0x6baa8508,
       typeUpdateEncryptedChatTyping = 0x1710f156,
       typeUpdateReadMessages = 0xc6649e31,
       typeUpdateUserPhoto = 0x95313b0c,
       typeUpdateContactRegistered = 0x2575bbb9,
       typeUpdateNewEncryptedMessage = 0x12bcbd9a,
       typeUpdateEncryptedMessagesRead = 0x38fe25b7,
       typeUpdateChatParticipants = 0x7761198,
       typeUpdateNewGeoChatMessage = 0x5a68e3f7
    };

    Update(UpdateType classType) :
        m_id(0),
        m_previous(false),
        m_device(""),
        m_myLink(ContactsMyLink::typeContactsMyLinkEmpty),
        m_chatId(0),
        m_userId(0),
        m_pts(0),
        m_version(0),
        m_qts(0),
        m_date(0),
        m_notifySettings(PeerNotifySettings::typePeerNotifySettingsEmpty),
        m_chat(EncryptedChat::typeEncryptedChatEmpty),
        m_inviterId(0),
        m_photo(UserProfilePhoto::typeUserProfilePhotoEmpty),
        m_participants(ChatParticipants::typeChatParticipants),
        m_randomId(0),
        m_firstName(""),
        m_lastName(""),
        m_status(UserStatus::typeUserStatusEmpty),
        m_message(Message::typeMessageEmpty),
        m_geoChatMessage(GeoChatMessage::typeGeoChatMessageEmpty),
        m_foreignLink(ContactsForeignLink::typeContactsForeignLinkUnknown),
        m_maxDate(0),
        m_authKeyId(0),
        m_location(""),
        m_peer(NotifyPeer::typeNotifyAll),
        m_blocked(false),
        m_classType(classType) {}

    void setUserId(qint32 userId) {
        m_userId = userId;
    }
    qint32 userId() const {
        return m_userId;
    }
    void setStatus(UserStatus status) {
        m_status = status;
    }
    UserStatus status() const {
        return m_status;
    }
    void setPeer(NotifyPeer peer) {
        m_peer = peer;
    }
    NotifyPeer peer() const {
        return m_peer;
    }
    void setNotifySettings(PeerNotifySettings notifySettings) {
        m_notifySettings = notifySettings;
    }
    PeerNotifySettings notifySettings() const {
        return m_notifySettings;
    }
    void setId(qint32 id) {
        m_id = id;
    }
    qint32 id() const {
        return m_id;
    }
    void setRandomId(qint64 randomId) {
        m_randomId = randomId;
    }
    qint64 randomId() const {
        return m_randomId;
    }
    void setChatId(qint32 chatId) {
        m_chatId = chatId;
    }
    qint32 chatId() const {
        return m_chatId;
    }
    void setMessages(const QList<qint32> & messages) {
        m_messages = messages;
    }
    QList<qint32> messages() const {
        return m_messages;
    }
    void setPts(qint32 pts) {
        m_pts = pts;
    }
    qint32 pts() const {
        return m_pts;
    }
    void setChat(EncryptedChat chat) {
        m_chat = chat;
    }
    EncryptedChat chat() {
        return m_chat;
    }
    void setDate(qint32 date) {
        m_date = date;
    }
    qint32 date() const {
        return m_date;
    }
    void setFirstName(const QString & firstName) {
        m_firstName = firstName;
    }
    QString firstName() const {
        return m_firstName;
    }
    void setLastName(const QString & lastName) {
        m_lastName = lastName;
    }
    QString lastName() const {
        return m_lastName;
    }
    void setBlocked(bool blocked) {
        m_blocked = blocked;
    }
    bool blocked() const {
        return m_blocked;
    }
    void setMessage(Message message) {
        m_message = message;
    }
    Message message() const {
        return m_message;
    }
    void setMessage(GeoChatMessage message) {
        m_geoChatMessage = message;
    }
    GeoChatMessage geoChatMessage() const {
        return m_geoChatMessage;
    }
    void setMyLink(ContactsMyLink myLink) {
        m_myLink = myLink;
    }
    ContactsMyLink myLink() const {
        return m_myLink;
    }
    void setForeignLink(ContactsForeignLink foreignLink) {
        m_foreignLink = foreignLink;
    }
    ContactsForeignLink foreignLink() const {
        return m_foreignLink;
    }
    void setVersion(qint32 version) {
        m_version = version;
    }
    qint32 version() const {
        return m_version;
    }
    void setAuthKeyId(qint64 authKeyId) {
        m_authKeyId = authKeyId;
    }
    qint64 authKeyId() const {
        return m_authKeyId;
    }
    void setDevice(const QString & device) {
        m_device = device;
    }
    QString device() const {
        return m_device;
    }
    void setLocation(const QString & location) {
        m_location = location;
    }
    QString location() const {
        return m_location;
    }
    void setInviterId(qint32 inviterId) {
        m_inviterId = inviterId;
    }
    qint32 inviterId() const {
        return m_inviterId;
    }
    void setDcOptions(const QList<DcOption> & dcOptions) {
        m_dcOptions = dcOptions;
    }
    QList<DcOption> dcOptions() const {
        return m_dcOptions;
    }
    void setPhoto(UserProfilePhoto photo) {
        m_photo = photo;
    }
    UserProfilePhoto photo() const {
        return m_photo;
    }
    void setPrevious(bool previous) {
        m_previous = previous;
    }
    bool previous() const {
        return m_previous;
    }
    void setQts(qint32 qts) {
        m_qts = qts;
    }
    qint32 qts() const {
        return m_qts;
    }
    void setMaxDate(qint32 maxDate) {
        m_maxDate = maxDate;
    }
    qint32 maxDate() const {
        return m_maxDate;
    }
    void setParticipants(ChatParticipants participants) {
        m_participants = participants;
    }
    ChatParticipants participants() const {
        return m_participants;
    }
    void setClassType(UpdateType classType) {
        m_classType = classType;
    }
    UpdateType classType() const {
        return m_classType;
    }

private:
    qint32 m_id;
    bool m_previous;
    QList<DcOption> m_dcOptions;
    QString m_device;
    ContactsMyLink m_myLink;
    qint32 m_chatId;
    qint32 m_userId;
    qint32 m_pts;
    qint32 m_version;
    qint32 m_qts;
    qint32 m_date;
    PeerNotifySettings m_notifySettings;
    EncryptedChat m_chat;
    qint32 m_inviterId;
    UserProfilePhoto m_photo;
    QList<qint32> m_messages;
    ChatParticipants m_participants;
    qint64 m_randomId;
    QString m_firstName;
    QString m_lastName;
    UserStatus m_status;
    Message m_message;
    GeoChatMessage m_geoChatMessage;
    ContactsForeignLink m_foreignLink;
    qint32 m_maxDate;
    qint64 m_authKeyId;
    QString m_location;
    NotifyPeer m_peer;
    bool m_blocked;
    UpdateType m_classType;
};
#endif // UPDATE_H
