#ifndef DATA_H
#define DATA_H

#include <QObject>
#include <QList>
#include <QLoggingCategory>
#include <types/message.h>
#include <types/dialog.h>
#include <types/user.h>
#include <types/chat.h>
#include "models/dialogitem.h"
#include "models/contactitem.h"
#include "models/messageitem.h"

class Data : public QObject
{
    Q_OBJECT
public:
    static Data *instance();

    // dialog operations
    bool insertDialogs(const QList<Dialog> &dialogs, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);
    bool insertOrUpdateDialog(const Dialog &dialog);
    bool insertDialog(const Dialog &dialog);
    bool updateDialog(const Dialog &dialog);
    bool deleteDialog(qint32 dialogId);
    bool deleteDialogs();
    DialogItem *getDialog(qint32 dialogId);
    QList<DialogItem *> getDialogs();

    // user operations
    bool insertOrUpdateUser(const User &user);
    bool insertUser(const User &user);
    bool updateUser(const User &user);
    bool updateUserThumbnail(qint32 userId, const QString &thumbnail);
    bool deleteUsers();
    ContactItem *getUser(qint32 id);
    QList<ContactItem *> getUsers();

    // chat operations
    bool insertOrUpdateChat(const Chat &chat);
    bool insertChat(const Chat &chat);
    bool updateChat(const Chat &chat);
    bool deleteChat(qint32 chatId);
    DialogItem *getChat(qint32 id);

    // message operations
    bool insertOrUpdateMessage(const Message &message);
    bool insertMessage(const Message &message);
    bool updateMessage(const Message &message);
    bool deleteMessages(qint32 dialogId);
    bool deleteMessages();
    bool markMessageAsRead(qint32 messageId);
    MessageItem *getMessage(qint32 messageId);
    MessageItem *topMessage(qint32 dialogId, bool isChat);

Q_SIGNALS:
    void contactsAdded(QList<ContactItem *> contacts);
    void dialogsAdded(QList<DialogItem *> dialogs, qint32 totalCount);

protected:
    explicit Data(QObject *parent = 0);

private Q_SLOTS:
    void onMessagesGetDialogsAnswer(qint64 id, qint32 totalCount, const QList<Dialog> &dialogs, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);

};

#endif // DATA_H
