#ifndef CONTACTSMODEL_H
#define CONTACTSMODEL_H

#include <QAbstractListModel>
#include "contactitem.h"

class ContactsModel : public QAbstractListModel
{
    Q_OBJECT
public:

    enum ContactsRoles {
        IdRole = Qt::UserRole,
        FirstNameRole,
        LastNameRole,
        PhoneRole,
        LastSeenOnlineRole,
        OnlineRole,
        ThumbnailRole
    };

    static ContactsModel *getInstance();

    explicit ContactsModel(QObject *parent = 0);

    // Overriding
    QHash<qint32, QByteArray> roleNames() const;
    qint32 rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;

    void append(ContactItem *contact);
    void clear();
    ContactItem *contact(qint32 id);
    void updateThumbnail(qint32 id, const QString &thumbnailPath);


private:
    QList<ContactItem *> m_contacts;
    // correlates contact->id() --> index in m_contacts
    QMap<qint32, qint32> m_contactIdsMap;


private Q_SLOTS:
    void onContactsAdded(const QList<ContactItem *> &contacts);
};

#endif // CONTACTSMODEL_H
