#include "dialogitem.h"
#include "modelsmanager.h"

QString DialogItem::whoisTyping() const {
    if (m_whoisTyping != 0) {
        ContactsModel &contactsModel = ModelsManager::instance()->contactsModel();
        ContactItem *user = contactsModel.contact(m_whoisTyping);
        return user->firstName() + " " + user->lastName();
    } else {
        return "";
    }
}

QString DialogItem::name() const {
    if (!isChat()) {
        // if not a chat, look for data in contacts model
        ContactsModel &contactsModel = ModelsManager::instance()->contactsModel();
        ContactItem *user = contactsModel.contact(this->id());
        return user->firstName() + " " + user->lastName();
    }
    // otherwise return stored name
    return m_name;
}

MessageItem *DialogItem::topMessage() {
    return m_messagesModel.topMessage();
}

void DialogItem::appendMessage(MessageItem *m) {
    m_messagesModel.append(m);
}

void DialogItem::createTopMessage() {
    MessageItem *msg = new MessageItem(this);
    m_messagesModel.append(msg);
}

void DialogItem::setIsTyping(qint32 whoisTyping) {
    m_whoisTyping = whoisTyping;
    m_typing = m_whoisTyping != 0;
    //remove previous timer if alive
    if (m_timerId != 0) {
        killTimer(m_timerId);
    }
    if (m_typing) {
        m_timerId = startTimer(TYPING_INTERVAL);
    } else {
       Q_EMIT typingFinished();
    }
}

MessagesModel &DialogItem::getMessagesModel() {
    return m_messagesModel;
}

void DialogItem::dump() {
    qCDebug(TG_PLUGIN_LOGIC) << "ID" << id();
    qCDebug(TG_PLUGIN_LOGIC) << "ischat" << isChat();
    qCDebug(TG_PLUGIN_LOGIC) << "name" << name();
    qCDebug(TG_PLUGIN_LOGIC) << "unreadCount" << unreadCount();
    qCDebug(TG_PLUGIN_LOGIC) << "thumbnail" << thumbnail();
    qCDebug(TG_PLUGIN_LOGIC) << "typing" << typing();
    qCDebug(TG_PLUGIN_LOGIC) << "whoisTyping" << whoisTyping();
    qCDebug(TG_PLUGIN_LOGIC) << "members" << members();
    qCDebug(TG_PLUGIN_LOGIC) << "membersOnline" << membersOnline();
    qCDebug(TG_PLUGIN_LOGIC) << "useronline" << userOnline();
    qCDebug(TG_PLUGIN_LOGIC) << "lastSeenOnline" << lastSeenOnline();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageId" << topMessage()->id();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageFromId" << topMessage()->fromId();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageFromFirstName" << topMessage()->fromFirstName();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageFromLastName" << topMessage()->fromLastName();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageFromPhone" << topMessage()->fromPhone();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageOut" << topMessage()->out();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageUnread" << topMessage()->unread();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageText" << topMessage()->text();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageMediaType" << QString::number(topMessage()->mediaType());
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageDate" << topMessage()->date();
    qCDebug(TG_PLUGIN_LOGIC) << "---";
}
