#ifndef DIALOGITEM_H
#define DIALOGITEM_H

#define TYPING_INTERVAL 5000

#include "types/tldialog.h"
#include "messageitem.h"
#include "messagesmodel.h"
#include <QTimerEvent>
#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(TG_PLUGIN_LOGIC)

class DialogItem : public TLDialog {
    Q_OBJECT
    Q_PROPERTY(qint32 id READ id)
    Q_PROPERTY(bool isChat READ isChat)
    Q_PROPERTY(QString name READ name)
    Q_PROPERTY(QString thumbnail READ thumbnail)
    Q_PROPERTY(bool typing READ typing)
    Q_PROPERTY(QString whoisTyping READ whoisTyping)
    Q_PROPERTY(qint32 members READ members)
    Q_PROPERTY(qint32 membersOnline READ membersOnline)
    Q_PROPERTY(bool userOnline READ userOnline)
    Q_PROPERTY(qint32 lastSeenOnline READ lastSeenOnline)
    Q_PROPERTY(MessageItem *topMessage READ topMessage)

public:

    explicit DialogItem(QObject *parent = 0) :
        TLDialog(parent),
        m_id(0),
        m_isChat(0),
        m_name(""),
        m_thumbnail(""),
        m_typing(false),
        m_whoisTyping(0),
        m_members(0),
        m_membersOnline(0),
        m_userOnline(false),
        m_lastSeenOnline(0),
        m_timerId(0) {}

    qint32 id() const { return m_id; }
    bool isChat() const { return m_isChat; }
    QString name() const;
    QString thumbnail() const { return m_thumbnail; }
    bool typing() const { return m_typing; }
    QString whoisTyping() const;
    qint32 members() const { return m_members; }
    qint32 membersOnline() const { return m_membersOnline; }
    bool userOnline() const { return m_userOnline; }
    qint32 lastSeenOnline() const { return m_lastSeenOnline; }

    void setId(qint32 id) { m_id = id; }
    void setIsChat(bool isChat) { m_isChat = isChat; }
    void setName(const QString &name) { m_name = name; }
    void setThumbnail(QString thumbnail) { m_thumbnail = thumbnail; }
    void setMembers(qint32 members) { m_members = members; }
    void setMembersOnline(qint32 membersOnline) { m_membersOnline = membersOnline; }
    void setUserOnline(bool userOnline) { m_userOnline = userOnline; }
    void setLastSeenOnline(qint32 lastSeenOnline) { m_lastSeenOnline = lastSeenOnline; }


    MessageItem *topMessage();
    void appendMessage(MessageItem *m);
    void createTopMessage();
    void setIsTyping(qint32 whoisTyping);

    MessagesModel &getMessagesModel();

    void dump();

Q_SIGNALS:
    void typingFinished();

private:
    // peer identifier (userId or chatId)
    qint32 m_id;
    // flag to set if this dialog belongs to a chat. if false, is a user
    bool m_isChat;
    // name of the user / title of the chat
    QString m_name;
    QString m_thumbnail;
    bool m_typing;
    qint32 m_whoisTyping;
    qint32 m_members; //XXX Could this be referenced to a chats list in Cache?
    qint32 m_membersOnline;
    bool m_userOnline;
    qint32 m_lastSeenOnline;

    // MessagesModel
    MessagesModel m_messagesModel;


    qint32 m_timerId;
    void timerEvent(QTimerEvent *timerEvent) {
        Q_ASSERT(m_timerId == timerEvent->timerId());
        killTimer(m_timerId);
        m_timerId = 0;
        m_typing = false;
        m_whoisTyping = 0;
        Q_EMIT typingFinished();
    }
};

#endif // DIALOGITEM_H
