#ifndef DIALOGSMODEL_H
#define DIALOGSMODEL_H

#include <QAbstractListModel>
#include "dialogitem.h"

class DialogsModel : public QAbstractListModel
{
    Q_OBJECT
public:

    enum DialogRoles {
        IdRole = Qt::UserRole,
        IsChatRole,
        NameRole,
        UnreadCountRole,
        ThumbnailRole,
        TypingRole,
        WhoisTypingRole,
        MembersRole,
        MembersOnlineRole,
        UserOnlineRole,
        LastSeenOnlineRole,
        TopMessageRole
    };

    explicit DialogsModel(QObject *parent = 0);
    ~DialogsModel();

    // dialog items
    void append(DialogItem *dialog);
    void clear();

    // updates
    void updateThumbnail(qint32 dialogId, QString filePath);
    void updateReceivedTopMessage(qint32 dialogId, qint32 messageId, const QString &text, qint32 date, qint32 fromId);
    void updateSendingTopMessage(qint32 dialogId, qint32 requestId, const QString &text);
    void updateSentTopMessage(qint32 requestId, qint32 messageId, qint32 date);
    void markAsRead(qint32 messageId);
    void setIsTyping(qint32 dialogId, qint32 userId);

    // Overriding
    QHash<qint32, QByteArray> roleNames() const;
    qint32 rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
    Qt::ItemFlags flags(const QModelIndex &index) const;

    Q_INVOKABLE QVariant get(int row);
    Q_INVOKABLE QVariant getMessagesModel(int row);
    Q_INVOKABLE QVariant getMessagesModelFromId(int dialogId);

private:
    QList<DialogItem *> m_dialogs;
    // correlates dialog->id() --> index in m_dialogs
    QMap<qint32, qint32> m_dlgIdsMap;
    // correlates dialog->topMessage()->id() --> dialog->id()
    QMap<qint32, qint32> m_topMessageIdsMap;

    bool m_canFetchMore;

private Q_SLOTS:
    void onTypingFinished();
    void onDialogsAdded(const QList<DialogItem *> &dialogs, qint32 totalCount);
};

#endif // DIALOGSMODEL_H
