#include "messagesmodel.h"

MessagesModel::MessagesModel(QObject *parent) :
    QAbstractListModel(parent) {
}

MessagesModel::~MessagesModel() {
}

QHash<int, QByteArray> MessagesModel::roleNames() const {
    QHash<int, QByteArray> roles;
    roles[IdRole] = "id";
    roles[FromIdRole] = "fromId";
    roles[FromFirstNameRole] = "fromFirstName";
    roles[FromLastNameRole] = "fromLastName";
    roles[FromPhoneRole] = "fromPhone";
    roles[FromThumbnailRole] = "fromThumbnail";
    roles[ToIdRole] = "toId";
    roles[OutRole] = "out";
    roles[UnreadRole] = "unread";
    roles[DateRole] = "date";
    roles[TextRole] = "text";
    roles[MediaTypeRole] = "mediaType";
    return roles;
}

qint32 MessagesModel::rowCount(const QModelIndex &parent) const {

    if (parent.isValid()) {
        return 0;
    }

    return m_messages.size();
}

QVariant MessagesModel::data(const QModelIndex &index, int role) const {

    if (!index.isValid() || index.row() < 0 || index.row() >= m_messages.count()) {
        return QVariant();
    }

    MessageItem* message = m_messages.at(index.row());

    QVariant result;
    // depending on role value, return one or another property as QVariant
    switch (role) {
    case IdRole:
        result = message->id();
        break;
    case FromIdRole:
        result = message->fromId();
        break;
    case FromFirstNameRole:
        result = message->fromFirstName();
        break;
    case FromLastNameRole:
        result = message->fromLastName();
        break;
    case FromPhoneRole:
        result = message->fromPhone();
        break;
    case FromThumbnailRole:
        result = message->fromThumbnail();
        break;
    case ToIdRole:
        result = message->toId();
        break;
    case OutRole:
        result = message->out();
        break;
    case UnreadRole:
        return message->unread();
        break;
    case TextRole:
        result = message->text();
        break;
    case MediaTypeRole:
        result = static_cast<qint32>(message->mediaType());
        break;
    }

    return result;
}

Qt::ItemFlags MessagesModel::flags(const QModelIndex &index) const {
    if (!index.isValid()) {
        return Qt::ItemIsEnabled;
    }
    return QAbstractItemModel::flags(index) /*| Qt::ItemIsEditable*/;
}

// management methods
MessageItem *MessagesModel::topMessage() {
    if (!m_messages.isEmpty()) {
        return m_messages.last();
    }
    return 0;
}

void MessagesModel::append(MessageItem *newMessage) {
    m_messages.append(newMessage);
}
