#ifndef MESSAGESMODEL_H
#define MESSAGESMODEL_H

#include <QAbstractListModel>
#include "messageitem.h"

class MessagesModel : public QAbstractListModel
{
    Q_OBJECT
public:

    enum MessageRoles {
        IdRole = Qt::UserRole,
        FromIdRole,
        FromFirstNameRole,
        FromLastNameRole,
        FromPhoneRole,
        FromThumbnailRole,
        ToIdRole,
        OutRole,
        UnreadRole,
        DateRole,
        TextRole,
        MediaTypeRole
    };
    explicit MessagesModel(QObject *parent = 0);
    ~MessagesModel();

    // abstract model overriding
    QHash<qint32, QByteArray> roleNames() const;
    qint32 rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
    Qt::ItemFlags flags(const QModelIndex &index) const;

    // management methods
    MessageItem *topMessage();
    void append(MessageItem *newMessage);

private:
    // list of the messages in model
    QList<MessageItem *> m_messages;
    // correlates message->id() --> index in m_messages
    QMap<qint32, qint32> m_keysMap;

};

#endif // MESSAGESMODEL_H
