#include "rawapiclient.h"
#include <QMetaType>

Telegram *RawApiClient::m_telegram = 0;

Telegram *RawApiClient::libObject() {
    return m_telegram;
}

RawApiClient::RawApiClient(QObject *parent) :
    QObject(parent) {
    //output types
    qRegisterMetaType<TLAudio>("TLAudio");
    qRegisterMetaType<TLChat>("TLChat");
    qRegisterMetaType<TLChatParticipant>("TLChatParticipant");
    qRegisterMetaType<TLChatParticipants>("TLChatParticipants");
    qRegisterMetaType<TLChatPhoto>("TLChatPhoto");
    qRegisterMetaType<TLContact>("TLContact");
    qRegisterMetaType<TLContactBlocked>("TLContactBlocked");
    qRegisterMetaType<TLContactsForeignLink>("TLContactsForeignLink");
    qRegisterMetaType<TLContactsLink>("TLContactsLink");
    qRegisterMetaType<TLContactsMyLink>("TLContactsMyLink");
    qRegisterMetaType<TLContactStatus>("TLContactStatus");
    qRegisterMetaType<TLDcOption>("TLDcOption");
    qRegisterMetaType<TLDialog>("TLDialog");
    qRegisterMetaType<TLDocument>("TLDocument");
    qRegisterMetaType<TLEncryptedChat>("TLEncryptedChat");
    qRegisterMetaType<TLEncryptedFile>("TLEncryptedFile");
    qRegisterMetaType<TLEncryptedMessage>("TLEncryptedMessage");
    qRegisterMetaType<TLFileLocation>("TLFileLocation");
    qRegisterMetaType<TLGeoChatMessage>("TLGeoChatMessage");
    qRegisterMetaType<TLGeoPoint>("TLGeoPoint");
    qRegisterMetaType<TLImportedContact>("TLImportedContact");
    qRegisterMetaType<TLMessage>("TLMessage");
    qRegisterMetaType<TLMessageAction>("TLMessageAction");
    qRegisterMetaType<TLMessageMedia>("TLMessageMedia");
    qRegisterMetaType<TLMessagesFilter>("TLMessagesFilter");
    qRegisterMetaType<TLNotifyPeer>("TLNotifyPeer");
    qRegisterMetaType<TLPeer>("TLPeer");
    qRegisterMetaType<TLPeerNotifySettings>("TLPeerNotifySettings");
    qRegisterMetaType<TLPhoto>("TLPhoto");
    qRegisterMetaType<TLPhotoSize>("TLPhotoSize");
    qRegisterMetaType<TLStorageFileType>("TLStorageFileType");
    qRegisterMetaType<TLUpdate>("TLUpdate");
    qRegisterMetaType<TLUser>("TLUser");
    qRegisterMetaType<TLUserProfilePhoto>("TLUserProfilePhoto");
    qRegisterMetaType<TLUserStatus>("TLUserStatus");
    qRegisterMetaType<TLVideo>("TLVideo");
    //input types
    qRegisterMetaType<TLInputFileLocation>("TLInputFileLocation");
    qRegisterMetaType<TLInputGeoPoint>("TLInputGeoPoint");
    qRegisterMetaType<TLInputPeer>("TLInputPeer");
    qRegisterMetaType<TLInputUser>("TLInputUser");
    qRegisterMetaType<TLMessagesFilter>("TLMessagesFilter");
}

RawApiClient::~RawApiClient() {
    if (m_telegram) delete m_telegram;
}

void RawApiClient::initTelegramLibrary() {
    ASSERT(!m_telegram);
    Config *c = Config::instance();
    Q_ASSERT(c->phoneNumber().length() > 0);
    if (c->configPath().isEmpty()) {
        m_telegram = new Telegram(c->phoneNumber());
    } else if (c->publicKeyFile().isEmpty()) {
        m_telegram = new Telegram(c->phoneNumber(), c->configPath());
    } else {
        m_telegram = new Telegram(c->phoneNumber(), c->configPath(), c->publicKeyFile());
    }

    // Errors signal-slot
    connect(m_telegram, SIGNAL(error(qint64,qint32,QString)), this, SIGNAL(error(qint64,qint32,QString)));
    connect(m_telegram, SIGNAL(authSignInError(qint64,qint32,QString)), this, SIGNAL(authSignInError(qint64,qint32,QString)));
    connect(m_telegram, SIGNAL(authSignUpError(qint64,qint32,QString)), this, SIGNAL(authSignUpError(qint64,qint32,QString)));
    // Registration / authorization signal-slot
    connect(m_telegram, SIGNAL(authNeeded()), this, SIGNAL(authNeeded()));
    connect(m_telegram, SIGNAL(authLoggedIn()), this, SIGNAL(authLoggedIn()));
    connect(m_telegram, SIGNAL(authCheckPhoneAnswer(qint64,bool,bool)), this, SIGNAL(authCheckPhoneAnswer(qint64,bool,bool)));
    connect(m_telegram, SIGNAL(authSendCodeAnswer(qint64,bool,qint32)), this, SIGNAL(authSendCodeAnswer(qint64,bool,qint32)));
    connect(m_telegram, SIGNAL(authSendCallAnswer(qint64,bool)), this, SIGNAL(authSendCallAnswer(qint64,bool)));
    connect(m_telegram, SIGNAL(authLogOutAnswer(qint64,bool)), this, SIGNAL(authLogOutAnswer(qint64,bool)));
    connect(m_telegram, SIGNAL(authSendInvitesAnswer(qint64,bool)), this, SIGNAL(authSendInvitesAnswer(qint64,bool)));
    connect(m_telegram, SIGNAL(authResetAuthorizationsAnswer(qint64,bool)), this, SIGNAL(authResetAuthorizationsAnswer(qint64,bool)));
    // Working with users signal-slot
    connect(m_telegram, SIGNAL(usersGetUsersAnswer(qint64,QList<User>)), this, SLOT(onUsersGetUsersAnswer(qint64,QList<User>)));
    connect(m_telegram, SIGNAL(usersGetFullUserAnswer(qint64,User,ContactsLink,Photo,PeerNotifySettings,bool,QString,QString)), this, SLOT(onUsersGetFullUserAnswer(qint64,User,ContactsLink,Photo,PeerNotifySettings,bool,QString,QString)));
    connect(m_telegram, SIGNAL(photosGetUserPhotosAnswer(qint64,qint32,QList<Photo>,QList<User>)), this, SLOT(onPhotosGetUserPhotosAnswer(qint64,qint32,QList<Photo>,QList<User>)));
    // Working with contacts signal-slot
    connect(m_telegram, SIGNAL(contactsGetStatusesAnswer(qint64,QList<ContactStatus>)), this, SLOT(onContactsGetStatusesAnswer(qint64,QList<ContactStatus>)));
    connect(m_telegram, SIGNAL(contactsGetContactsAnswer(qint64,bool,QList<Contact>,QList<User>)), this, SLOT(onContactsGetContactsAnswer(qint64,bool,QList<Contact>,QList<User>)));
    connect(m_telegram, SIGNAL(contactsImportContactsAnswer(qint64,QList<ImportedContact>,QList<qint64>,QList<User>)), this, SLOT(onContactsImportContactsAnswer(qint64,QList<ImportedContact>,QList<qint64>,QList<User>)));
    connect(m_telegram, SIGNAL(contactsDeleteContactAnswer(qint64,ContactsMyLink,ContactsForeignLink,User)), this, SLOT(onContactsDeleteContactAnswer(qint64,ContactsMyLink,ContactsForeignLink,User)));
    connect(m_telegram, SIGNAL(contactsDeleteContactsAnswer(qint64,bool)), this, SIGNAL(contactsDeleteContactsAnswer(qint64,bool)));
    // Working with blacklist signal-slot
    connect(m_telegram, SIGNAL(contactsBlockAnswer(qint64,bool)), this, SIGNAL(contactsBlockAnswer(qint64,bool)));
    connect(m_telegram, SIGNAL(contactsUnblockAnswer(qint64,bool)), this, SIGNAL(contactsUnblockAnswer(qint64,bool)));
    connect(m_telegram, SIGNAL(contactsGetBlockedAnswer(qint64,qint32,QList<ContactBlocked>,QList<User>)), this, SLOT(onContactsGetBlockedAnswer(qint64,qint32,QList<ContactBlocked>,QList<User>)));
    // Working with messages signal-slot
    connect(m_telegram, SIGNAL(messagesSendMessageAnswer(qint64,qint32,qint32,qint32,qint32,QList<ContactsLink>)), this, SLOT(onMessagesSendMessageAnswer(qint64,qint32,qint32,qint32,qint32,QList<ContactsLink>)));
    connect(m_telegram, SIGNAL(messagesSendMediaAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendMediaAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(m_telegram, SIGNAL(messagesSendPhotoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendPhotoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(m_telegram, SIGNAL(messagesSendGeoPointAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendGeoPointAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(m_telegram, SIGNAL(messagesSendContactAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendContactAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(m_telegram, SIGNAL(messagesSendVideoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendVideoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(m_telegram, SIGNAL(messagesSendAudioAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendAudioAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(m_telegram, SIGNAL(messagesSendDocumentAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendDocumentAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(m_telegram, SIGNAL(messagesSetTypingAnswer(qint64,bool)), this, SIGNAL(messagesSetTypingAnswer(qint64,bool)));
    connect(m_telegram, SIGNAL(messagesGetMessagesAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)), this, SLOT(onMessagesGetMessagesAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)));
    connect(m_telegram, SIGNAL(messagesGetDialogsAnswer(qint64,qint32, QList<Dialog>,QList<Message>,QList<Chat>,QList<User>)), this, SLOT(onMessagesGetDialogsAnswer(qint64,qint32, QList<Dialog>,QList<Message>,QList<Chat>,QList<User>)));
    connect(m_telegram, SIGNAL(messagesGetHistoryAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)), this, SLOT(onMessagesGetHistoryAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)));
    connect(m_telegram, SIGNAL(messagesSearchAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)), this, SLOT(onMessagesSearchAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)));
    connect(m_telegram, SIGNAL(messagesReadHistoryAnswer(qint64,qint32,qint32,qint32)), this, SIGNAL(messagesReadHistoryAnswer(qint64,qint32,qint32,qint32)));
    connect(m_telegram, SIGNAL(messagesDeleteHistoryAnswer(qint64,qint32,qint32,qint32)), this, SIGNAL(messagesDeleteHistoryAnswer(qint64,qint32,qint32,qint32)));
    connect(m_telegram, SIGNAL(messagesDeleteMessagesAnswer(qint64,QList<qint32>)), this, SIGNAL(messagesDeleteMessagesAnswer(qint64,QList<qint32>)));
    connect(m_telegram, SIGNAL(messagesRestoreMessagesAnswer(qint64,QList<qint32>)), this, SIGNAL(messagesRestoreMessagesAnswer(qint64,QList<qint32>)));
    connect(m_telegram, SIGNAL(messagesReceivedMessagesAnswer(qint64,QList<qint32>)), this, SIGNAL(messagesReceivedMessagesAnswer(qint64,QList<qint32>)));
    connect(m_telegram, SIGNAL(messagesForwardMessageAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesForwardMessageAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    connect(m_telegram, SIGNAL(messagesForwardMessagesAnswer(qint64,QList<Message>,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesForwardMessagesAnswer(qint64,QList<Message>,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
    // Working with updates signal-slot
    connect(m_telegram, SIGNAL(updatesGetStateAnswer(qint64,qint32,qint32,qint32,qint32,qint32)), this, SIGNAL(updatesGetStateAnswer(qint64,qint32,qint32,qint32,qint32,qint32)));
    connect(m_telegram, SIGNAL(updatesGetDifferenceAnswer(qint64,qint32,qint32)), this, SIGNAL(updatesGetDifferenceAnswer(qint64,qint32,qint32)));
    connect(m_telegram, SIGNAL(updatesGetDifferenceAnswer(qint64,QList<Message>,QList<EncryptedMessage>,QList<Update>,QList<Chat>,QList<User>,UpdatesState,bool)), this, SLOT(onUpdatesGetDifferenceAnswer(qint64,QList<Message>,QList<EncryptedMessage>,QList<Update>,QList<Chat>,QList<User>,UpdatesState,bool)));
    // Working with files signal-slot
    connect(m_telegram, SIGNAL(uploadGetFileAnswer(qint64,StorageFileType,qint32,QByteArray,qint32,qint32,qint32)), this, SLOT(onUploadGetFileAnswer(qint64,StorageFileType,qint32,QByteArray,qint32,qint32,qint32)));
    // Updates
    connect(m_telegram, SIGNAL(updatesTooLong()), this, SIGNAL(updatesTooLong()));
    connect(m_telegram, SIGNAL(updateShortMessage(qint32,qint32,QString,qint32,qint32,qint32)), this, SIGNAL(updateShortMessage(qint32,qint32,QString,qint32,qint32,qint32)));
    connect(m_telegram, SIGNAL(updateShortChatMessage(qint32,qint32,qint32,QString,qint32,qint32,qint32)), this, SIGNAL(updateShortChatMessage(qint32,qint32,qint32,QString,qint32,qint32,qint32)));
    connect(m_telegram, SIGNAL(updateShort(Update,qint32)), this, SLOT(onUpdateShort(Update,qint32)));
    connect(m_telegram, SIGNAL(updatesCombined(QList<Update>,QList<User>,QList<Chat>,qint32,qint32,qint32)), this, SLOT(onUpdatesCombined(QList<Update>,QList<User>,QList<Chat>,qint32,qint32,qint32)));
    connect(m_telegram, SIGNAL(updates(QList<Update>,QList<User>,QList<Chat>,qint32,qint32)), this, SLOT(onUpdates(QList<Update>,QList<User>,QList<Chat>,qint32,qint32)));
}

// Registration / authorization - input operations
qint64 RawApiClient::authCheckPhone() {
    return m_telegram->authCheckPhone();
}

qint64 RawApiClient::authSendCode() {
    return m_telegram->authSendCode();
}

qint64 RawApiClient::authSignIn(const QString &code) {
    return m_telegram->authSignIn(code);
}

qint64 RawApiClient::authSignUp(const QString &code, const QString &firstName, const QString &lastName) {
    return m_telegram->authSignUp(code, firstName, lastName);
}

qint64 RawApiClient::authSendCall() {
    return m_telegram->authSendCall();
}

qint64 RawApiClient::authLogOut() {
    return m_telegram->authLogOut();
}

qint64 RawApiClient::authSendInvites(const QStringList &phoneNumbers, const QString &inviteText) {
    return m_telegram->authSendInvites(phoneNumbers, inviteText);
}

qint64 RawApiClient::authResetAuthorizations() {
    return m_telegram->authResetAuthorizations();
}

// Working with users - input operations
qint64 RawApiClient::usersGetUsers(QVariantList users) {
    QList<InputUser> inputUsers = toQList<InputUser>(users);
    return m_telegram->usersGetUsers(inputUsers);
}

qint64 RawApiClient::usersGetFullUser(TLInputUser *user) {
    return m_telegram->usersGetFullUser(*user);
}

qint64 RawApiClient::photosGetUserPhotos(TLInputUser *user, qint32 offset, qint32 maxId, qint32 limit) {
    return m_telegram->photosGetUserPhotos(*user, offset, maxId, limit);
}

// Working with contacts - input operations
qint64 RawApiClient::contactsGetStatuses() {
    return m_telegram->contactsGetStatuses();
}

qint64 RawApiClient::contactsGetContacts() {
    return m_telegram->contactsGetContacts();
}

qint64 RawApiClient::contactsImportContacts(QVariantList contacts, bool replace) {
    QList<InputContact> inputContacts = toQList<InputContact>(contacts);
    return m_telegram->contactsImportContacts(inputContacts, replace);
}

qint64 RawApiClient::contactsDeleteContact(TLInputUser *user) {
    return m_telegram->contactsDeleteContact(*user);
}

qint64 RawApiClient::contactsDeleteContacts(QVariantList users) {
    QList<InputUser> inputUsers = toQList<InputUser>(users);
    return m_telegram->contactsDeleteContacts(inputUsers);
}

// Working with blacklist - input operations
qint64 RawApiClient::contactsBlock(InputUser *user) {
    return m_telegram->contactsBlock(*user);
}

qint64 RawApiClient::contactsUnblock(InputUser *user) {
    return m_telegram->contactsUnblock(*user);
}

qint64 RawApiClient::contactsGetBlocked(qint32 offset, qint32 limit) {
    return m_telegram->contactsGetBlocked(offset, limit);
}

// Working with messages - input operations
qint64 RawApiClient::messagesSendMessage(TLInputPeer *peer, const QString &message) {
    return m_telegram->messagesSendMessage(*peer, message);
}

qint64 RawApiClient::messagesSendPhoto(TLInputPeer *peer, const QByteArray &bytes, const QString &fileName) {
    return m_telegram->messagesSendPhoto(*peer, bytes, fileName);
}

qint64 RawApiClient::messagesSendPhoto(TLInputPeer *peer, const QString &filePath) {
    return m_telegram->messagesSendPhoto(*peer, filePath);
}

qint64 RawApiClient::messagesSendGeoPoint(TLInputPeer *peer, InputGeoPoint *inputGeoPoint) {
    return m_telegram->messagesSendGeoPoint(*peer, *inputGeoPoint);
}

qint64 RawApiClient::messagesSendContact(TLInputPeer *peer, const QString &phoneNumber, const QString &firstName, const QString &lastName) {
    return m_telegram->messagesSendContact(*peer, phoneNumber, firstName, lastName);
}

qint64 RawApiClient::messagesSendVideo(TLInputPeer *peer, const QByteArray &bytes, const QString &fileName, qint32 duration, qint32 width, qint32 height, const QString &mimeType, const QByteArray &thumbnailBytes, const QString &thumbnailName) {
    return m_telegram->messagesSendVideo(*peer, bytes, fileName, duration, width, height, mimeType, thumbnailBytes, thumbnailName);
}

qint64 RawApiClient::messagesSendVideo(TLInputPeer *peer, const QString &filePath, qint32 duration, qint32 width, qint32 height, const QString &thumbnailFilePath) {
    return m_telegram->messagesSendVideo(*peer, filePath, duration, width, height, thumbnailFilePath);
}

qint64 RawApiClient::messagesSendAudio(TLInputPeer *peer, const QByteArray &bytes, const QString &fileName, qint32 duration, const QString &mimeType) {
    return m_telegram->messagesSendAudio(*peer, bytes, fileName, duration, mimeType);
}

qint64 RawApiClient::messagesSendAudio(TLInputPeer *peer, const QString &filePath, qint32 duration) {
    return m_telegram->messagesSendAudio(*peer, filePath, duration);
}

qint64 RawApiClient::messagesSendDocument(TLInputPeer *peer, const QByteArray &bytes, const QString &fileName, const QString &mimeType, const QByteArray &thumbnailBytes, const QString &thumbnailName) {
    return m_telegram->messagesSendDocument(*peer, bytes, fileName, mimeType, thumbnailBytes, thumbnailName);
}

qint64 RawApiClient::messagesSendDocument(TLInputPeer *peer, const QString &filePath, const QString &thumbnailFilePath) {
    return m_telegram->messagesSendDocument(*peer, filePath, thumbnailFilePath);
}

qint64 RawApiClient::messagesForwardPhoto(TLInputPeer *peer, qint64 photoId, qint64 accessHash) {
    return m_telegram->messagesForwardPhoto(*peer, photoId, accessHash);
}

qint64 RawApiClient::messagesForwardVideo(TLInputPeer *peer, qint64 videoId, qint64 accessHash) {
    return m_telegram->messagesForwardVideo(*peer, videoId, accessHash);
}

qint64 RawApiClient::messagesForwardAudio(TLInputPeer *peer, qint64 audioId, qint64 accessHash) {
    return m_telegram->messagesForwardAudio(*peer, audioId, accessHash);
}

qint64 RawApiClient::messagesForwardDocument(TLInputPeer *peer, qint64 documentId, qint64 accessHash) {
    return m_telegram->messagesForwardDocument(*peer, documentId, accessHash);
}

qint64 RawApiClient::messagesSetTyping(TLInputPeer *peer, bool typing) {
    return m_telegram->messagesSetTyping(*peer, typing);
}

qint64 RawApiClient::messagesGetMessages(const QList<qint32> &msgIds) {
    return m_telegram->messagesGetMessages(msgIds);
}

qint64 RawApiClient::messagesGetDialogs(qint32 offset, qint32 maxId, qint32 limit) {
    return m_telegram->messagesGetDialogs(offset, maxId, limit);
}

qint64 RawApiClient::messagesGetHistory(TLInputPeer *peer, qint32 offset, qint32 maxId, qint32 limit) {
    return m_telegram->messagesGetHistory(*peer, offset, maxId, limit);
}

qint64 RawApiClient::messagesSearch(TLInputPeer *peer, const QString &query, TLMessagesFilter *filter, qint32 minDate, qint32 maxDate, qint32 offset, qint32 maxId, qint32 limit) {
    return m_telegram->messagesSearch(*peer, query, *filter, minDate, maxDate, offset, maxId, limit);
}

qint64 RawApiClient::messagesReadHistory(TLInputPeer *peer, qint32 maxId, qint32 offset) {
    return m_telegram->messagesReadHistory(*peer, maxId, offset);
}

qint64 RawApiClient::messagesDeleteHistory(TLInputPeer *peer, qint32 offset) {
    return m_telegram->messagesDeleteHistory(*peer, offset);
}

qint64 RawApiClient::messagesDeleteMessages(const QList<qint32> &msgIds) {
    return m_telegram->messagesDeleteMessages(msgIds);
}

qint64 RawApiClient::messagesRestoreMessages(const QList<qint32> &msgIds) {
    return m_telegram->messagesRestoreMessages(msgIds);
}

qint64 RawApiClient::messagesReceivedMessages(qint32 maxId) {
    return m_telegram->messagesReceivedMessages(maxId);
}

qint64 RawApiClient::messagesForwardMessage(TLInputPeer *peer, qint32 msgId) {
    return m_telegram->messagesForwardMessage(*peer, msgId);
}

qint64 RawApiClient::messagesForwardMessages(TLInputPeer *peer, const QList<qint32> &msgIds) {
    return m_telegram->messagesForwardMessages(*peer, msgIds);
}

// Working with users - received signals
void RawApiClient::onUsersGetUsersAnswer(qint64 id, const QList<User> &users) {
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT usersGetUsersAnswer(id, usersList);
}

void RawApiClient::onUsersGetFullUserAnswer(qint64 id, const User &user, const ContactsLink &link, const Photo &profilePhoto, const PeerNotifySettings &notifySettings, bool blocked, const QString &realFirstName, const QString &realLastName) {
    TLUser *tlUser = new TLUser(this);
    tlUser->setData(&user);
    TLContactsLink *tlLink = new TLContactsLink(this);
    tlLink->setData(&link);
    TLPhoto *tlProfilePhoto = new TLPhoto(this);
    tlProfilePhoto->setData(&profilePhoto);
    TLPeerNotifySettings *tlNotifySettings = new TLPeerNotifySettings(this);
    tlNotifySettings->setData(&notifySettings);
    Q_EMIT usersGetFullUserAnswer(id, tlUser, tlLink, tlProfilePhoto, tlNotifySettings, blocked, realFirstName, realLastName);
}

void RawApiClient::onPhotosGetUserPhotosAnswer(qint64 id, qint32 sliceCount, const QList<Photo> &photos, const QList<User> &users) {
    QVariantList photosList = toVariantList<Photo, TLPhoto>(photos);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT photosGetUserPhotosAnswer(id, sliceCount, photosList, usersList);
}

// Working with contacts - received signals
void RawApiClient::onContactsGetStatusesAnswer(qint64 id, const QList<ContactStatus> &statuses) {
    QVariantList statusesList = toVariantList<ContactStatus, TLContactStatus>(statuses);
    Q_EMIT contactsGetStatusesAnswer(id, statusesList);
}

void RawApiClient::onContactsGetContactsAnswer(qint64 id, bool modified, const QList<Contact> &contacts, const QList<User> &users) {
    QVariantList contactsList = toVariantList<Contact, TLContact>(contacts);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT contactsGetContactsAnswer(id, modified, contactsList, usersList);
}

void RawApiClient::onContactsImportContactsAnswer(qint64 id, const QList<ImportedContact> &importedContacts, const QList<qint64> &retryContacts, const QList<User> &users) {
    QVariantList importedContactsList = toVariantList<ImportedContact, TLImportedContact>(importedContacts);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT contactsImportContactsAnswer(id, importedContactsList, retryContacts, usersList);
}

void RawApiClient::onContactsDeleteContactAnswer(qint64 id, const ContactsMyLink &myLink, const ContactsForeignLink &foreignLink, const User &user) {
    TLContactsMyLink *tlMyLink = new TLContactsMyLink(this);
    tlMyLink->setData(&myLink);
    TLContactsForeignLink *tlForeignLink = new TLContactsForeignLink(this);
    tlForeignLink->setData(&foreignLink);
    TLUser *tlUser = new TLUser(this);
    tlUser->setData(&user);
    Q_EMIT contactsDeleteContactAnswer(id, tlMyLink, tlForeignLink, tlUser);
}

// Working with blacklist - received signals
void RawApiClient::onContactsGetBlockedAnswer(qint64 id, qint32 sliceCount, QList<ContactBlocked> blocked, QList<User> users) {
    QVariantList blockedList = toVariantList<ContactBlocked, TLContactBlocked>(blocked);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT contactsGetBlockedAnswer(id, sliceCount, blockedList, usersList);
}

// Working with messages - received signals
void RawApiClient::onMessagesSendMessageAnswer(qint64 id, qint32 msgId, qint32 date, qint32 pts, qint32 seq, const QList<ContactsLink> &links) {
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    Q_EMIT messagesSendMessageAnswer(id, msgId, date, pts, seq, linksList);
}

void RawApiClient::onMessagesSendMediaAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendMediaAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesSendPhotoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendPhotoAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesSendGeoPointAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendGeoPointAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesSendContactAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendContactAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesSendVideoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendVideoAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesSendAudioAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendAudioAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesSendDocumentAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendDocumentAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesGetMessagesAnswer(qint64 id, qint32 sliceCount, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users) {
    QVariantList messagesList = toVariantList<Message, TLMessage>(messages);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesGetMessagesAnswer(id, sliceCount, messagesList, chatsList, usersList);
}

void RawApiClient::onMessagesGetDialogsAnswer(qint64 id, qint32 sliceCount, const QList<Dialog> &dialogs, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users) {
    // emit single message of received answer to get dialogs operation
    QVariantList dialogsList = toVariantList<Dialog, TLDialog>(dialogs);
    QVariantList messagesList = toVariantList<Message, TLMessage>(messages);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesGetDialogsAnswer(id, sliceCount, dialogsList, messagesList, chatsList, usersList);
}

void RawApiClient::onMessagesGetHistoryAnswer(qint64 id, qint32 sliceCount, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users) {
    QVariantList messagesList = toVariantList<Message, TLMessage>(messages);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesGetHistoryAnswer(id, sliceCount, messagesList, chatsList, usersList);
}

void RawApiClient::onMessagesSearchAnswer(qint64 id, qint32 sliceCount, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users) {
    QVariantList messagesList = toVariantList<Message, TLMessage>(messages);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSearchAnswer(id, sliceCount, messagesList, chatsList, usersList);
}

void RawApiClient::onMessagesForwardMessageAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesForwardMessageAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesForwardMessagesAnswer(qint64 id, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    QVariantList messagesList = toVariantList<Message, TLMessage>(messages);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesForwardMessagesAnswer(id, messagesList, chatsList, usersList, linksList, pts, seq);
}

// Working with updates - received signals
void RawApiClient::onUpdatesGetDifferenceAnswer(qint64 id, const QList<Message> &messages, const QList<EncryptedMessage> &newEncryptedMessages, const QList<Update> &otherUpdates, const QList<Chat> &chats, const QList<User> &users, const UpdatesState &state, bool isIntermediateState) {
    QVariantList messagesList = toVariantList<Message, TLMessage>(messages);
    QVariantList newEncryptedMessagesList = toVariantList<EncryptedMessage, TLEncryptedMessage>(newEncryptedMessages);
    QVariantList otherUpdatesList = toVariantList<Update, TLUpdate>(otherUpdates);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    TLUpdatesState *tlUpdatesState = new TLUpdatesState(this);
    tlUpdatesState->setData(&state);
    Q_EMIT updatesGetDifferenceAnswer(id, messagesList, newEncryptedMessagesList, otherUpdatesList, chatsList, usersList, tlUpdatesState, isIntermediateState);
}


// Working with files - received signals
void RawApiClient::onUploadGetFileAnswer(qint64 id, const StorageFileType &type, qint32 mtime, QByteArray bytes, qint32 partId, qint32 downloaded, qint32 total) {
    TLStorageFileType *tlType = new TLStorageFileType(this);
    tlType->setData(&type);
    Q_EMIT uploadGetFileAnswer(id, tlType, mtime, bytes, partId, downloaded, total);
}


void RawApiClient::onUpdateShort(const Update &update, qint32 date) {
    TLUpdate *tlUpdate = new TLUpdate(this);
    tlUpdate->setData(&update);
    Q_EMIT updateShort(tlUpdate, date);
}

void RawApiClient::onUpdatesCombined(const QList<Update> &updates, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seqStart, qint32 seq) {
}

void RawApiClient::onUpdates(const QList<Update> &udts, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seq) {
}

// private methods
template <typename TOrig, typename TDest>
QVariantList RawApiClient::toVariantList(const QList<TOrig> &list) {
    QVariantList newList;
    int id = qMetaTypeId<TDest>();
    Q_ASSERT(id);
    Q_FOREACH(const TOrig &item, list) {
        TDest *typeObject = (TDest *)QMetaType::create(id);
        TLObject *tlObject = (TLObject *)typeObject;
        tlObject->setData(&item);
        newList.append(qVariantFromValue(typeObject));
    }
    return newList;
}

template <typename T>
QList<T> RawApiClient::toQList(const QVariantList &variantList) {
    QList<T> tList;
    for (int i = 0; i < variantList.size(); i++) {
        T *element = dynamic_cast<T *>(variantList.at(i).value<QObject *>());
        tList.append(*element);
    }
    return tList;
}

