#ifndef RAWAPICLIENT_H
#define RAWAPICLIENT_H

#include <QVariantList>
#include <QObject>
#include <QString>
#include <QSortFilterProxyModel>
#include <telegram.h>
#include <types/types.h>
#include "config.h"
#include "types/tlaudio.h"
#include "types/tlchat.h"
#include "types/tlchatparticipant.h"
#include "types/tlchatparticipants.h"
#include "types/tlchatphoto.h"
#include "types/tlcontact.h"
#include "types/tlcontactblocked.h"
#include "types/tlcontactsforeignlink.h"
#include "types/tlcontactslink.h"
#include "types/tlcontactsmylink.h"
#include "types/tlcontactstatus.h"
#include "types/tldcoption.h"
#include "types/tldialog.h"
#include "types/tldocument.h"
#include "types/tlencryptedchat.h"
#include "types/tlencryptedfile.h"
#include "types/tlencryptedmessage.h"
#include "types/tlfilelocation.h"
#include "types/tlgeochatmessage.h"
#include "types/tlgeopoint.h"
#include "types/tlimportedcontact.h"
#include "types/tlinputfilelocation.h"
#include "types/tlinputgeopoint.h"
#include "types/tlinputpeer.h"
#include "types/tlinputuser.h"
#include "types/tlmessage.h"
#include "types/tlmessageaction.h"
#include "types/tlmessagemedia.h"
#include "types/tlmessagesfilter.h"
#include "types/tlnotifypeer.h"
#include "types/tlpeer.h"
#include "types/tlpeernotifysettings.h"
#include "types/tlphoto.h"
#include "types/tlphotosize.h"
#include "types/tlstoragefiletype.h"
#include "types/tlupdate.h"
#include "types/tlupdatesstate.h"
#include "types/tluserprofilephoto.h"
#include "types/tluserstatus.h"
#include "types/tluser.h"
#include "types/tlvideo.h"

class RawApiClient : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QString phoneNumber READ phoneNumber WRITE setPhoneNumber)
    Q_PROPERTY(QString configPath READ configPath WRITE setConfigPath)
    Q_PROPERTY(QString publicKeyFile READ publicKeyFile WRITE setPublicKeyFile)

public:
    explicit RawApiClient(QObject *parent = 0);
    ~RawApiClient();

    static Telegram *libObject();

    Q_INVOKABLE void initTelegramLibrary();
    // Registration / authorization
    Q_INVOKABLE qint64 authCheckPhone();
    Q_INVOKABLE qint64 authSendCode();
    Q_INVOKABLE qint64 authSignIn(const QString &code);
    Q_INVOKABLE qint64 authSignUp(const QString &code, const QString &firstName, const QString &lastName);
    Q_INVOKABLE qint64 authSendCall();
    Q_INVOKABLE qint64 authLogOut();
    Q_INVOKABLE qint64 authSendInvites(const QStringList &phoneNumbers, const QString &inviteText);
    Q_INVOKABLE qint64 authResetAuthorizations();
    // Working with users
    Q_INVOKABLE qint64 usersGetUsers (QVariantList users);
    Q_INVOKABLE qint64 usersGetFullUser (TLInputUser *user);
    Q_INVOKABLE qint64 photosGetUserPhotos (TLInputUser *user, qint32 offset = 0, qint32 maxId = 0, qint32 limit = 0);
    // Working with contacts
    Q_INVOKABLE qint64 contactsGetStatuses();
    Q_INVOKABLE qint64 contactsGetContacts();
    Q_INVOKABLE qint64 contactsImportContacts (QVariantList contacts, bool replace);
    Q_INVOKABLE qint64 contactsDeleteContact(TLInputUser *user);
    Q_INVOKABLE qint64 contactsDeleteContacts(QVariantList users);
    // Working with blacklist
    Q_INVOKABLE qint64 contactsBlock(InputUser *user);
    Q_INVOKABLE qint64 contactsUnblock(InputUser *user);
    Q_INVOKABLE qint64 contactsGetBlocked(qint32 offset = 0, qint32 limit = 0);
    // Working with messages
    Q_INVOKABLE qint64 messagesSendMessage (TLInputPeer *peer, const QString &message);
    Q_INVOKABLE qint64 messagesSendPhoto(TLInputPeer *peer, const QByteArray &bytes, const QString &fileName);
    Q_INVOKABLE qint64 messagesSendPhoto(TLInputPeer *peer, const QString &filePath);
    Q_INVOKABLE qint64 messagesSendGeoPoint(TLInputPeer *peer, InputGeoPoint *inputGeoPoint);
    Q_INVOKABLE qint64 messagesSendContact(TLInputPeer *peer, const QString &phoneNumber, const QString &firstName, const QString &lastName);
    Q_INVOKABLE qint64 messagesSendVideo(TLInputPeer *peer, const QByteArray &bytes, const QString &fileName, qint32 duration, qint32 width, qint32 height, const QString &mimeType, const QByteArray &thumbnailBytes = 0, const QString &thumbnailName = "");
    Q_INVOKABLE qint64 messagesSendVideo(TLInputPeer *peer, const QString &filePath, qint32 duration, qint32 width, qint32 height, const QString &thumbnailFilePath = "");
    Q_INVOKABLE qint64 messagesSendAudio(TLInputPeer *peer, const QByteArray &bytes, const QString &fileName, qint32 duration, const QString &mimeType);
    Q_INVOKABLE qint64 messagesSendAudio(TLInputPeer *peer, const QString &filePath, qint32 duration);
    Q_INVOKABLE qint64 messagesSendDocument(TLInputPeer *peer, const QByteArray &bytes, const QString &fileName, const QString &mimeType, const QByteArray &thumbnailBytes = 0, const QString &thumbnailName = "");
    Q_INVOKABLE qint64 messagesSendDocument(TLInputPeer *peer, const QString &filePath, const QString &thumbnailFilePath = "");
    Q_INVOKABLE qint64 messagesForwardPhoto(TLInputPeer *peer, qint64 photoId, qint64 accessHash);
    Q_INVOKABLE qint64 messagesForwardVideo(TLInputPeer *peer, qint64 videoId, qint64 accessHash);
    Q_INVOKABLE qint64 messagesForwardAudio(TLInputPeer *peer, qint64 audioId, qint64 accessHash);
    Q_INVOKABLE qint64 messagesForwardDocument(TLInputPeer *peer, qint64 documentId, qint64 accessHash);
    Q_INVOKABLE qint64 messagesSetTyping(TLInputPeer *peer, bool typing);
    Q_INVOKABLE qint64 messagesGetMessages(const QList<qint32> &msgIds);
    Q_INVOKABLE qint64 messagesGetDialogs(qint32 offset = 0, qint32 maxId = 0, qint32 limit = 0);
    Q_INVOKABLE qint64 messagesGetHistory(TLInputPeer *peer, qint32 offset = 0, qint32 maxId = 0, qint32 limit = 0);
    Q_INVOKABLE qint64 messagesSearch(TLInputPeer *peer, const QString &query, TLMessagesFilter *filter, qint32 minDate = 0, qint32 maxDate = 0, qint32 offset = 0, qint32 maxId = 0, qint32 limit = 0);
    Q_INVOKABLE qint64 messagesReadHistory(TLInputPeer *peer, qint32 maxId = 0, qint32 offset = 0);
    Q_INVOKABLE qint64 messagesDeleteHistory(TLInputPeer *peer, qint32 offset = 0);
    Q_INVOKABLE qint64 messagesDeleteMessages(const QList<qint32> &msgIds);
    Q_INVOKABLE qint64 messagesRestoreMessages(const QList<qint32> &msgIds);
    Q_INVOKABLE qint64 messagesReceivedMessages(qint32 maxId);
    Q_INVOKABLE qint64 messagesForwardMessage(TLInputPeer *peer, qint32 msgId);
    Q_INVOKABLE qint64 messagesForwardMessages(TLInputPeer *peer, const QList<qint32> &msgIds);
    // Working with updates
    Q_INVOKABLE qint64 updatesGetState();
    Q_INVOKABLE qint64 updatesGetDifference(qint32 pts, qint32 date, qint32 qts);
    // Working with files
    Q_INVOKABLE qint64 uploadGetFile(TLInputFileLocation *file, qint32 fileSize, qint32 dc = 0);

    QString phoneNumber() { return Config::instance()->phoneNumber(); }
    QString configPath() { return Config::instance()->configPath(); }
    QString publicKeyFile() { return Config::instance()->publicKeyFile(); }

    void setPhoneNumber(const QString &phoneNumber) { Config::instance()->setPhoneNumber(phoneNumber); }
    void setConfigPath(const QString &configPath) { Config::instance()->setConfigPath(configPath); }
    void setPublicKeyFile(const QString &publicKeyFile) { Config::instance()->setPublicKeyFile(publicKeyFile); }

Q_SIGNALS:
    // Errors
    void error(qint64 id, qint32 errorCode, QString errorText);
    void authSignInError(qint64 id, qint32 errorCode, QString errorText);
    void authSignUpError(qint64 id, qint32 errorCode, QString errorText);
    // Registration / authorization
    void authNeeded();
    void authLoggedIn();
    void authCheckPhoneAnswer(qint64 id, bool phoneRegistered, bool phoneInvited);
    void authSendCodeAnswer(qint64 id, bool phoneRegistered, qint32 sendCallTimeout);
    void authSendCallAnswer(qint64 id, bool ok);
    void authLogOutAnswer(qint64 id, bool ok);
    void authSendInvitesAnswer(qint64 id, bool ok);
    void authResetAuthorizationsAnswer(qint64 id, bool ok);
    // Working with users
    void usersGetUsersAnswer(qint64 id, QVariantList users);
    void usersGetFullUserAnswer(qint64 id, TLUser *user, TLContactsLink *link, TLPhoto *profilePhoto, TLPeerNotifySettings *notifySettings, bool blocked, QString realFirstName, QString realLastName);
    void photosGetUserPhotosAnswer(qint64 id, qint32 sliceCount, QVariantList photos, QVariantList users);
    // Working with contacts
    void contactsGetStatusesAnswer(qint64 id, QVariantList statuses);
    void contactsGetContactsAnswer(qint64 id, bool modified, QVariantList contacts, QVariantList users);
    void contactsImportContactsAnswer(qint64 id, QVariantList importedContacts, QList<qint64> retryContacts, QVariantList users);
    void contactsDeleteContactAnswer(qint64 id, TLContactsMyLink *myLink, TLContactsForeignLink *foreignLink, TLUser *user);
    void contactsDeleteContactsAnswer(qint64 id, bool ok);
    // Working with blacklist
    void contactsBlockAnswer(qint64 id, bool ok);
    void contactsUnblockAnswer(qint64 id, bool ok);
    void contactsGetBlockedAnswer(qint64 id, qint32 sliceCount, QVariantList blocked, QVariantList users);
    // Working with messages
    void messagesSendMessageAnswer(qint64 id, qint32 msgId, qint32 date, qint32 pts, qint32 seq, QVariantList links);
    void messagesSendMediaAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSendPhotoAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSendGeoPointAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSendContactAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSendVideoAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSendAudioAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSendDocumentAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSetTypingAnswer(qint64 id, bool ok);
    void messagesGetMessagesAnswer(qint64 id, qint32 sliceCount, QVariantList messages, QVariantList chats, QVariantList users);
    void messagesGetDialogsAnswer(qint64 id, qint32 sliceCount, QVariantList dialogs, QVariantList messages, QVariantList chats, QVariantList users);
    void messagesGetHistoryAnswer(qint64 id, qint32 sliceCount, QVariantList messages, QVariantList chats, QVariantList users);
    void messagesSearchAnswer(qint64 id, qint32 sliceCount, QVariantList messages, QVariantList chats, QVariantList users);
    void messagesReadHistoryAnswer(qint64 id, qint32 pts, qint32 seq, qint32 offset);
    void messagesDeleteHistoryAnswer(qint64 id, qint32 pts, qint32 seq, qint32 offset);
    void messagesDeleteMessagesAnswer(qint64 id, QList<qint32> deletedMsgIds);
    void messagesRestoreMessagesAnswer(qint64 id, QList<qint32> restoredMsgIds);
    void messagesReceivedMessagesAnswer(qint64 id, QList<qint32> confirmedMsgIds);
    void messagesForwardMessageAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesForwardMessagesAnswer(qint64 id, QVariantList messages, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    // Working with updates
    void updatesGetStateAnswer(qint64 id, qint32 pts, qint32 qts, qint32 date, qint32 seq, qint32 unreadCount);
    void updatesGetDifferenceAnswer(qint64 id, qint32 date, qint32 seq);
    void updatesGetDifferenceAnswer(qint64 id, QVariantList messages, QVariantList newEncryptedMessages, QVariantList otherUpdates, QVariantList chats, QVariantList users, TLUpdatesState *state, bool isIntermediateState);
    // Working with files
    void uploadGetFileAnswer(qint64 id, TLStorageFileType *type, qint32 mtime, QByteArray bytes, qint32 partId, qint32 downloaded, qint32 total);
    // Updates
    void updatesTooLong();
    void updateShortMessage(qint32 id, qint32 fromId, QString message, qint32 pts, qint32 date, qint32 seq);
    void updateShortChatMessage(qint32 id, qint32 fromId, qint32 chatId, QString message, qint32 pts, qint32 date, qint32 seq);
    void updateShort(TLUpdate *update, qint32 date);
    void updatesCombined(QVariantList updates, QVariantList users, QVariantList chats, qint32 date, qint32 seqStart, qint32 seq);
    void updates(QVariantList udts, QVariantList users, QVariantList chats, qint32 date, qint32 seq);

private:

    /**
     * Converts signal received QList<T> data into QVariantList to be used from qml
     */
    template <typename TOrig, typename TDest>
    static QVariantList toVariantList(const QList<TOrig> &list);

    /**
     * Converts QVariantList to QList
     */
    template <typename T>
    QList<T> toQList(const QVariantList &variantList);

protected:
    static Telegram *m_telegram;

protected Q_SLOTS:
    // Working with users
    void onUsersGetUsersAnswer(qint64 id, const QList<User> &users);
    void onUsersGetFullUserAnswer(qint64 id, const User &user, const ContactsLink &link, const Photo &profilePhoto, const PeerNotifySettings &notifySettings, bool blocked, const QString &realFirstName, const QString &realLastName);
    void onPhotosGetUserPhotosAnswer(qint64 id, qint32 sliceCount, const QList<Photo> &photos, const QList<User> &users);
    // Working with contacts
    void onContactsGetStatusesAnswer(qint64 id, const QList<ContactStatus> &statuses);
    void onContactsGetContactsAnswer(qint64 id, bool modified, const QList<Contact> &contacts, const QList<User> &users);
    void onContactsImportContactsAnswer(qint64 id, const QList<ImportedContact> &importedContacts, const QList<qint64> &retryContacts, const QList<User> &users);
    void onContactsDeleteContactAnswer(qint64 id, const ContactsMyLink &myLink, const ContactsForeignLink &foreignLink, const User &user);
    // Working with blacklist
    void onContactsGetBlockedAnswer(qint64 id, qint32 sliceCount, QList<ContactBlocked> blocked, QList<User> users);
    // Working with messages
    void onMessagesSendMessageAnswer(qint64 id, qint32 msgId, qint32 date, qint32 pts, qint32 seq, const QList<ContactsLink> &links);
    void onMessagesSendMediaAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendPhotoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendGeoPointAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendContactAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendVideoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendAudioAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendDocumentAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesGetMessagesAnswer(qint64 id, qint32 sliceCount, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesGetDialogsAnswer(qint64 id, qint32 sliceCount, const QList<Dialog> &dialogs, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesGetHistoryAnswer(qint64 id, qint32 sliceCount, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesSearchAnswer(qint64 id, qint32 sliceCount, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesForwardMessageAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesForwardMessagesAnswer(qint64 id, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendBroadcastAnswer(qint64 id, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    // Working with updates
    void onUpdatesGetDifferenceAnswer(qint64 id, const QList<Message> &messages, const QList<EncryptedMessage> &newEncryptedMessages, const QList<Update> &otherUpdates, const QList<Chat> &chats, const QList<User> &users, const UpdatesState &state, bool isIntermediateState);
    // Working with files
    void onUploadGetFileAnswer(qint64 id, const StorageFileType &type, qint32 mtime, QByteArray bytes, qint32 partId, qint32 downloaded, qint32 total);
    // Updates
    void onUpdateShort(const Update &update, qint32 date);
    void onUpdatesCombined(const QList<Update> &updates, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seqStart, qint32 seq);
    void onUpdates(const QList<Update> &udts, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seq);
};

#endif // RAWAPICLIENT_H
