#include "telegramclient.h"
#include <QDir>
#include "models/dialogitem.h"

#include "modelsmanager.h"
#include "data.h"
#include "dbmanager.h"

Q_LOGGING_CATEGORY(TG_PLUGIN_LOGIC, "tg.plugin.logic")

TelegramClient::TelegramClient(QObject *parent) :
    RawApiClient(parent) {
    // build dialogs model and proxy
    m_dialogsModelProxy.setSourceModel(&ModelsManager::instance()->dialogsModel());
    m_dialogsModelProxy.setSortRole(DialogsModel::TopMessageRole);
    m_dialogsModelProxy.sort(0, Qt::DescendingOrder);
    m_dialogsModelProxy.setDynamicSortFilter(true);
    // build contacts model and proxy
    m_contactsModelProxy.setSourceModel(&ModelsManager::instance()->contactsModel());
    m_contactsModelProxy.setSortRole(ContactsModel::FirstNameRole);
    m_contactsModelProxy.sort(0, Qt::AscendingOrder);
    m_contactsModelProxy.setDynamicSortFilter(true);
}

TelegramClient::~TelegramClient() {
}

void TelegramClient::initTelegramLibrary() {
    RawApiClient::initTelegramLibrary();

    if (m_telegram) {
        Settings::getInstance()->setOurId(m_telegram->ourId());
    }

    // when initialized library object, initialize database
    DbManager::instance()->initializeDatabase();

    // connect lib object server received rpc messages with data instance
    connect(m_telegram,
            SIGNAL(messagesGetDialogsAnswer(qint64,qint32,QList<Dialog>,QList<Message>,QList<Chat>,QList<User>)),
            Data::instance(),
            SLOT(onMessagesGetDialogsAnswer(qint64,qint32,QList<Dialog>,QList<Message>,QList<Chat>,QList<User>)));



    // connect state out of sync to slot for query differences
    connect(&m_state, SIGNAL(outOfSync()), this, SLOT(onOutOfSync()));
    // Updates signal-slot
    connect(m_telegram, SIGNAL(updatesGetStateAnswer(qint64,qint32,qint32,qint32,qint32,qint32)), this, SLOT(onUpdatesGetStateAnswer(qint64,qint32,qint32,qint32,qint32,qint32)));
    connect(m_telegram, SIGNAL(updatesGetDifferenceAnswer(qint64,qint32,qint32)), this, SLOT(onUpdatesGetDifferenceAnswer(qint64,qint32,qint32)));
    // Updates
    connect(m_telegram, SIGNAL(updateShortMessage(qint32,qint32,QString,qint32,qint32,qint32)), this, SLOT(onUpdateShortMessage(qint32,qint32,QString,qint32,qint32,qint32)));
    connect(m_telegram, SIGNAL(updateShortChatMessage(qint32,qint32,qint32,QString,qint32,qint32,qint32)), this, SLOT(onUpdateShortChatMessage(qint32,qint32,qint32,QString,qint32,qint32,qint32)));
}

qint64 TelegramClient::messagesSendMessage(TLInputPeer *peer, const QString &message) {
   qint64 requestId = RawApiClient::messagesSendMessage(peer, message);
   qint32 dialogsItemId = peer->userId();
   if (peer->classType() == TLInputPeer::typeInputPeerChat) {
       dialogsItemId = peer->chatId();
   }
   ModelsManager::instance()->dialogsModel().updateSendingTopMessage(dialogsItemId, requestId, message);
   return requestId;
}

void TelegramClient::onMessagesSendMessageAnswer(qint64 requestId, qint32 msgId, qint32 date, qint32 pts, qint32 seq, const QList<ContactsLink> &links) {
    RawApiClient::onMessagesSendMessageAnswer(requestId, msgId, date, pts, seq, links);
    m_state.setDate(date);
    m_state.setPts(pts);
    m_state.setSeq(seq);
    ModelsManager::instance()->dialogsModel().updateSentTopMessage(requestId, msgId, date);
}


//void TelegramClient::onMessagesGetDialogsAnswer(qint64 id, qint32 sliceCount, const QList<Dialog> &dialogs, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users) {
//    RawApiClient::onMessagesGetDialogsAnswer(id, sliceCount, dialogs, messages, chats, users);

//    // fill the dialogs model with received data model

//    // fill the contacts model with the mentioned users in dialogs (if they already exists in model, are updated)
//    Q_FOREACH(User user, users) {
//        ContactItem *contact = new ContactItem(this);
//        User *u = new User(user.classType());
//        *u = user;
//        contact->setData(u);
//        ModelsManager::instance()->contactsModel().append(contact);
//    }
//    QMap<qint32, const Chat*> mentionedChats;
//    for (qint32 i=0; i<chats.size(); i++) {
//        mentionedChats.insert(chats.at(i).id(), &(chats.at(i)));
//    }
//    ModelsManager::instance()->dialogsModel().clear();
//    for (qint32 i = 0; i < dialogs.size(); ++i) {
//        DialogItem *item = new DialogItem(this);
//        Dialog thisdialog = dialogs.at(i);
//        Message thismessage = messages.at(i);
//        Peer peer = thisdialog.peer();
//        FileLocation dialogPhoto(FileLocation::typeFileLocationUnavailable);
//        if (peer.classType() == Peer::typePeerChat) {
//            const Chat *c = mentionedChats.value(peer.chatId());
//            item->setIsChat(true);
//            item->setId(c->id());
//            if (c->classType() != Chat::typeChatEmpty) {
//                item->setName(c->title());
//                if (c->classType() != Chat::typeChatForbidden) {
//                    dialogPhoto = c->photo().photoSmall();
//                }
//            }
//        } else if (peer.classType() == Peer::typePeerUser) {
//            ContactItem *contact = ModelsManager::instance()->contactsModel().contact(peer.userId());
//            item->setIsChat(false);
//            item->setId(contact->id());
//            item->setName(contact->firstName() + " " + contact->lastName());
//            dialogPhoto = *contact->photo()->photoSmall();
//        }
//        // retrieve user or chat photo asynchronously
//        if (dialogPhoto.classType() != FileLocation::typeFileLocationUnavailable) {
//            InputFileLocation f(InputFileLocation::typeInputFileLocation);
//            f.setLocalId(dialogPhoto.localId());
//            f.setSecret(dialogPhoto.secret());
//            f.setVolumeId(dialogPhoto.volumeId());
//            qint64 fileId = m_telegram->uploadGetFile(f, 0, dialogPhoto.dcId());
//            m_photoFiles.insert(fileId, item->id());
//        }
//        item->setUnreadCount(thisdialog.unreadCount());
//        // if not exists top message object, create it
//        if (!item->topMessage()) {
//            item->createTopMessage();
//        }
//        // fill topmessage with received data
//        item->topMessage()->setId(thismessage.id());
//        item->topMessage()->setMessage(thismessage.message());
//        item->topMessage()->setMedia(thismessage.media());
//        item->topMessage()->setUnread(thismessage.unread());
//        item->topMessage()->setFromId(thismessage.fromId());
//        item->topMessage()->setDate(thismessage.date());
//        ModelsManager::instance()->dialogsModel().append(item);
//    }
//}

void TelegramClient::onUploadGetFileAnswer(qint64 fileId, const StorageFileType &type, qint32 mtime, QByteArray bytes, qint32 partId, qint32 downloaded, qint32 total) {
    RawApiClient::onUploadGetFileAnswer(fileId, type, mtime, bytes, partId, downloaded, total);

    // check if received file id is one of the pending ones
    qint32 dialogItemId = m_photoFiles.take(fileId);
    if (dialogItemId) {

        if (type.classType() == StorageFileType::typeStorageFilePartial) {
            qCWarning(TG_PLUGIN_LOGIC) << "Received profile photo in several parts. Only single parts profile photos are managed";
            return;
        }

        Q_ASSERT(partId == 0);

        // calculate name extension
        QString extension;
        switch (type.classType()) {
        case StorageFileType::typeStorageFileGif: extension = ".gif";
            break;
        case StorageFileType::typeStorageFileJpeg: extension = ".jpg";
            break;
        case StorageFileType::typeStorageFilePng: extension = ".png";
            break;
        default:
            //if not a image, don't continue
            return;
        }

        // save the file if not exists in folder
        QString fileName = QString::number(dialogItemId) + "_" + QString::number(mtime) + extension;
        if (m_downloadsPath.isEmpty()) {
            QString configPath = (this->configPath()).replace("~", QDir::homePath());
            m_downloadsPath = configPath + "/" + phoneNumber() + "/" + DOWNLOADS_DIRECTORY;
        }
        QString filePath = m_downloadsPath + "/" + fileName;
        QFile f(filePath);
        if (!f.exists()) {

            //remove any previous profile file found at this path for this id
            QDir dir(m_downloadsPath);
            QStringList filters;
            filters << QString::number(dialogItemId) + "_*";
            Q_FOREACH (QString fileNameToDelete, dir.entryList(filters, QDir::Files)) {
                QFile fToDelete(m_downloadsPath + "/" + fileNameToDelete);
                if (fToDelete.remove()) {
                    qCDebug(TG_PLUGIN_LOGIC) << "removed previous profile photo at " << fToDelete.fileName();
                }
            }
            // store the new profile photo
            if (!f.open(QIODevice::WriteOnly)) {
                return;
            }
            f.write(bytes);
            f.close();
        }

        // update thumbnail in dialog and in contacts if there is a contact with received id
        ModelsManager::instance()->dialogsModel().updateThumbnail(dialogItemId, filePath);
        ModelsManager::instance()->contactsModel().updateThumbnail(dialogItemId, filePath);
    }
}

// Updates - received signals
void TelegramClient::onUpdateShortMessage(qint32 id, qint32 fromId, const QString &message, qint32 pts, qint32 date, qint32 seq) {
    qCDebug(TG_PLUGIN_LOGIC) << "updateShortMessage received";
    m_state.setDate(date);
    m_state.setPts(pts);
    m_state.setSeq(seq);
    ModelsManager::instance()->dialogsModel().updateReceivedTopMessage(fromId, id, message, date, fromId);
}

void TelegramClient::onUpdateShortChatMessage(qint32 id, qint32 fromId, qint32 chatId, const QString &message, qint32 pts, qint32 date, qint32 seq) {
    qCDebug(TG_PLUGIN_LOGIC) << "updateShortChatMessage received";
    ModelsManager::instance()->dialogsModel().updateReceivedTopMessage(chatId, id, message, date, fromId);
}

void TelegramClient::onUpdateShort(const Update &update, qint32 date) {
    RawApiClient::onUpdateShort(update, date);
    qCDebug(TG_PLUGIN_LOGIC) << "updateShort received";
    m_state.setDate(date);
    processUpdate(update);
}

void TelegramClient::onUpdatesCombined(const QList<Update> &updates, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seqStart, qint32 seq) {
    RawApiClient::onUpdatesCombined(updates, users, chats, date, seqStart, seq);
    qCDebug(TG_PLUGIN_LOGIC) << "updatesCombined received";
}

void TelegramClient::onUpdates(const QList<Update> &udts, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seq) {
    RawApiClient::onUpdates(udts, users, chats, date, seq);
    qCDebug(TG_PLUGIN_LOGIC) << "updates received";
    m_state.setDate(date);
    m_state.setSeq(seq);
    Q_FOREACH (Update udt, udts) {
        processUpdate(udt, users, chats);
    }
}

void TelegramClient::processUpdate(const Update &update, const QList<User> &users, const QList<Chat> &chats) {
    //TODO COMPLETE UPDATES
    switch (update.classType()) {
        case Update::typeUpdateReadMessages: {
            Q_FOREACH (qint32 id, update.messages()) {
                ModelsManager::instance()->dialogsModel().markAsRead(id);
            }
            break;
        }
        case Update::typeUpdateUserTyping: {
            ModelsManager::instance()->dialogsModel().setIsTyping(update.userId(), update.userId());
            break;
        }
        case Update::typeUpdateChatUserTyping: {
            ModelsManager::instance()->dialogsModel().setIsTyping(update.chatId(), update.userId());
            break;
        }
    }
}

void TelegramClient::onOutOfSync() {
    m_telegram->updatesGetDifference(m_state.pts(), m_state.date(), m_state.qts());
}

void TelegramClient::onUpdatesGetStateAnswer(qint64 id, qint32 pts, qint32 qts, qint32 date, qint32 seq, qint32 unreadCount) {
    //TODO Implement
}

void TelegramClient::onUpdatesGetDifferenceAnswer(qint64 id, qint32 date, qint32 seq) {
    //TODO Implement
}

void TelegramClient::onUpdatesGetDifferenceAnswer(qint64 id, const QList<Message> &messages, const QList<EncryptedMessage> &newEncryptedMessages, const QList<Update> &otherUpdates, const QList<Chat> &chats, const QList<User> &users, const UpdatesState &state, bool isIntermediateState) {
    RawApiClient::onUpdatesGetDifferenceAnswer(id, messages, newEncryptedMessages, otherUpdates, chats, users, state, isIntermediateState);
    //TODO Implement
}

