#ifndef TELEGRAMCLIENT_H
#define TELEGRAMCLIENT_H

#include "rawapiclient.h"
#include "clientstate.h"
#include "config.h"
#include "models/dialogsproxy.h"
#include <qsortfilterproxymodel.h>

#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(TG_PLUGIN_LOGIC)

class TelegramClient : public RawApiClient
{
    Q_OBJECT
    Q_PROPERTY(DialogsProxy *dialogsModel READ dialogsModel NOTIFY dialogsModelChanged)
    Q_PROPERTY(QSortFilterProxyModel *contactsModel READ contactsModel NOTIFY contactsModelChanged)
    Q_PROPERTY(qint32 ourId READ ourId NOTIFY ourIdChanged)
    Q_PROPERTY(QString databasePath READ databasePath WRITE setDatabasePath NOTIFY databasePathChanged)
public:
    explicit TelegramClient(QObject *parent = 0);
    ~TelegramClient();

    DialogsProxy *dialogsModel() { return &m_dialogsModelProxy; }
    QSortFilterProxyModel *contactsModel() { return &m_contactsModelProxy; }
    qint32 ourId() { return m_telegram->ourId(); }
    QString databasePath() { return Config::instance()->databasePath(); }
    void setDatabasePath(const QString &path) { Config::instance()->setDatabasePath(path); }


    Q_INVOKABLE void initTelegramLibrary();

    // Working with messages
    Q_INVOKABLE qint64 messagesSendMessage (TLInputPeer *peer, const QString &message);

Q_SIGNALS:
    void dialogsModelChanged(DialogsProxy *);
    void contactsModelChanged(QSortFilterProxyModel *);
    void databasePathChanged(QString);
    void ourIdChanged(qint32);

private:
    // models
    DialogsProxy m_dialogsModelProxy;
    QSortFilterProxyModel m_contactsModelProxy;
    // correlates file id with user or chat id. This is used to assing a profile or chat photo being downloaded to a dialog
    QMap<qint64, qint32> m_photoFiles;
    // path where store donwloaded profile photos
    QString m_downloadsPath;

    // updates state
    ClientState m_state;

    // process notifications to update models
    void processUpdate(const Update &update, const QList<User> &users = QList<User>(), const QList<Chat> &chats =  QList<Chat>());


private Q_SLOTS:
    void onMessagesSendMessageAnswer(qint64 id, qint32 msgId, qint32 date, qint32 pts, qint32 seq, const QList<ContactsLink> &links);
    void onUploadGetFileAnswer(qint64 id, const StorageFileType &type, qint32 mtime, QByteArray bytes, qint32 partId, qint32 downloaded, qint32 total);
    void onUpdatesGetStateAnswer(qint64 id, qint32 pts, qint32 qts, qint32 date, qint32 seq, qint32 unreadCount);
    void onUpdatesGetDifferenceAnswer(qint64 id, qint32 date, qint32 seq);
    void onUpdatesGetDifferenceAnswer(qint64 id, const QList<Message> &messages, const QList<EncryptedMessage> &newEncryptedMessages, const QList<Update> &otherUpdates, const QList<Chat> &chats, const QList<User> &users, const UpdatesState &state, bool isIntermediateState);
    void onUpdateShortMessage(qint32 id, qint32 fromId, const QString &message, qint32 pts, qint32 date, qint32 seq);
    void onUpdateShortChatMessage(qint32 id, qint32 fromId, qint32 chatId, const QString &message, qint32 pts, qint32 date, qint32 seq);
    void onUpdateShort(const Update &update, qint32 date);
    void onUpdatesCombined(const QList<Update> &updates, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seqStart, qint32 seq);
    void onUpdates(const QList<Update> &udts, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seq);


    // slot to get updates difference
    void onOutOfSync();
};

#endif // TELEGRAMCLIENT_H
