#ifndef TLCHAT_H
#define TLCHAT_H

#include "tlobject.h"
#include "tlchatphoto.h"
#include "tlgeopoint.h"
#include <types/chat.h>

class TLChat : public TLObject, public Chat {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLChatType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qint32 participantsCount READ participantsCount)
    Q_PROPERTY(qint32 id READ id)
    Q_PROPERTY(qint32 version READ version)
    Q_PROPERTY(QString venue READ venue)
    Q_PROPERTY(QString title READ title)
    Q_PROPERTY(QString address READ address)
    Q_PROPERTY(qint32 date READ date)
    Q_PROPERTY(qint64 accessHash READ accessHash)
    Q_PROPERTY(bool checkedIn READ checkedIn)
    Q_PROPERTY(bool left READ left)
    Q_PROPERTY(TLChatPhoto *photo READ photo)
    Q_PROPERTY(TLGeoPoint *geo READ geo)
public:
    enum TLChatType {
       TypeChatEmpty = Chat::typeChatEmpty,
       TypeChatForbidden = Chat::typeChatForbidden,
       TypeGeoChat = Chat::typeGeoChat,
       TypeChat = Chat::typeChat
    };

    explicit TLChat(QObject *parent = 0) :
        TLObject(parent),
        Chat(Chat::typeChatEmpty) {
    }
    TLChat(const TLChat &other) :
        TLObject(other.parent()),
        Chat(other.classType()) {
        setData(&other);
    }
    ~TLChat() {}

    void setData(const void *data) {
        Chat *c = (Chat*)data;
        setParticipantsCount(c->participantsCount());
        setId(c->id());
        setVersion(c->version());
        setVenue(c->venue());
        setTitle(c->title());
        setAddress(c->address());
        setDate(c->date());
        setPhoto(c->photo());
        setGeo(c->geo());
        setAccessHash(c->accessHash());
        setCheckedIn(c->checkedIn());
        setLeft(c->left());
        setClassType(c->classType());
    }

    void setPhoto(const ChatPhoto &photo) { m_photo.setData(&photo); }
    void setGeo(const GeoPoint &geo) { m_geo.setData(&geo); }
    TLChatPhoto *photo() { return &m_photo; }
    TLGeoPoint *geo() { return &m_geo; }

private:
    TLChatPhoto m_photo;
    TLGeoPoint m_geo;
};
Q_DECLARE_METATYPE(TLChat)

#endif // TLCHAT_H
