#ifndef TLENCRYPTEDMESSAGE_H
#define TLENCRYPTEDMESSAGE_H

#include "tlobject.h"
#include "tlencryptedfile.h"
#include <types/encryptedmessage.h>

class TLEncryptedMessage : public TLObject, public EncryptedMessage {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLEncryptedMessageType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qint32 date READ date)
    Q_PROPERTY(QByteArray bytes READ bytes)
    Q_PROPERTY(TLEncryptedFile *file READ file)
    Q_PROPERTY(qint64 randomId READ randomId)
    Q_PROPERTY(qint32 chatId READ chatId)

public:
    enum TLEncryptedMessageType {
        TypeEncryptedMessageService = EncryptedMessage::typeEncryptedMessageService,
        TypeEncryptedMessage = EncryptedMessage::typeEncryptedMessage
    };

    explicit TLEncryptedMessage(QObject *parent = 0) : 
        TLObject(parent), 
        EncryptedMessage(EncryptedMessage::typeEncryptedMessage) {
    }
    TLEncryptedMessage(const TLEncryptedMessage &other) :
        TLObject(other.parent()),
        EncryptedMessage(other.classType()) {
        setData(&other);
    }
    ~TLEncryptedMessage() {}

    void setData(const void *data) {
        EncryptedMessage *obj = (EncryptedMessage *)data;
        setClassType(obj->classType());
        setDate(obj->date());
        setBytes(obj->bytes());
        setFile(obj->file());
        setRandomId(obj->randomId());
        setChatId(obj->chatId());
    }

    void setFile(const EncryptedFile &file) { m_file.setData(&file); }
    TLEncryptedFile *file() { return &m_file; }

private:
    TLEncryptedFile m_file;
};
Q_DECLARE_METATYPE(TLEncryptedMessage)

#endif // TLENCRYPTEDMESSAGE_H
