#ifndef TLINPUTFILELOCATION_H
#define TLINPUTFILELOCATION_H

#include "tlobject.h"
#include <types/inputfilelocation.h>

class TLInputFileLocation : public TLObject, public InputFileLocation {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLInputFileLocationType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qint64 accessHash READ accessHash)
    Q_PROPERTY(qint64 secret READ secret)
    Q_PROPERTY(qint32 localId READ localId)
    Q_PROPERTY(qint64 id READ id)
    Q_PROPERTY(qint64 volumeId READ volumeId)

public:
    enum TLInputFileLocationType {
        TypeInputVideoFileLocation = InputFileLocation::typeInputVideoFileLocation,
        TypeInputEncryptedFileLocation = InputFileLocation::typeInputEncryptedFileLocation,
        TypeInputFileLocation = InputFileLocation::typeInputFileLocation,
        TypeInputAudioFileLocation = InputFileLocation::typeInputAudioFileLocation,
        TypeInputDocumentFileLocation = InputFileLocation::typeInputDocumentFileLocation
    };

    explicit TLInputFileLocation(QObject *parent = 0) : 
        TLObject(parent), 
        InputFileLocation(InputFileLocation::typeInputFileLocation) {
    }
    TLInputFileLocation(const TLInputFileLocation &other) :
        TLObject(other.parent()),
        InputFileLocation(other.classType()) {
        setData(&other);
    }
    ~TLInputFileLocation() {}

    void setClassType(qint32 classType) {
        InputFileLocation::setClassType((InputFileLocationType)classType);
    }
    void setData(const void *data) {
        InputFileLocation *obj = (InputFileLocation *)data;
        setClassType(obj->classType());
        setAccessHash(obj->accessHash());
        setSecret(obj->secret());
        setLocalId(obj->localId());
        setId(obj->id());
        setVolumeId(obj->volumeId());
    }
};
Q_DECLARE_METATYPE(TLInputFileLocation)

#endif // TLINPUTFILELOCATION_H
