#ifndef TLMESSAGE_H
#define TLMESSAGE_H

#include "tlobject.h"
#include "tlpeer.h"
#include "tlmessagemedia.h"
#include "tlmessageaction.h"
#include <types/message.h>

class TLMessage : public TLObject, public Message {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLMessageType)
    Q_PROPERTY(qint32 classType READ classType NOTIFY classTypeChanged)
    Q_PROPERTY(qint32 id READ id NOTIFY idChanged)
    Q_PROPERTY(bool unread READ unread NOTIFY unreadChanged)
    Q_PROPERTY(qint32 fromId READ fromId NOTIFY fromIdChanged)
    Q_PROPERTY(bool out READ out NOTIFY outChanged)
    Q_PROPERTY(qint32 date READ date NOTIFY dateChanged)
    Q_PROPERTY(qint32 fwdDate READ fwdDate NOTIFY fwdDateChanged)
    Q_PROPERTY(qint32 fwdFromId READ fwdFromId NOTIFY fwdFromIdChanged)
    Q_PROPERTY(QString message READ message NOTIFY messageChanged)
    Q_PROPERTY(TLPeer* toId READ toId NOTIFY toIdChanged)
    Q_PROPERTY(TLMessageMedia* media READ media NOTIFY mediaChanged)
    Q_PROPERTY(TLMessageAction* action READ action NOTIFY actionChanged)

public:
    enum TLMessageType {
        TypeMessageEmpty = Message::typeMessageEmpty,
        TypeMessageService = Message::typeMessageService,
        TypeMessage = Message::typeMessage,
        TypeMessageForwarded = Message::typeMessageForwarded
    };

    explicit TLMessage(QObject *parent = 0) :
        TLObject(parent),
        Message(Message::typeMessageEmpty) {
    }
    TLMessage(const TLMessage &other) :
        TLObject(other.parent()),
        Message(other.classType()) {
        setData(&other);
    }
    ~TLMessage() {}

    void setData(const void *data) {
        Message *s = (Message *)data;
        setId(s->id());
        setToId(s->toId());
        setUnread(s->unread());
        setAction(s->action());
        setFromId(s->fromId());
        setOut(s->out());
        setDate(s->date());
        setMedia(s->media());
        setFwdDate(s->fwdDate());
        setFwdFromId(s->fwdFromId());
        setMessage(s->message());
        setClassType(s->classType());
    }

    void setToId(const Peer &toId) { m_toId.setData(&toId); }
    void setMedia(const MessageMedia &media) { m_media.setData(&media); }
    void setAction(const MessageAction &action) { m_action.setData(&action); }
    TLPeer *toId() { return &m_toId; }
    TLMessageMedia *media() { return &m_media; }
    TLMessageAction *action() { return &m_action; }

Q_SIGNALS:
    void classTypeChanged(qint32);
    void idChanged(qint32);
    void unreadChanged(bool);
    void fromIdChanged(qint32);
    void outChanged(bool);
    void dateChanged(qint32);
    void fwdDateChanged(qint32);
    void fwdFromIdChanged(qint32);
    void messageChanged(QString);
    void toIdChanged(TLPeer *);
    void mediaChanged(TLMessageMedia *);
    void actionChanged(TLMessageAction *);

private:
    TLPeer m_toId;
    TLMessageMedia m_media;
    TLMessageAction m_action;
};
Q_DECLARE_METATYPE(TLMessage)

#endif // TLMESSAGE_H
