#ifndef TLUPDATE_H
#define TLUPDATE_H

#include "tlobject.h"
#include "tlcontactsmylink.h"
#include "tluserprofilephoto.h"
#include "tlencryptedchat.h"
#include "tlgeochatmessage.h"
#include "tlchatparticipants.h"
#include "tldcoption.h"
#include "tluserstatus.h"
#include "tlnotifypeer.h"
#include "tlmessage.h"
#include "tlcontactsforeignlink.h"
#include "tlpeernotifysettings.h"
#include <types/update.h>
#include <QQmlListProperty>

class TLUpdate : public TLObject, public Update {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLUpdateType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qint32 qts READ qts)
    Q_PROPERTY(TLContactsMyLink *myLink READ myLink)
    Q_PROPERTY(qint32 inviterId READ inviterId)
    Q_PROPERTY(TLUserProfilePhoto *photo READ photo)
    Q_PROPERTY(qint32 userId READ userId)
    Q_PROPERTY(QString device READ device)
    Q_PROPERTY(TLEncryptedChat *chat READ chat)
    Q_PROPERTY(TLGeoChatMessage *geoChatMessage READ geoChatMessage)
    Q_PROPERTY(TLMessage *message READ message)
    Q_PROPERTY(qint32 pts READ pts)
    Q_PROPERTY(qint32 id READ id)
    Q_PROPERTY(bool blocked READ blocked)
    Q_PROPERTY(TLChatParticipants *participants READ participants)
    Q_PROPERTY(qint32 version READ version)
    Q_PROPERTY(QString location READ location)
    Q_PROPERTY(QQmlListProperty<TLDcOption> dcOptions READ dcOptions)
    Q_PROPERTY(qint32 maxDate READ maxDate)
    Q_PROPERTY(bool previous READ previous)
    Q_PROPERTY(TLUserStatus *status READ status)
    Q_PROPERTY(TLNotifyPeer *peer READ peer)
    Q_PROPERTY(TLContactsForeignLink *foreignLink READ foreignLink)
    Q_PROPERTY(qint32 date READ date)
    Q_PROPERTY(qint32 chatId READ chatId)
    Q_PROPERTY(TLPeerNotifySettings *notifySettings READ notifySettings)
    Q_PROPERTY(qint64 randomId READ randomId)
    Q_PROPERTY(QString firstName READ firstName)
    Q_PROPERTY(qint64 authKeyId READ authKeyId)
    Q_PROPERTY(QList<qint32> messages READ messages)
    Q_PROPERTY(QString lastName READ lastName)

public:
    enum TLUpdateType {
        TypeUpdateUserStatus = Update::typeUpdateUserStatus,
        TypeUpdateNotifySettings = Update::typeUpdateNotifySettings,
        TypeUpdateMessageID = Update::typeUpdateMessageID,
        TypeUpdateChatUserTyping = Update::typeUpdateChatUserTyping,
        TypeUpdateActivation = Update::typeUpdateActivation,
        TypeUpdateRestoreMessages = Update::typeUpdateRestoreMessages,
        TypeUpdateEncryption = Update::typeUpdateEncryption,
        TypeUpdateUserName = Update::typeUpdateUserName,
        TypeUpdateUserBlocked = Update::typeUpdateUserBlocked,
        TypeUpdateNewMessage = Update::typeUpdateNewMessage,
        TypeUpdateContactLink = Update::typeUpdateContactLink,
        TypeUpdateChatParticipantDelete = Update::typeUpdateChatParticipantDelete,
        TypeUpdateNewAuthorization = Update::typeUpdateNewAuthorization,
        TypeUpdateChatParticipantAdd = Update::typeUpdateChatParticipantAdd,
        TypeUpdateDcOptions = Update::typeUpdateDcOptions,
        TypeUpdateDeleteMessages = Update::typeUpdateDeleteMessages,
        TypeUpdateUserTyping = Update::typeUpdateUserTyping,
        TypeUpdateEncryptedChatTyping = Update::typeUpdateEncryptedChatTyping,
        TypeUpdateReadMessages = Update::typeUpdateReadMessages,
        TypeUpdateUserPhoto = Update::typeUpdateUserPhoto,
        TypeUpdateContactRegistered = Update::typeUpdateContactRegistered,
        TypeUpdateNewEncryptedMessage = Update::typeUpdateNewEncryptedMessage,
        TypeUpdateEncryptedMessagesRead = Update::typeUpdateEncryptedMessagesRead,
        TypeUpdateChatParticipants = Update::typeUpdateChatParticipants,
        TypeUpdateNewGeoChatMessage = Update::typeUpdateNewGeoChatMessage
    };

    explicit TLUpdate(QObject *parent = 0) : 
        TLObject(parent), 
        Update(Update::typeUpdateUserStatus) {
    }
    TLUpdate(const TLUpdate &other) :
        TLObject(other.parent()),
        Update(other.classType()) {
        setData(&other);
    }
    ~TLUpdate() {}

    void setData(const void *data) {
        Update *obj = (Update *)data;
        setClassType(obj->classType());
        setQts(obj->qts());
        setMyLink(obj->myLink());
        setInviterId(obj->inviterId());
        setPhoto(obj->photo());
        setUserId(obj->userId());
        setDevice(obj->device());
        setChat(obj->chat());
        setMessage(obj->message());
        setPts(obj->pts());
        setId(obj->id());
        setBlocked(obj->blocked());
        setParticipants(obj->participants());
        setVersion(obj->version());
        setLocation(obj->location());
        setMaxDate(obj->maxDate());
        setPrevious(obj->previous());
        setStatus(obj->status());
        setPeer(obj->peer());
        setForeignLink(obj->foreignLink());
        setDate(obj->date());
        setChatId(obj->chatId());
        setNotifySettings(obj->notifySettings());
        setRandomId(obj->randomId());
        setFirstName(obj->firstName());
        setAuthKeyId(obj->authKeyId());
        setMessages(obj->messages());
        setLastName(obj->lastName());
        Q_FOREACH (DcOption s, obj->dcOptions()) {
            TLDcOption *tlDcOption = new TLDcOption(this);
            tlDcOption->setData(&s);
            m_dcOptions.append(tlDcOption);
        }
    }

    void setMyLink(const ContactsMyLink &myLink) { m_myLink.setData(&myLink); }
    void setNotifySettings(const PeerNotifySettings &notifySettings) { m_notifySettings.setData(&notifySettings); }
    void setChat(const EncryptedChat &chat) { m_chat.setData(&chat); }
    void setPhoto(const UserProfilePhoto &photo) { m_photo.setData(&photo); }
    void setParticipants(const ChatParticipants &participants) { m_participants.setData(&participants); }
    void setStatus(const UserStatus &status) { m_status.setData(&status); }
    void setMessage(const Message &message) { m_message.setData(&message); }
    void setGeoChatMessage(const GeoChatMessage &geoChatMessage) { m_geoChatMessage.setData(&geoChatMessage); }
    void setForeignLink(const ContactsForeignLink &foreignLink) { m_foreignLink.setData(&foreignLink); }
    void setPeer(const NotifyPeer &peer) { m_peer.setData(&peer); }
    TLNotifyPeer *peer() { return &m_peer; }
    TLContactsForeignLink *foreignLink() { return &m_foreignLink; }
    TLGeoChatMessage *geoChatMessage() { return &m_geoChatMessage; }
    TLMessage *message() { return &m_message; }
    TLUserStatus *status() { return &m_status; }
    TLChatParticipants *participants() { return &m_participants; }
    TLUserProfilePhoto *photo() { return &m_photo; }
    TLEncryptedChat *chat() { return &m_chat; }
    TLPeerNotifySettings *notifySettings() { return &m_notifySettings; }
    TLContactsMyLink *myLink() { return &m_myLink; }

    QQmlListProperty<TLDcOption> dcOptions() {
        return QQmlListProperty<TLDcOption>(this, m_dcOptions);
    }

private:
    QList<TLDcOption *> m_dcOptions;
    TLContactsMyLink m_myLink;
    TLPeerNotifySettings m_notifySettings;
    TLEncryptedChat m_chat;
    TLUserProfilePhoto m_photo;
    TLChatParticipants m_participants;
    TLUserStatus m_status;
    TLMessage m_message;
    TLGeoChatMessage m_geoChatMessage;
    TLContactsForeignLink m_foreignLink;
    TLNotifyPeer m_peer;
};
Q_DECLARE_METATYPE(TLUpdate)

#endif // TLUPDATE_H
