#ifndef TLUSER_H
#define TLUSER_H

#include <types/types.h>
#include "tlobject.h"
#include "tluserstatus.h"
#include "tluserprofilephoto.h"

class TLUser : public TLObject, public User {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLUserType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(TLUserStatus *status READ status)
    Q_PROPERTY(QString firstName READ firstName)
    Q_PROPERTY(qint64 accessHash READ accessHash)
    Q_PROPERTY(QString lastName READ lastName)
    Q_PROPERTY(QString phone READ phone)
    Q_PROPERTY(bool inactive READ inactive)
    Q_PROPERTY(TLUserProfilePhoto *photo READ photo)
    Q_PROPERTY(qint32 id READ id)
public:
    enum TLUserType {
        TypeUserSelf = User::typeUserSelf,
        TypeUserContact = User::typeUserContact,
        TypeUserDeleted = User::typeUserDeleted,
        TypeUserForeign = User::typeUserForeign,
        TypeUserEmpty = User::typeUserEmpty,
        TypeUserRequest = User::typeUserRequest
    };

    explicit TLUser(QObject *parent = 0) :
        TLObject(parent),
        User(User::typeUserEmpty) {
    }
    TLUser(const TLUser &other) :
        TLObject(other.parent()),
        User(other.classType()) {
        setData(&other);
    }
    ~TLUser() {}

    void setData(const void *data) {
        User *u = (User*)data;
        setId(u->id());
        setAccessHash(u->accessHash());
        setInactive(u->inactive());
        setPhone(u->phone());
        setFirstName(u->firstName());
        setLastName(u->lastName());
        setPhoto(u->photo());
        setStatus(u->status());
        setClassType(u->classType());
    }

    void setPhoto(const UserProfilePhoto &photo) { m_photo.setData(&photo); }
    void setStatus(const UserStatus &status) { m_status.setData(&status); }
    TLUserStatus *status() { return &m_status; }
    TLUserProfilePhoto *photo() { return &m_photo; }

private:
    TLUserStatus m_status;
    TLUserProfilePhoto m_photo;
};
Q_DECLARE_METATYPE(TLUser)

#endif // TLUSER_H
