import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1
import "../components/listitems"
import Ubuntu.Plugins.Telegram 0.1

Page {

    title: i18n.tr('Dialogs');

    ListView {
        id: dialogsListView
        anchors.fill: parent

        model: telegramClient.dialogsModel

        delegate: DlgItem {
            icon: model.thumbnail
            fallbackIconSource: "../assets/nophoto.png"
            iconFrame: true
            text: model.name + " (" + (model.isChat? "Chat" : "User") + ")";
            subText: model.topMessage.fromFirstName + ": " + model.topMessage.text + " (" + dialogsListView.mediaTypeToString(model.topMessage.mediaType) + ") " + (model.typing ? model.whoisTyping + "typing" : "");
            date: dialogsListView.messageUnread(model.topMessage.fromId, model.topMessage.unread) + dialogsListView.formatDate(model.topMessage.date);
            unread: model.unreadCount
        }

        function formatDate(dateAsLong) {
            var d = new Date(dateAsLong * 1000);
            var today = new Date();
            // if date is today, return hour in hh:mm format
            if ((d.getDate() === today.getDate()) &&
                    (d.getTime() > today.getTime() - 86400000) &&
                    (d.getTime() < today.getTime() + 86400000)){
                return Qt.formatTime(d, "hh:mm");
            } else{
                var oneWeekAgo = new Date();
                oneWeekAgo.setTime(today.getTime() - 7 * 86400000);
                // if current date is in the last 6 days, set the day of the week
                if ((d.getTime() > oneWeekAgo.getTime()) && (d.getDate() !== oneWeekAgo.getDate())) {
                    return Qt.formatDate(d, "ddd");
                }
            }
            return Qt.formatDate(d, "MMM d");
        }

        function messageUnread(sender, unread) {
            if (sender === telegramClient.ourId) {
                return unread ? "Sent - " : "Read - ";
            }
            return "";
        }

        function mediaTypeToString(mediaType) {
            if (mediaType === TLMessageMedia.TypeMessageMediaEmpty) {
                return "Text";
            } else if (mediaType === TLMessageMedia.TypeMessageMediaPhoto) {
                return "Photo";
            }
            //TODO complete with the rest of types
            return "Other";
        }
    }

}
