import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Plugins.Telegram 0.1
import "ui"

/*!
    \brief MainView with Tabs element.
           First Tab has a single Label and
           second Tab has a single ToolbarAction.
*/

MainView {
    // objectName for functional testing purposes (autopilot-qt5)
    objectName: "mainView"

    // Note! applicationName needs to match the "name" field of the click manifest
    applicationName: "com.ubuntu.developer.libqtelegram.uitest"

    /*
     This property enables the application to change orientation
     when the device is rotated. The default is false.
    */
    //automaticOrientation: true

    width: units.gu(45)
    height: units.gu(80)

    PageStack {

        id: pageStack

        SplashPage {
            id: splashPage
            visible: false
        }

        SignInPage {
            id: signInPage
            visible: false
        }

        SignUpPage {
            id: signUpPage
            visible: false
        }

        VerificationCodePage {
            id: verificationCodePage
            visible: false
        }

        DialogsPage {
            id: dialogsPage
            visible: false
        }

        DialogPage {
            id: dialogPage
            visible: false
        }


        Component.onCompleted: showPage(signInPage)

        function showPage(page) {
            if (pageStack.depth > 0)
                pageStack.pop();
            pageStack.push(page);
        }
    }

    TelegramClient {
        id: telegramClient
        configPath: "~/.libqtelegram"
        publicKeyFile: "tg.pub"

        onAuthNeeded: authCheckPhone();

        onAuthSignInError: {
            verificationCodePage.errorLabel.visible = true;
            verificationCodePage.errorLabel.text = i18n.tr("The code you have entered is not valid. Try again");
            verificationCodePage.codeTextField.enabled = true;
            verificationCodePage.codeTextField.text = "";
            verificationCodePage.verifyButton.enabled = true;
        }

        onAuthSignUpError: {
            verificationCodePage.errorLabel.visible = true;
            verificationCodePage.errorLabel.text = i18n.tr("The code you have entered is not valid. Try again");
            verificationCodePage.codeTextField.enabled = true;
            verificationCodePage.codeTextField.text = "";
            verificationCodePage.verifyButton.enabled = true;
        }

        onAuthCheckPhoneAnswer: {
            if (phoneRegistered) {
                authSendCode();
            } else {
                pageStack.showPage(signUpPage);
            }
        }

        onAuthSendCodeAnswer: {
            verificationCodePage.countdownTimer.start();
            pageStack.showPage(verificationCodePage);
        }

        onAuthSendCallAnswer: verificationCodePage.countdownLabel.text = i18n.tr("calling user's mobile");

        onAuthLoggedIn: {
            console.log("onAuthLoggedIn");
            messagesGetDialogs();
        }

        onMessagesGetDialogsAnswer: {
            pageStack.showPage(dialogsPage);
        }

    }
}
